/******************************************************************************
 *                                                                            *
 * @file    fjexsock_ioctl.h                                                  *
 * @brief   Extended Socket Linux デバイスドライバ ioctlヘッダ                *
 * @author  葛綿孝智、長毅、小糸啓介                                          *
 *                                                                            *
 * @par History                                                               *
 *   - 2014/08/13  作成                                                       *
 *                                                                            *
 * @par Copyright                                                             *
 *   All Rights Reserved, Copyright (C) Fujitsu Limited, 2014                 *
 *                                                                            *
 * @par License                                                               *
 *   This program is free software; you can redistribute it and/or modify it  *
 *   under the terms and conditions of the GNU General Public License,        *
 *   version 2, as published by the Free Software Foundation.                 *
 *                                                                            *
 *   This program is distributed in the hope it will be useful, but WITHOUT   *
 *   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    *
 *   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    *
 *   for more details.                                                        *
 *                                                                            *
 *   You should have received a copy of the GNU General Public License along  *
 *   with this program; if not, write to the Free Software Foundation, Inc.,  *
 *   51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.                 *
 *                                                                            *
 *   The full GNU General Public License is included in this distribution in  *
 *   the file called "COPYING".                                               *
 *                                                                            *
 ******************************************************************************/
/*
TAB=4 encode=UTF8
//$DD は詳細仕様書へ記載
//$SD は構成仕様書へ記載
 */
 
/**$DD
******************************************************************************
                                                                              
 File Name    : Linux デバイスドライバ詳細仕様書                              
                                                                              
 Function     : Extended Socket Linux デバイスドライバ ioctlヘッダ            
                                                                              
 Author       : CEC)長毅                                                      
                                                                              
 History      : R00   元ソースより分離                                        
                日付  2014/08/13    作成者 : CEC)長                           
                                                                              
                                                                              
 All Rights Reserved, Copyright (C) Fujitsu Limited, 2014.                    
                                                                              
******************************************************************************
 */

#ifndef _EXT_SOCKT_IOCTL_H_
#define _EXT_SOCKT_IOCTL_H_

//$SD ioctlコマンド定数定義
//$SD トレースioctl
//ES_IOCTL_TRACE_START	SIOCDEVPRIVATE + 1
#define ES_IOCTL_TRACE_START  (SIOCDEVPRIVATE+1)
//ES_IOCTL_TRACE_STOP	SIOCDEVPRIVATE + 2
#define ES_IOCTL_TRACE_STOP   (SIOCDEVPRIVATE+2)
//ES_IOCTL_TRACE_GETCFG	SIOCDEVPRIVATE + 3
#define ES_IOCTL_TRACE_GETCFG (SIOCDEVPRIVATE+3)
//$SD レジスタioctl
//ES_IOCTL_ESDEV_GETREG	SIOCDEVPRIVATE + 4
#define ES_IOCTL_ESDEV_GETREG (SIOCDEVPRIVATE+4)
//ES_IOCTL_ESDEV_SETREG	SIOCDEVPRIVATE + 5
#define ES_IOCTL_ESDEV_SETREG (SIOCDEVPRIVATE+5)


//$SD トレース開始要求構造体
struct es_ioctl_trace_start_req_val{
	uint32_t	mode;
		//ESデバイスのアクセストレース対象を指定。
		//  Bit 0 : 割り込み制御レジスタ(0:OFF/1:ON)
		//  Bit 1 : コマンド制御レジスタ(0:OFF/1:ON)
	uint32_t	reserved;
	uint64_t	trace_size;
		//トレースバッファのサイズ(Byte)を指定
		//（4096 Byte単位でのみ指定可能で、指定可能最大サイズは4G Byte）
};

//$SD トレース停止要求構造体
struct es_ioctl_trace_stop_req_val{
	uint64_t	trace_buff_size;
	//トレースデータを受け取るためにユーザ側で用意したバッファ(trace_data_buff)のサイズ	
};
	
//$SD トレース要求共用体
union es_ioctl_trace_req_val{
	struct es_ioctl_trace_start_req_val	start;
	struct es_ioctl_trace_stop_req_val	stop;
};

	
//$SD トレース開始応答構造体
struct es_ioctl_trace_start_res_val{
	uint32_t	err_code;
		//ESデバイス処理の完了コード
		//0x0000 : 正常完了
		//0x0001 : 処理中（BUSY）
		//0x0100 : パラメータ異常
		//0x0200 : トレース開始済み
		//0x0400 : メモリ不足
		//上記以外の値は異常
	uint32_t	reserved;
		//予備
};

//$SD トレース情報取得応答構造体
struct es_ioctl_trace_getcfg_res_val{
	uint32_t	err_code;
		//ESデバイス処理の完了コード
		//  0x0000 : 正常完了
		//  0x0300 : トレース開始前
	uint32_t	mode;
		//開始時に指定されたESデバイスのアクセストレース対象
		//  Bit 0 : 割り込み制御レジスタ(0:OFF/1:ON)
		//  Bit 1 : コマンド制御レジスタ(0:OFF/1:ON)
	uint64_t	trace_size;
		//開始時に指定されたトレースバッファのサイズ
};

//$SD トレース停止応答構造体
struct es_ioctl_trace_stop_res_val{
	uint32_t	err_code;
		//ESデバイス処理の完了コード
		//  0x0000 : 正常完了
		//  0x0001 : 処理中（BUSY）
		//  0x0100 : パラメータ異常
		//  0x0300 : トレース開始前
		//  上記以外の値は異常
	uint32_t	reserve;
		//予備
	uint64_t	written_size;
		//トレース受信用領域に書き込んだトレースデータのサイズ。
	uint16_t	trace_data_buff[];
		//サイズはtrace_buff_size
		//トレース受信用領域。トレースデータはxsocket機能仕様書で定義されているトレースレコードの配列の形で格納される。
};
	
//$SD トレース応答共用体
union es_ioctl_trace_res_val{
	struct es_ioctl_trace_start_res_val		start;
	struct es_ioctl_trace_getcfg_res_val	getcfg;
	struct es_ioctl_trace_stop_res_val		stop;
};
	
//$SD トレースioctl時コマンドパラメタ(要求/応答構造体)
struct es_ioctl_trace_param{
	union es_ioctl_trace_req_val	req;
	union es_ioctl_trace_res_val	res;
};

//$SD レジスタioの構造体
struct es_ioctl_esdev_reg_param{
	uint32_t	offset_addr;
	uint32_t	val;
};

//$SD エラーコード定義など
#define ES_IOCTL_TRACE_START_ERR_NORMAL          (0x0000)
#define ES_IOCTL_TRACE_START_ERR_BUSY            (0x0001)
#define ES_IOCTL_TRACE_START_ERR_PARAM           (0x0100)
#define ES_IOCTL_TRACE_START_ERR_ALREADY_STARTED (0x0200)
#define ES_IOCTL_TRACE_START_ERR_MEMORY          (0x0400)

#define ES_IOCTL_TRACE_STOP_ERR_NORMAL           (0x0000)
#define ES_IOCTL_TRACE_STOP_ERR_BUSY             (0x0001)
#define ES_IOCTL_TRACE_STOP_ERR_BEFORE_START     (0x0300)

#define ES_IOCTL_TRACE_GETCFG_ERR_NORMAL         (0x0000)
#define ES_IOCTL_TRACE_GETCFG_ERR_BEFORE_START   (0x0300)

#endif /* _EXT_SOCKT_IOCTL_H_ */
