/******************************************************************************
 *                                                                            *
 * @file    fjexsock.c                                                        *
 * @brief   Extended Socket Linux デバイスドライバ                            *
 * @author  葛綿孝智、長毅、小糸啓介                                          *
 *                                                                            *
 * @par History                                                               *
 *   - 2014/05/28  作成                                                       *
 *   - 2014/09/22  R01リリース                                                *
 *                 LTUT用初回リリース                                         *
 *   - 2014/10/09  R02リリース                                                *
 *                 LTUT用2回目リリース                                        *
 *                 レビュー指摘対応                                           *
 *   - 2014/10/22  R03リリース                                                *
 *                 CT用リリース                                               *
 *                 R01,02とあわせ以下の不具合を修正済み                       *
 *     * LTUT-007 LTUT-009に同じ                                              *
 *     * LTUT-008 esdrv_open異常終了時におけるリカバリー処理不備              *
 *     * LTUT-009 トレースの領域解放での判定間違い                            *
 *     * LTUT-010 受信ラウンドロビンでのEP ID選定ロジック                     *
 *     * (LTUT-011 試験データ不備)                                            *
 *     * LTUT-012 tx_stallでの送信状態判定間違い                              *
 *     * LTUT-013 tx_stall ワークキュースケジュール回数の不整合               *
 *     * LTUT-014 VLAN 機能フラグの設定ミス                                   *
 *     * LTUT-015 API動作変更によるremove_proc_entryでのエラー                *
 *     * LTUT-016 ACPIデータの実機確認による整合性不備                        *
 *     * LTUT-017 unreg時のメモリ参照エラー                                   *
 *   - 2014/10/24  R04リリース                                                *
 *                 以下の不具合を修正済み                                     *
 *     * CT-005 status表示での自EPID多重表示不備                              *
 *     * CT-008 コマンド要求レジスタでのビット配置ミス                        *
 *                 レビュー指摘などより、以下を改善                           *
 *     * 独自統計に共有解除監視、コマンド各種の統計を追加                     *
 *   - 2014/10/27  R05リリース                                                *
 *                 以下の不具合を修正済み                                     *
 *     * CT-012,CT-013 ログ文字列が古い箇所があった                           *
 *                 レビュー指摘などより、以下を改善                           *
 *     * デバイスリセットの完了待ちとタイムアウト、insmod失敗                 *
 *     * esif_close時の受信バッファ初期化を削除                               *
 *     * esvmm_command_request時のbusy確認タイミング修正                      *
 *   - 2014/10/28  R06リリース                                                *
 *                 レビュー指摘などより、以下を改善                           *
 *     * esif_open時の受信バッファ初期化を削除                                *
 *     * esvmm_command_requestを最新の仕様を確認し、対応しなおし              *
 *   - 2014/10/29  R07リリース                                                *
 *                 以下の不具合を修正済み                                     *
 *     * CT-004 共有していない時、キャリアエラーは計上しない                  *
 *                 レビュー指摘などより、以下を改善                           *
 *     * esif_epstop_waitでの状態チェックの記述順序を変更                     *
 *     * ログ変更追従                                                         *
 *     * 統計の扱いを一部変更                                                 *
 *   - 2014/10/31  R08リリース                                                *
 *                 レビュー指摘などより、以下を改善                           *
 *     * マクロで分岐させている不要処理を一部削除                             *
 *     * Zoneのnoneを定義し、対応する箇所に処理追加(same_zone,update_zone他)  *
 *     * バッファサイズマクロを統一                                           *
 *     * irq_disableを除去                                                    *
 *   - 2014/11/05  R09リリース                                                *
 *                 以下の不具合を修正済み                                     *
 *     * CT-020 トレースのバッファ登録でアドレス計算が正しくなかった          *
 *              合せて送受信バッファをvoid*からu8*に(gccではvoid*でも正常)    *
 *     * CT-015,016 メッセージ出力に改行付与                                  *
 *   - 2014/11/06  R10リリース                                                *
 *                 以下の不具合を修正済み                                     *
 *     * CT-019 lock_txのデッドロックの発生抑止(発生源自体を除去)             *
 *   - 2014/11/10  R11リリース                                                *
 *                 レビュー指摘などより、以下を改善                           *
 *     * 共有なしでZone同じになった際の登録処理時のTxの初期化                 *
 *     * 共有解除完了待ち/完了でZoneが違う所になった際の登録削除処理          *
 *     * Zone変更ログは変化があった場合のみに変更                             *
 *   - 2014/11/11  R12リリース                                                *
 *                 以下の不具合を修正済み                                     *
 *     * CT-029 VLAN addでの統計処理の判定不足                                *
 *     * CT-026 VLAN IDを再初期化時に保持するように変更                       *
 *                 レビュー指摘などより、以下を改善                           *
 *     * トレースのバッファコピーを先頭から実施するように修正                 *
 *   - 2014/11/12  R13リリース                                                *
 *                 レビュー指摘などより、以下を改善                           *
 *     * 機能仕様書の変更にともなうログの変更強化                             *
 *     * 送受信バッファを確保時に一度仮で初期化する                           *
 *                 以下の不具合を修正済み                                     *
 *     * CT-023 不要な統計の計上があった                                      *
 *   - 2014/11/14  R14リリース                                                *
 *                 以下の不具合を修正済み                                     *
 *     * CT-025 関連して0除算チェックを追加                                   *
 *                 レビュー指摘などより、以下を改善                           *
 *     * 共有解除完了待ち/完了でZoneが違う所になった際の登録削除処理の統計    *
 *   - 2014/11/14  R15リリース                                                *
 *                 レビュー指摘などより、以下を改善                           *
 *     * 停止要求時の強制解除処理実装と、共有監視のタイムアウトリカバリー     *
 *     * device disableはなくし、not foundのみでINFOレベル出力とする          *
 *       Rel 1 リリース                                                       *
 *     * 性能調整とCT-030の検討課題対策としてtx_stallにusleep_rangeを追加     *
 *   - 2014/11/19  R16リリース                                                *
 *     * Tx stallのタイムアウト処理追加                                       *
 *     * メモリリージョンをモジュール名で設定                                 *
 *     * netif_carrier_on/offを追加                                           *
 *     * デバイスから各種待ち時間を受け、設定値へ展開                         *
 *     * softlockup対応としてmdelayをmsleepにし、                             *
 *       合せてZone更新をワークキューに変換(内容は変化なし)                   *
 *     * unknown_drop時の統計上昇でEPID指定だったのを自EPIDに                 *
 *   - 2014/11/26  R17リリース                                                *
 *     * ST-002関連 unknown_drop判定のため、MACアドレス完全チェック           *
 *     * ST-001 MACアドレスをモジュール初期化時に設定                         *
 *     * ST-011関連 停止要求処理タスクレットをワークキューへ                  *
 *   - 2014/12/02  R17.1リリース                                              *
 *     * ST-015 表示桁を改善                                                  *
 *   - 2014/12/10  R18リリース                                                *
 *                 レビュー指摘などより、以下を改善                           *
 *     * ST-014 バッファの登録削除でのリトライとclose時の削除失敗でのリセット *
 *     * 機能仕様書レビューによりログメッセージ変更                           *
 *   - 2014/12/15  R19リリース                                                *
 *                 レビュー指摘などより、以下を改善                           *
 *     * ST-017 送受信停止におけるフラグ制御追加                              *
 *     * ST-018 送信のwatchdogタイムアウトを10jifから20secへ拡大              *
 *     * ST-018 ワークキューを独自に作成し、送受信と制御の2つへ振り分け       *
 *     * msleepに使う数値は20以上になるよう調整                               *
 *   - 2014/12/16  R20リリース                                                *
 *                 レビュー指摘などより、以下を改善                           *
 *     * ST-011 spin_lockは廃止し、バッファの排他のみmutexで排他              *
 *   - 2014/12/16  R21リリース                                                *
 *                 レビュー指摘などより、以下を改善                           *
 *     * ST-017関係 受信要求割り込みの低減のためrx_statusへフラグ追加         *
 *   - 2014/12/16  R21.1リリース                                              *
 *     * module_exitのunregister_netdevからクローズする場合があるため順序変更 *
 *   - 2014/12/16  R22リリース                                                *
 *                 レビュー指摘などより、以下を改善                           *
 *     * リセット、タイムアウトにおけるエラーログ強化                         *
 *     * オープン時のES情報更新通知を3回に修正                                *
 *     * バッファ登録削除の時間延長およびEPによるシフト時間付与               *
 *   - 2014/12/21  R23リリース                                                *
 *                 レビュー指摘などより、以下を改善                           *
 *     * 応答バッファの長さチェックとそのエラーログ強化                       *
 *     * タイムアウト時間の調整                                               *
 *     * 割り込み解除とNAPI disableのタイミング、                             *
 *         および開始時のステータス読み捨て                                   *
 *   - 2014/12/29  R24リリース                                                *
 *                 ST障害の対策検討の結果、以下の対応を実施                   *
 *     * ST-007,018対応 割り込みの遅延監視ワークキューの実装                  *
 *         合せて、NAPIでの完了ディレイとその際のフラグ制御改善               *
 *   - 2015/01/07  R25リリース                                                *
 *                 レビュー指摘などより、以下を改善                           *
 *     * オープン時のステータス読み捨てタイミング調整                         *
 *     * IOエラー時の強制リセットクローズ処理の追加およびリセット失敗対策     *
 *   - 2015/01/08  R25.1リリース                                              *
 *     * レビュー指摘:esif_closeでの強制リセット設定時の統計                  *
 *   - 2015/01/08  R25.2リリース                                              *
 *     * indent -linuxで整形                                                  *
 *   - 2015/01/15  R25.3リリース                                              *
 *     * ログにjiffiesの10進整数値出力追加(マクロ制御、当座有効)              *
 *   - 2015/01/21  R26リリース                                                *
 *     * リンクアップ時にES情報更新割り込みを各EP毎に３回生成する処理を削除   *
 *       （割り込み周期監視を追加したことで割り込み抜けががなくなったため）   *
 *     * リンクアップ時にES情報更新割り込みを発生させた後の安定化待ちのための *
 *       待ち時間を1000ms x 8から300ms × 8に変更                             *
 *       （リンクアップ処理の時間短縮のため）                                 *
 *     * デバイスリセット時のDCTLレジスタのTry & Checkの周期を1000msに変更    *
 *       （デバイスアクセス負荷の軽減のため）                                 *
 *     * コマンド要求時のCSレジスタのTry & Checkの周期を1000msに変更          *
 *       （デバイスアクセス負荷の軽減のため）                                 *
 *     * ログにjiffy値を出力するマクロを無効化                                *
 *   - 2015/01/22  R27リリース                                                *
 *     * MTU変更でOpen/Closeしない構造に変更                                  *
 *     * あわせてMTU変更中フラグを用意し、tx_stallおよび対向same_mtuがチェック*
 *     * バッファ初期化時にrx_statusを保持。また各種バッファ登録箇所で0設定   *
 *   - 2015/01/23  R27.1リリース(未)                                          *
 *     * MTU変更中フラグの論理を反転、そのため、27.0以前とは通信できません    *
 *     * バッファ初期化時にrx_statusに初期値の設定も追加                      *
 *   - 2015/01/23  R27.2リリース                                              *
 *     * 待ち時間について、レビュー指摘を含めて再検討、延長を実施             *
 *   - 2015/01/23  R27.3リリース                                              *
 *     *  MTUチェックで対向受信側のチェック対応、emptyでフラグチェック        *
 *   - 2015/01/24  R28リリース                                                *
 *     *  MTU変更でcarrier on/off追加                                         *
 *   - 2015/01/25  R29リリース                                                *
 *     *  R27.1での論理反転により、バッファ初期化時のrx_status保持は不要だった*
 *   - 2015/02/03  R30リリース                                                *
 *     *  rx_statusの排他制御(spinlock)を追加、ST-023再オープンに対応         *
 *   - 2015/02/23  R31リリース                                                *
 *     *  内部用ログ機能を実装                                                *
 *   - 2015/02/24  R32リリース                                                *
 *     *  内部用ログ機能を拡充(レビュー対応)                                  *
 *        + ログサイズを4MByte(16 * 1024 msg * 256char)へ拡大                 *
 *        + ログ出力時のTxRx Infoダンプ追加                                   *
 *        + ログの書式統一(パースのため:をメッセージに入れない、=にする)      *
 *        + ログ出力強化                                                      *
 *   - 2015/03/18  R32.1リリース                                              *
 *     *  内部用ログのアドレス情報出力関係を強化(物理アドレス、最初のページ)  *
 *   - 2015/04/09  R33リリース                                                *
 *     *  feature flagからHW_CSUMを除去                                       *
 *                                                                            *
 * @par Copyright                                                             *
 *   All Rights Reserved, Copyright (C) Fujitsu Limited, 2014                 *
 *                                                                            *
 * @par License                                                               *
 *   This program is free software; you can redistribute it and/or modify it  *
 *   under the terms and conditions of the GNU General Public License,        *
 *   version 2, as published by the Free Software Foundation.                 *
 *                                                                            *
 *   This program is distributed in the hope it will be useful, but WITHOUT   *
 *   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    *
 *   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    *
 *   for more details.                                                        *
 *                                                                            *
 *   You should have received a copy of the GNU General Public License along  *
 *   with this program; if not, write to the Free Software Foundation, Inc.,  *
 *   51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.                 *
 *                                                                            *
 *   The full GNU General Public License is included in this distribution in  *
 *   the file called "COPYING".                                               *
 *                                                                            *
 ******************************************************************************/

/*
TAB=8 encode=UTF8
//$DD は詳細仕様書へ記載
//$SD は構成仕様書へ記載
*/

/**$DD
******************************************************************************
                                                                              
 File Name    : Linux デバイスドライバ詳細仕様書                              
                                                                              
 Function     : Extended Socket Linux デバイスドライバ 関数                   
                                                                              
 Author       : CEC)葛綿孝智                                                  
                                                                              
 History      : R00   詳細仕様書作成                                          
                日付  2014/05/28    作成者 : CEC)葛綿                         
                                                                              
                R01   全面改訂                                                
                日付  2014/06/25    作成者 : CEC)葛綿・長                     
                                                                              
                R02   ソース'14/8/26版(r147)を反映                            
                日付  2014/08/26    作成者 : CEC)葛綿・長                     
                                                                              
                R03   8EP対応改訂                                             
                日付  2014/09/04    作成者 : CEC)葛綿・長                     
                                                                              
                R04   レビュー指摘反映 (r227と同期)                           
                日付  2014/09/16    作成者 : CEC)長                           
                                                                              
                R05   レビュー指摘反映 (r233と同期)                           
                      関数名正規化、別名定義削除、エラーコード明記            
                      呼び出し関数明記                                        
                日付  2014/09/17    作成者 : CEC)長                           
                                                                              
                R06   レビュー指摘反映 (r264と同期)                           
                      統計項目明記、古い引数説明除去                          
                      esdrv_netdev_setup 初期化値の明記、説明明確化           
                      esdrv_open/esdrv_stop シーケンスの説明追加              
                      esdrv_start_xmit 送信対象EP明記                         
                      esdrv_rx_irq 古い不要な記述があったので削除             
                      esdrv_rx_napi_poll 処理ループとbudgetの関係明確化       
                      esdrv_vlan_rx_add_vid/esdrv_vlan_rx_kill_vid VID範囲変更
                      esdrv_do_ioctl サブ関数に分割                           
                日付  2014/09/26    作成者 : CEC)長                           
                                                                              
                R07   仕様の変更に追従 (r390と同期)                           
                      初期化関係および終了関係のシーケンス見直しによる順序変更
                       特にesdrv_openにおけるエラー処理                       
                      VLANの最新の仕様に追従した実装                          
                      各種統計の強化と取得箇所改善                            
                      送信での割り込み通知、バッファフルリカバリのロックレス化
                      統計(32bit)を欠番                                       
                      トレース停止での処理単純化                              
                      IRQの無効化フラグの処理削除                             
                      esif_rxframe_search_existなどの処理整理、共有してない時 
                       のRxバッファの操作を削除(Rxの初期化も削除)             
                      コマンドのエラー処理                                    
                      esif_closeでの共有の最終解除                            
                      esif_epstop_raise、esif_epstop_waitの明確化と定義       
                      esif_txrx_stop_done_irqは欠番                           
                      esif_txrx_stop_req_irq、esif_dev_stop_req_irq、         
                       esif_update_zone_irq、esif_epstop_tasklet、            
                       esif_update_zone_tasklet、esif_watch_unshareの仕様明記 
                      esif_epstop_process_doneの名称変更と参照先修正(Rx->Tx)  
                      esmem_epbuf_sizeは欠番                                  
                      ACPI関係の実装方法を最新の情報および実機での確認に合わせ
                      署名系機能の削除                                        
                      esvmm_command_requestの実行方法をデバイスの仕様書に追従 
                      esvmm_automask_setを欠番に                              
                      Rxでの0除算チェック                                     
                      共有監視のタイムアウト処理強化                          
                日付  2014/11/14    作成者 : CEC)長                           
                                                                              
                R08   仕様の変更に追従 (r426と同期)                           
                      MACアドレスの初期化をモジュールロード時に               
                      carrier_off/onも行なうようにし、送受信の処理抑制        
                      Zone更新処理、共有中止処理をワークキュー化              
                      esdrv_tx_stallでのwatchdog回避のため時間測定とスリープ  
                      状態安定時間を3*maxepへ変更、他、コマンドおよびリセット 
                       の時間もデバイス側展開の時間に正式化                   
                      close時の削除失敗における強制リセットとレジスタ再初期化 
                      バッファの登録削除でのリトライ処理                      
                日付  2014/12/10    作成者 : CEC)長                           
                                                                              
                R09   仕様の変更に追従 (r457と同期)                           
                      独自キュー作成と対応に伴う関数変更                      
                      強制クローズのワークキュー、割り込み周期監視ワークキュー
                       の追加による対応                                       
                       (esdrv_irq_status_work,esdrv_force_close_work)         
                      オープン時にオープン禁止処理、割り込みステータス読み捨て
                       追加                                                   
                      クローズ時の割り込みの削除時の手順調整                  
                      送受信スピンロック削除                                  
                      NAPIポーリングでのステータス設定と完了状態とマージン時間
                       の調整方法を整理、また割り込み生成側での確認処理追加   
                      デバイスへのコマンドの異常でのエラー処理として、強制    
                       クローズとリセットを追加、また応答バッファ長チェック   
                      統計の追加(napi_polled_empty,device_reset_executed,     
                       device_force_reset_scheduled)                          
                      割り込み周期遅延ワークキューを割り込み登録/削除時に設定 
                日付  2015/01/09    作成者 : CEC)長                           
                                                                              
                R10   仕様の変更に追従 (r501と同期)                           
                      タイムアウト時間を拡張                                  
                      状態フラグにMTU変更完了を追加(on)、またMTU変更中は削除  
                      あわせてrx_statusの排他処理追加                         
                      OpenでのZone変更リトライなどを仕様改訂に合せ変更        
                      ログにjiffies出力マクロフラグ追加                       
                      内部ログ格納、表示機能追加                              
                日付  2015/03/03    作成者 : CEC)長                           
                                                                              
 All Rights Reserved, Copyright (C) Fujitsu Limited, 2014.                    
                                                                              
******************************************************************************

	用語定義
	epbuf : EPバッファ
		EP間のリングバッファ。１つで１方向。送信・受信には２つ必要。
	送受信リング : EPバッファの別表記として利用。
	frmbuf: フレームバッファ
		EPバッファから１フレーム分の通信に使用される領域

 １． モジュール構造

 <モジュール構造図>

  (凡例)
  参照元(呼出し元)
  ↓
  参照先(呼出し先)



    ┌────────────────────┐
    │1.ネットワークドライバ  カーネル部      │
    └────────────────────┘
            ↓※1           ↑
    ┌────────────────────┐    ┐
    │2.ドライバインターフェイス              │    │ES ドライバ本体
    │シンボル名: esdrv_...                   │    │Linuxデバイスドライバ
    └────────────────────┘    │
            ↓※2                                   │
    ┌────────────────────┐    │
    │3.ES インターフェイス                   │    │
    │シンボル名: esif_...                    │    │
    └────────────────────┘    │
            │              ↓                      │
            │      ┌────────────┐    │
            │      │4.メモリ管理            │    │
            │      │シンボル名: esmem_...   │    │
            │      └────────────┘    │
            ↓              ↓                      │
    ┌────────────────────┐    │
    │5.ES デバイス(VMM)インターフェイス      │    │
    │シンボル名: esvmm_...                   │    │
    └────────────────────┘    ┘
  
※1 カーネルからネットワークインターフェイスへはハンドラを通じて呼出しされる
※2 直下の層を飛び越えて、メモリ管理、ESデバイス(VMM)インターフェイスを呼び出す場合もある

  1.ネットワークドライバ  カーネル部
    Linuxカーネルのネットワークドライバ機能

  2.ドライバインターフェイス
    Linuxネットワークドライバとして必要な機能を実装する。
    構成要素は以下の通り
      モジュールのエントリ(Linuxコマンド insmod,rmmod より呼び出される)
      ネットワークドライバのハンドラ(カーネルから呼び出される)
      ESデバイスのハンドラ(受信割込み。ESデバイスインターフェイスから呼び出され、カーネルに通知する)

  3.ESインターフェイス
    Extended Socketの動作仕様に強く依存する部分。ネットワークドライバとは直接関係しない部分について管理する。
    主な機能は以下の通り
      ESデバイスの管理
      EP間のバッファ共有操作の制御
      ゾーンの管理
      送受信処理の制御
      EPの状態管理
    
  4.メモリ管理
    ESデバイス操作に必要なバッファ類の生成/破棄、送受信バッファの生成/破棄とそのバッファでのフレームの送受信操作などを提供する。

  5. ESデバイス(VMM)インターフェイス
    ACPIでの検出、メモリアクセスでのハードウェア制御の処理集合。


**/

#include <linux/module.h>
#include <linux/init.h>
#include <linux/moduleparam.h>

#include <linux/sched.h>
#include <linux/kernel.h>	/* printk() */
#include <linux/slab.h>		/* kmalloc() */
#include <linux/errno.h>	/* error codes */
#include <linux/types.h>	/* size_t */
#include <linux/interrupt.h>	/* mark_bh */
#include <linux/delay.h>	/* msleep() */

#include <linux/bitops.h>
// 3.14くらいから定義されているGENMASKの後方互換実装(移植)
#ifndef GENMASK
#define GENMASK(h, l)           (((U32_C(1) << ((h) - (l) + 1)) - 1) << (l))
#endif
#ifndef GENMASK_ULL
#define GENMASK_ULL(h, l)       (((U64_C(1) << ((h) - (l) + 1)) - 1) << (l))
#endif

#include <linux/in.h>
#include <linux/netdevice.h>	/* struct device, and other headers */
#include <linux/etherdevice.h>	/* eth_type_trans */
#include <linux/ip.h>		/* struct iphdr */
#include <linux/tcp.h>		/* struct tcphdr */
#include <linux/skbuff.h>
#include <linux/string.h>
#include <linux/if_ether.h>
#include <linux/if_vlan.h>	/* vlan_get_tag,etc */
#include <linux/proc_fs.h>	/* proc_mkdir_data */
#include <linux/nls.h>		/* wchar_t */

#include <linux/acpi.h>		/* ACPI */

//#include "fjexsock.h" //当面、本ファイルに統合
// ioctlはユーザアプリも参照する&固定的なので、分離
#include "fjexsock_ioctl.h"	/* ioctl構造体 */

#include <linux/in6.h>
#include <asm/checksum.h>

//-----------------------------------------------------------------------------
//デバッグ表示マクロ

//デバッグ表示のカテゴリ別  有効・無効設定
//表示する項目を0以外(1)に設定する

#define PD_ALL	0		//すべてのデバッグメッセージを表示

//関数の入り口
#define PD_ENT		0	// 関数入口(Enter)
//関数の挙動
#define PD_INF		0	// 全体で出力すべき情報(Info)
#define PD_COMMON	0	// 基本有効にせず、各種テストのPD_LTUTxxxのor表記にしていくこと PD_COMMON -> PD_COMMON||PD_LTUTxx1 -> PD_COMMON||PD_LTUTxx1||PD_LTUTxx2

// 以下機能毎の確認用
#define PD_START_XMIT	0
#define PD_DEV_STOP	0
#define PD_ACPI		0
#define PD_VLAN		0
#define PD_MTU		0
#define PD_STATUS	0
#define PD_BUFFER_REG	0

//マクロ関数
//#define PDEBUG_LOG

#undef PDEBUG
#undef PDEBUG_LIMIT
#undef PDEBUG_DEV
#undef PDEBUG_XDEV
#if defined(ESDRV_DEBUG) || defined(PDEBUG_LOG)
	//デバッグ表示有効
	//condに表示カテゴリ・レベル等を設定し、容易に表示・非表示を切り替えられるようにする
#define PDEBUG(cond, fmt, args...)  { if(PD_ALL||cond) printk(KERN_DEBUG "ESDRV: %s(): " fmt "\n", __func__, ## args); }
#define PDEBUG_LIMIT(cond, fmt, args...)  { if(PD_ALL||cond) printk_ratelimited(KERN_DEBUG "ESDRV: %s(): " fmt "\n", __func__, ## args); }
	//デバイス(ESデバイス/テストデバイス)の表示を追加
		//struct net_device *devが定義されていると
#define PDEBUG_DEV(cond, fmt, args...)  { if(PD_ALL||cond) printk(KERN_DEBUG "ESDRV(%d): %s(): " fmt "\n", dev!=esdrv_device, __func__, ## args); }
		//struct esif_device *xdevが定義されていると
#define PDEBUG_XDEV(cond, fmt, args...)  { if(PD_ALL||cond) printk(KERN_DEBUG "ESDRV(%d): %s(): " fmt "\n", xdev->container_dev!=esdrv_device, __func__, ## args); }
#else
	//デバッグ表示無効
#define PDEBUG(cond, fmt, args...)
#define PDEBUG_LIMIT(cond, fmt, args...)
#define PDEBUG_DEV(cond, fmt, args...)
#define PDEBUG_XDEV(cond, fmt, args...)
#endif

//分岐等のチェックポイント
#undef CHKPOINT
//#define CHKPOINT() printk(KERN_DEBUG "CHECKPOINT:%u %s()\n", __LINE__, __func__)
//#define CHKPOINT() printk(KERN_DEBUG "CHECKPOINT: %s:%u %s()\n", __FILE__, __LINE__, __func__)        //ファイル名も表示
#define CHKPOINT()		//チェックポイント無効化

//-----------------------------------------------------------------------------
//マクロ  シンボル

//コード有効・無効設定

//#define ESVMM_MMIO_LOG

// Control Register Access Macro
#ifndef ESVMM_MMIO_LOG
#define esvmm_writel(value, address) ( writel(value, address) )
#define esvmm_readl(address) ( readl(address) )
#else
unsigned int esvmm_dbg_value;
#define esvmm_writel(value, address) \
   ( printk("xsct:writel (value:0x%08X addr:0x%016lX offset:0x%04X)\n", (unsigned int )value, (unsigned long int )address, 0) , writel(value, address) )
#define esvmm_readl(address) \
   ( printk("xsct:readl  (value:0x%08X addr:0x%016lX offset:0x%04X)\n", esvmm_dbg_value = (unsigned int )readl(address), (unsigned long int )address, 0), esvmm_dbg_value  )
#endif

#define NUM_QUEUE 1		// mq(マルチキュー)対応無し

//#define ESDRV_VLAN_OFF
//#define ESDRV_ZONE_OFF
#define ESDRV_ETHERTOOL_OFF

//#define FUNC_STATIC           //関数のstatic宣言無効化 
#define FUNC_STATIC static	//関数のstatic宣言

//#define FUNC_INLINE           //関数のinline宣言無効化 
#define FUNC_INLINE inline	//関数のinline宣言

#define FUNC_STATIC_INLINE FUNC_STATIC FUNC_INLINE

//設定
#define DEVICE_NAME	"es%d"	//デバイス名

//-----------------------------------------------------------------------------
// デバイスパラメータ

// @note
// デバッグなどで変数が必要な場合
// static xxx g_xx と変数を用意し
// module_param_named(name, g_xx, xxx, 0);
// としてモジュール変数を公開できる

//-----------------------------------------------------------------------------
// 型定義(エイリアス)1

typedef int epno_t;		//EP数/EPIDの型(0オリジン EP#4(EPID0)=0, 0未満=無効／エラー等)
typedef void *framebuf_t;	//フレームバッファ(sk_buffのデータ領域)
typedef void *memptr_t;		//汎用ポインタ(できるだけ意味のある型に移行すること)

//$SD 値の定義は enum INTREG_IRQMASK, enum INTREG_MASK
typedef __u32 intreg_t;		//各割り込み制御レジスタの値

//-----------------------------------------------------------------------------
// 型定義(事前名前参照と代用定義)2

//$DD レジスタ構造体の参照(以前は構造体/共用体だったが、なくなったのでu8 *とするため)
typedef u8 es_device_register;	//デバイスレジスタ(esvmm)

struct es_device_command_param;
struct ep_share_mem_info;
union es_device_command_req;
union es_device_command_res;
struct es_device_shared_info;

//-----------------------------------------------------------------------------
// 定数・変数・型宣言

//$SD メッセージ表示の文字列定数とバージョン定義

#define _ES_DRV_TO_STRING(_a) #_a
#define ES_DRV_TO_STRING(_a) _ES_DRV_TO_STRING(_a)

//$SD バージョン定義
#define ES_DRV_MAJOR_NO    0
#define ES_DRV_MINOR_NO    33
#define ES_DRV_REVISION_NO 0
#define ES_DRV_VERSION_STRING ES_DRV_TO_STRING(ES_DRV_MAJOR_NO) "." ES_DRV_TO_STRING(ES_DRV_MINOR_NO) "." ES_DRV_TO_STRING(ES_DRV_REVISION_NO)

//$SD モジュール名
#ifdef KBUILD_MODNAME
#define ES_DRV_MODULE_NAME   KBUILD_MODNAME
#else				/* KBUILD_MODNAME */
#define ES_DRV_MODULE_NAME   "fjexsock"
#endif				/* KBUILD_MODNAME */
#define ES_DRV_MODULE_AUTHOR "FUJITSU Ltd."
#define ES_DRV_MODULE_DESC   "FUJITSU Extended Socket Network Device Driver"

// ログへのjiffies追加
//#define ES_DRV_MSG_INCLUDE_JIFFIES

#if defined(ES_DRV_MSG_INCLUDE_JIFFIES)
#define ES_DRV_MSG(a) ES_DRV_MODULE_NAME ": (%ld) " a "\n"
#define ES_DRV_MSG_TIME ((long)jiffies)
#else /* ES_DRV_MSG_INCLUDE_JIFFIES */
#define ES_DRV_MSG(a) ES_DRV_MODULE_NAME ": %s" a "\n"
#define ES_DRV_MSG_TIME ""
#endif /* ES_DRV_MSG_INCLUDE_JIFFIES */

//$SD ログメッセージ文字列定義
#define ES_DRV_MSG_INTRO       ES_DRV_MSG(ES_DRV_MODULE_DESC " version " ES_DRV_VERSION_STRING)
#define ES_DRV_MSG_COPYRIGHT   ES_DRV_MSG("Copyright (c) 2014 " ES_DRV_MODULE_AUTHOR)
#define ES_DRV_MSG_LINKUP      ES_DRV_MSG("FUJITSU Extended Socket linked up.")
#define ES_DRV_MSG_LINKDOWN    ES_DRV_MSG("FUJITSU Extended Socket link downed.")
#define ES_DRV_MSG_RMMOD       ES_DRV_MSG(ES_DRV_MODULE_DESC " is removed.")
#define ES_DRV_MSG_MEMFAIL     "Insufficient Memory. "
#define ES_DRV_MSG_MEMFAIL1    ES_DRV_MSG(ES_DRV_MSG_MEMFAIL "(%s)")
#define ES_DRV_MSG_MEMFAIL2    ES_DRV_MSG(ES_DRV_MSG_MEMFAIL "(%s%d)")
#define ES_DRV_MSG_MEMFAIL1_NETDEV        "net_device"	// : ネットデバイス構造体
#define ES_DRV_MSG_MEMFAIL1_SHM_STATUS    "shm_status"	// : 共有ステータス領域
#define ES_DRV_MSG_MEMFAIL1_DEV_REQ       "es_dev_req"	// : ESデバイス要求データ領域
#define ES_DRV_MSG_MEMFAIL1_DEV_RES       "es_dev_res"	// : ESデバイス応答データ領域
#define ES_DRV_MSG_MEMFAIL1_SHM_INFO      "shm_info"	// :共有領域を管理するための領域
#define ES_DRV_MSG_MEMFAIL2_SHM_TO_EPx    "shm_to_EP"	// x : EP#xに見せる共有領域（送信バッファ）
#define ES_DRV_MSG_MEMFAIL2_SHM_FROM_EPx  "shm_from_EP"	// x : EP#xを見る共有領域（受信バッファ）
// #define ES_DRV_MSG_DEV_DISABLE  ES_DRV_MSG("FUJITSU Extended Socket is disabled.") // 使用しない
#define ES_DRV_MSG_DEV_NOTFOUND ES_DRV_MSG("FUJITSU Extended Socket device is not found.")
#define ES_DRV_MSG_INSMOD_FAIL  ES_DRV_MSG("Can not add " ES_DRV_MODULE_DESC ".")
#define ES_DRV_MSG_EP_SYSDOWN1  ES_DRV_MSG("Disconnected from the target. (%d)")
#define ES_DRV_MSG_ZONE_CHANGE  ES_DRV_MSG("Zone is changed.")
#define ES_DRV_MSG_NOTREADY_ERR1    ES_DRV_MSG("Device is not ready. (%d)")
// 用法が確定しないため、無効にしておく
//#define ES_DRV_MSG_INTERNAL_ERR1    ES_DRV_MSG("Internal Error. (%d)")

//$SD Internal Error/Device is not readyの識別番号 必ず、項目を単調増加で追加する(同根でも別箇所なら新しい項目を作ること)
enum {
	ES_DRV_INTERNAL_ERR_ADDR_SETUP = 0,
	ES_DRV_INTERNAL_ERR_RESET_NG1 = 1,
	ES_DRV_INTERNAL_ERR_NETDEV_REG = 2,
	ES_DRV_INTERNAL_ERR_ESIF_OPEN = 3,
	ES_DRV_INTERNAL_ERR_IRQ_INIT = 4,
	ES_DRV_INTERNAL_ERR_RX_0DIV = 5,
	ES_DRV_INTERNAL_ERR_RESET_NG2 = 6,
	ES_DRV_INTERNAL_ERR_RESET_NG3 = 7,
	ES_DRV_INTERNAL_ERR_LENGTH1 = 8,
	ES_DRV_INTERNAL_ERR_LENGTH2 = 9,
	ES_DRV_INTERNAL_ERR_LENGTH3 = 10,
	ES_DRV_INTERNAL_ERR_LENGTH4 = 11,
	ES_DRV_INTERNAL_ERR_LENGTH5 = 12,
	ES_DRV_INTERNAL_ERR_CMD_BUSY1 = 13,
	ES_DRV_INTERNAL_ERR_CMD_BUSY2 = 14,
	ES_DRV_INTERNAL_ERR_CMD_TIMEO1 = 15,
	ES_DRV_INTERNAL_ERR_CMD_TIMEO2 = 16,
	ES_DRV_INTERNAL_ERR_CMD_TIMEO3 = 17,
	ES_DRV_INTERNAL_ERR_CMD_TIMEO4 = 18,
	ES_DRV_INTERNAL_ERR_CMD_TIMEO5 = 19,
	ES_DRV_INTERNAL_ERR_OPEN_GUARD = 20,
	ES_DRV_INTERNAL_ERR_FORCE_CLOSE = 21,
};

//$SD procファイルシステムパス定義  
#define ES_DRV_PROC_CONCAT(_a,_b)	_a "/" _b
#define ES_DRV_PROC_ROOT	ES_DRV_MODULE_NAME
#define ES_DRV_PROC_INFO	ES_DRV_PROC_CONCAT(ES_DRV_PROC_ROOT,"info")
#define ES_DRV_PROC_STATUS	ES_DRV_PROC_CONCAT(ES_DRV_PROC_ROOT,"status")
#define ES_DRV_PROC_STATS	ES_DRV_PROC_CONCAT(ES_DRV_PROC_ROOT,"statistics")
#define ES_DRV_PROC_LOG		ES_DRV_PROC_CONCAT(ES_DRV_PROC_ROOT,"log")

// デバイスへのアクセス時間は安全係数*3、EP数*8として拡張すること
// デバイスリセットタイムアウト デバイスからの計測と8EP机上試算から17秒
#define ES_DEVICE_RESET_TIMEOUT_UNIT_MSEC (1000)
#define ES_DEVICE_RESET_TIMEOUT_MAX_SEC   ((17 + 1) * 3 * 8)	// (設定値17esc + 補助1sec) * 安全係数3 * EP数8
#define ES_DEVICE_RESET_TIMEOUT_MAX_MSEC  (ES_DEVICE_RESET_TIMEOUT_MAX_SEC * 1000)

// コマンド制御タイムアウト デバイスからの計測と8EP机上試算から(登録 5秒、削除3秒) 設定は5+1秒としておく
// -> 他と合せて安全係数*3としておく
#define ES_COMMAND_REQ_TIMEOUT_UNIT_MSEC (1000)
#define ES_COMMAND_REQ_TIMEOUT_MAX_SEC   ((5 + 1) * 3 * 8) // (設定値5esc + 補助1sec) * 安全係数3 * EP数8
#define ES_COMMAND_REQ_TIMEOUT_MAX_MSEC  (ES_COMMAND_REQ_TIMEOUT_MAX_SEC * 1000)

// 送受信バッファの登録削除でのタイムアウト時間など
// 最大待ち時間算出 処理時間(登録/削除) (5s + 3s) * 安全係数 3 = (8 * 3)
// あるEPでのリトライ時間の調整 最低単位時間 + (自EPID * 20) / 20msecはmsleepでの最小時間
// このリトライ単位時間の差は最大同時衝突数 4組(max/2) * 衝突処理時間3-4msを考慮した単位時間になっている必要がある -> 20msecの差で大丈夫
// -> レビューから延長の話もあり、1min=60secとする
#define ES_COMMAND_REQ_BUFF_TIMEOUT_MAX_SEC   (60 * 8) // ((設定値 + 補助) * 安全係数3) = 60 * EP数8
#define ES_COMMAND_REQ_BUFF_TIMEOUT_MAX_MSEC  (ES_COMMAND_REQ_BUFF_TIMEOUT_MAX_SEC * 1000)
#define ES_COMMAND_REQ_BUFF_TIMEOUT_UNIT_MSEC (200)
#define ES_COMMAND_REQ_BUFF_TIMEOUT_EP_UNIT_MSEC(_ep) (ES_COMMAND_REQ_BUFF_TIMEOUT_UNIT_MSEC + ((_ep) * 20))

// コマンド制御でのエラー状態
#define ES_COMMAND_REQ_ERR_INFO_PARAM  (0x001)
#define ES_COMMAND_REQ_ERR_INFO_STATUS (0x002)

// コマンドの応答バッファでのエラーコード
#define ES_COMMAND_REQ_RES_CODE_NORMAL (0)
#define ES_COMMAND_REQ_RES_CODE_BUSY   (1)

// 通信管理、Head/Tail初期値
#define EP_RING_INITIAL_HEAD (0)
#define EP_RING_INITIAL_TAIL (1)

// ES管理情報 ES設定
#define ES_ZONING_STATUS_DISABLE (0x00)
#define ES_ZONING_STATUS_ENABLE  (0x01)
#define ES_ZONING_STATUS_INVALID (0xFF)

// ES管理情報 Zone設定
#define ES_ZONING_ZONE_TYPE_NONE (0xFF)

// 通信管理 tx_statusで利用する通知切り離し状態
#define ES_TX_DELAY_SEND_NONE    (0)
#define ES_TX_DELAY_SEND_PENDING (1)

// 送信バッファフルでの再送チェックループ数
#define ES_TX_STALL_CHECK_MAX    (5)

// 送信リトライのインターバル(jiffies)
#define ES_TX_RETRY_INTERVAL     (msecs_to_jiffies(20 * 1000))
// 送信リトライのタイムアウト(jiffies)
#define ES_TX_RETRY_TIMEOUT      (100)
// 送信ストールチェックのタイムアウト(インターバルの1/2)(jiffies)
#define ES_TX_TX_STALL_TIMEOUT   (ES_TX_RETRY_INTERVAL/2)

// 割り込みの遅延監視ワークキュー ディレイ 1sec
#define ES_DRV_IRQ_WATCH_DELAY   (msecs_to_jiffies(1000))

// 通信管理 rx_statusで利用する共有解除処理完了通知
#define ES_RX_STOP_REQ_NONE      (0x0)
#define ES_RX_STOP_REQ_DONE      (0x1)
#define ES_RX_STOP_REQ_REQUEST   (0x2)
#define ES_RX_POLL_WORK          (0x4)
#define ES_RX_MTU_CHANGING_DONE  (0x8)

// 受信のNAPI状態維持のタイムアウト(jiffies)
#define ES_RX_NAPI_MARGIN        (3)

// NAPIでのbudget値
#define ES_DRV_NAPI_POLLING_BUDGET (64)

// 共有解除状態を監視する待ち
// ifconfig down(close)時
#define ES_COMMAND_EPSTOP_WAIT_UNIT_MSEC (100)
#define ES_COMMAND_EPSTOP_WAIT_MAX_MSEC  (1000)
// 共有解除割り込み時 デバイスからの計測と8EP机上試算から(共有バッファ削除)3秒
#define ES_UNSHARE_WATCH_WAIT_UNIT_MSEC  (100)
#define ES_UNSHARE_WATCH_WAIT_MAX_MSEC   (3000)

// Open時のリトライ数とWait時間
#define ES_DRV_OPEN_ZONE_UPDATE_WAIT_MSEC (300)

// traceのステータスフラグ
#define ESDRV_TRACE_CYCLIC_MASK (0x01)

// ロギングの設定パラメタ
#define ES_DRV_LOG_LINESIZE (256) // ログバッファの1行のサイズ(NULL終端分確保)
#define ES_DRV_LOG_MSG_MAX  ((4 * 1024 * 1024)/ES_DRV_LOG_LINESIZE)   // メッセージのmax数(4M/1line)
#define ES_DRV_LOG_MSG_PAGE_NUM ((ES_DRV_LOG_MSG_MAX * ES_DRV_LOG_LINESIZE) / PAGE_SIZE) // ログバッファの確保ページ数(native PAGE_SIZE)
#define ES_DRV_LOG_MSG_LEN  (ES_DRV_LOG_LINESIZE - 1)   // ログ1行での文字長(最後の安全確保用NULL分なしの長さ)

// ACPI関連
#define ES_DEVICE_ACPI_DEVICE   "PNP0C02"
#define ES_DEVICE_ACPI_STRING   "_STR"
#define ES_DEVICE_ACPI_SYMBOL   "Extended Socket"
#define ES_DEVICE_ACPI_STATUS   "_STA"
#define ES_DEVICE_ACPI_STATUS_ES_ENABLE (0x0F)
#define ES_DEVICE_ACPI_RESOURCE "_CRS"

#define ES_DEVICE_MEM_REGION_NAME       ES_DRV_MODULE_NAME

	//$SD ESデバイスとのやり取りの基準サイズとしての、最小ページサイズ定義(4K)       
#define MINIMUM_PAGE_SIZE (4096)	///< x86-64での最小ページサイズ4Kbyteを基準サイズにする

/// 仮想レジスタのサイズ
/// ---------------------+
/// 情報レジスタ         |
/// デバイス制御レジスタ |  4Kbyte
/// コマンド制御レジスタ |
/// 割り込み制御レジスタ |
/// ---------------------+
#define ES_DEVICE_REGISTER_SIZE (0x1000)

//$SD ESデバイスレジスタ構造体群
//$SD 情報レジスタ構造体
//$SD Offset 0x0000 OwnerEPID
typedef union _INFO0 {
	struct {
		__le32 epid:16;	///<自EPのEPID
		 __le32:16;	///<予約領域
	} Bits;
	__le32 Reg;
} INFO0_REG;

//$SD Offset 0x0004 最大EP数
typedef union _INFO1 {
	struct {
		__le32 maxep:16;	///<最大EP数
		 __le32:16;	///<予約領域
	} Bits;
	__le32 Reg;
} INFO1_REG;

//$SD デバイス制御レジスタ
//$SD Offset 0x0010
typedef union DCTL {
	struct {
		__le32 reset:1;	///< bit0 Device Reset
		__le32 _rsv_1_15:15;

		__le32 _rsv_16_31:16;
	} Bits;
	__le32 Reg;
} DCTL_REG;

//$SD コマンド要求
//$SD Offset 0x0020
typedef union CR {
	struct {
		__le32 req_code:16;	///< bit15-0 Request Code
		__le32 err_info:14;	///< bit16-29 Error Information
		__le32 error:1;	///< bit30 Error
		__le32 req_start:1;	///< bit31 Request Start
	} Bits;
	__le32 Reg;
} CR_REG;

//$SD コマンドステータス
//$SD Offset 0x0024
typedef union CS {
	struct {
		__le32 req_code:16;	///< bit15-0 Request Code
		__le32 _rsv_16_29:14;
		__le32 busy:1;	///< bit30 Busy
		__le32 complete:1;	///< bit31 Complete
	} Bits;
	__le32 Reg;
} CS_REG;

//$SD 共有ステータス領域アドレス  Low
//$SD Offset 0x0028
typedef union SHSTSAL {
	struct {
		__le32 low;
	} Bits;
	__le32 Reg;
} SHSTSAL_REG;

//$SD 共有ステータス領域アドレス  High
//$SD Offset 0x002C
typedef union SHSTSAH {
	struct {
		__le32 high;
	} Bits;
	__le32 Reg;
} SHSTSAH_REG;

//$SD 要求データ領域長
//$SD Offset 0x0034
typedef union REQBL {
	struct {
		__le32 len;
	} Bits;
	__le32 Reg;
} REQBL_REG;

//$SD 要求データ領域アドレスLow
//$SD Offset 0x0038
typedef union REQBAL {
	struct {
		__le32 low;
	} Bits;
	__le32 Reg;
} REQBAL_REG;

//$SD 要求データ領域アドレスHigh
//$SD Offset 0x003C
typedef union REQBAH {
	struct {
		__le32 high;
	} Bits;
	__le32 Reg;
} REQBAH_REG;

//$SD 応答データ領域長
//$SD Offset 0x0044
typedef union RESPBL {
	struct {
		__le32 len;
	} Bits;
	__le32 Reg;
} RESPBL_REG;

//$SD 応答データ領域アドレス Low
//$SD Offset 0x0048
typedef union RESPBAL {
	struct {
		__le32 low;
	} Bits;
	__le32 Reg;
} RESPBAL_REG;

//$SD 応答データ領域アドレス High
//$SD Offset 0x004C
typedef union RESPBAH {
	struct {
		__le32 high;
	} Bits;
	__le32 Reg;
} RESPBAH_REG;

//$SD 割り込み制御レジスタ
//$SD 割り込みステータス
//$SD Offset 0x0080
intreg_t IS;

//$SD 割り込みマスク設定
//$SD Offset 0x0084
intreg_t IMS;

//$SD 割り込みマスク解除
//$SD Offset 0x0088
intreg_t IMC;

//$SD 割り込み生成
//$SD Offset 0x008C
intreg_t IG;

//$SD 割り込み制御
//$SD Offset 0x0090
typedef union ICTL {
	struct {
		__le32 automask:1;	///< bit0 Automask
		__le32 _rsv_1_31:31;
	} Bits;
	__le32 Reg;
} ICTL_REG;

#if 0				/* 現状不要になった */
//$SD バージョン通知
//$SD Offset 0x00A0
typedef union VERNTFY {
	struct {
		__le32 notify:1;	///< bit0 Version Notify(Signing Reset)
		__le32 _rsv_1_31:31;
	} Bits;
	__le32 Reg;
} VERNTFY_REG;

//$SD バージョンシーケンス
//$SD Offset 0x00A4
typedef union VERSEQ {
	struct {
		__le32 seq;
	} Bits;
	__le32 Reg;
} VERSEQ_REG;

//$SD バージョンアドレス Low
//$SD Offset 0x00A8
typedef union VERAL {
	struct {
		__le32 low;
	} Bits;
	__le32 Reg;
} VERAL_REG;

//$SD バージョンアドレス High
//$SD Offset 0x00AC
typedef union VERAH {
	struct {
		__le32 high;
	} Bits;
	__le32 Reg;
} VERAH_REG;
#endif				/* 0 */

// レジスタ定義はVMM TPより借用
//
//  Extended device ( xsocket device ) Control Register
//

//$SD レジスタオフセット定数
//情報レジスタ  
#define XSCT_REG_OWNER_EPID     (0x0000)	//  INFO[0]  Owner EPID  自EPのEPID
#define XSCT_REG_MAX_EP         (0x0004)	//  INFO[1]  Maximum EP  最大EP数

//デバイス制御レジスタ
#define XSCT_REG_DCTL           (0x0010)	//  DCTL  Device Control  デバイス制御レジスタ

//コマンド制御レジスタ
#define XSCT_REG_CR             (0x0020)	//  CR Command request   コマンド要求
#define XSCT_REG_CS             (0x0024)	//  CS Command status    コマンドステータス
#define XSCT_REG_SHSTSAL        (0x0028)	//  SHSTSAL Share status address low      共有ステータス領域アドレス Low
#define XSCT_REG_SHSTSAH        (0x002C)	//  SHSTSAH Share status address High     共有ステータス領域アドレス High

#define XSCT_REG_REQBL          (0x0034)	//  REQBL   Request Buffer length         要求データ領域長
#define XSCT_REG_REQBAL         (0x0038)	//  REQBAL  Request Buffer Address Low    要求データ領域アドレス Low
#define XSCT_REG_REQBAH         (0x003C)	//  REQBAH  Request Buffer Address High   要求データ領域アドレス High

#define XSCT_REG_RESPBL         (0x0044)	//  RESPBL  Response Buffer length        応答データ領域長
#define XSCT_REG_RESPBAL        (0x0048)	//  RESPBAL Response Buffer address Low   応答データ領域アドレス Low
#define XSCT_REG_RESPBAH        (0x004C)	//  RESPBAH Response Buffer address High  応答データ領域アドレス High

//割り込み制御レジスタ
#define XSCT_REG_IS             (0x0080)	//  IS    Interrupt status      割り込みステータス
#define XSCT_REG_IMS            (0x0084)	//  IMS   Interrupt mask set   割り込みマスク設定
#define XSCT_REG_IMC            (0x0088)	//  IMC   Interrupt mask clear  割り込みマスク解除
#define XSCT_REG_IG             (0x008C)	//  IG    Interrupt generator  割り込み生成
#define XSCT_REG_ICTL           (0x0090)	//  ICTL  Interrupt control    割り込み制御

#if 0				/* 現状不要になった */
//バージョン番号設定レジスタ
#define XSCT_REG_VERNTFY        (0x00A0)	//  VERNTFY Version Notify
#define XSCT_REG_VERSEQ         (0x00A4)	//  VERSEQ  Version Sequence
#define XSCT_REG_VERAL          (0x00A8)	//  VERAL   Version address low
#define XSCT_REG_VERAH          (0x00AC)	//  VERAH   Version address high
#endif				/* 0 */

//$SD 割り込みレジスタintreg_tのビットマスク定義       []内はデバイス機能仕様書、ドライバ機能仕様書での表記
//$SD メモ：「受信中止」の内容はEP間の共有を停止する動作なので、「共有停止」に変更した。「停止要求」は何を停止するのか明確でないので「デバイス停止」とした。
enum INTREG_IRQMASK {
	INTREG_IRQMASK_INFO_UPDATE = 1 << 20,	///< [ES情報更新割り込み]       Zone情報更新時の割り込み 現在未使用 
	INTREG_IRQMASK_DEV_STOP_REQ = 1 << 19,	///< [停止要求割り込み]         デバイス停止割り込み(デバイスの電源OFF/リセットによりデバイスが発行する)
	INTREG_IRQMASK_TXRX_STOP_REQ = 1 << 18,	///< [受信中止要求割り込み]   送受信停止要求割り込み(共有を停止する為に他EPの自EP向け送受信を止める要求をする)
	INTREG_IRQMASK_TXRX_STOP_DONE = 1 << 17,	///< [受信中止完了割り込み]   送受信停止完了割り込み(共有を停止する為に送受信が停止したことを要求元に伝える)
	INTREG_IRQMASK_RX_DATA = 1 << 16,	///< [受信割り込み]               (送信完了し受信できる状態になったら発生)
	INTREG_IRQMASK_ALL = GENMASK(20, 16),	///< すべての割り込み要因マスク(bit16～20が1)
};
//$SD 割り込みレジスタintreg_tのビットマスク定義
enum INTREG_MASK {
	INTREG_MASK_IS_ASSERT = 1 << 31,	//割り込みアサ―ト(割り込みステータス isでのみ有効)のビットマスク
	INTREG_MASK_EPID = GENMASK(15, 0),	///< EPIDフィールドのマスク(bit0～15が1) このフィールドはintreg_is,intreg_igでのみ有効
};

//$SD 共有ステータス領域
//$SD 構造体を確保する側はEP x 1byte(+sizeof(struct es_device_shared_info))で動的確保すること
//$SD また、struct es_device_command_paramのshare_startにはstatusのアドレスを入れること(サイズ要素は不要)
struct es_device_shared_info {
	epno_t epnum;		///< 最大EP数のはずだが、確保数を記録できるように & 構造体がゼロ長配列だけだとエラーするのでその回避
	u8 ep_status[];		///< 最大EPx1byteの配列、このアドレスをstruct es_device_command_paramのshare_startに設定
};

//$SD コマンド制御要求データの構造体
//後ろでサイズ定義 ES_DEV_REQ_BUF_SIZE()を行う
union es_device_command_req {
	struct {
#define ES_DEV_COMMAND_INFO_REQ_LEN (4)
		__le32 length;
	} info;			//$SD xscoket機能仕様書の0001(H):ES情報通知で利用する要求データの構造を定義した内部構造体
	struct {
#define ES_DEV_COMMAND_SHARE_BUFFER_REQ_LEN(_tx_byte,_rx_byte) \
 (24 + (8 * ((_tx_byte)/MINIMUM_PAGE_SIZE + (_rx_byte)/MINIMUM_PAGE_SIZE)))
		__le32 length;
		__le32 epid;
		__le64 buffer[];
	} share_buffer;		//$SD xscoket機能仕様書の0002(H):共有バッファ登録で利用する要求データの構造を定義した内部構造体
	struct {
#define ES_DEV_COMMAND_UNSHARE_BUFFER_REQ_LEN (8)
		__le32 length;
		__le32 epid;
	} unshare_buffer;	//$SD xscoket機能仕様書の0004(H):共有バッファ削除で利用する要求データの構造を定義した内部構造体
	struct {
#define ES_DEV_COMMAND_START_TRACE_REQ_LEN(_byte) (16 + (8 * (_byte)/MINIMUM_PAGE_SIZE))
		__le32 length;
		__le32 mode;
		__le64 buffer_len;
		__le64 buffer[];
	} start_trace;		//$SD xscoket機能仕様書の0100(H):トレース開始で利用する要求データの構造を定義した内部構造体
	struct {
#define ES_DEV_COMMAND_STOP_TRACE_REQ_LEN (4)
		__le32 length;
	} stop_trace;		//$SD xscoket機能仕様書の0200(H):トレース終了で利用する要求データの構造を定義した内部構造体
};

//$SD コマンド制御応答データの構造体
//後ろでサイズ定義 ES_DEV_RES_BUF_SIZE() を行う
union es_device_command_res {
	struct {
#define ES_DEV_COMMAND_INFO_RES_LEN(_epnum) (8 + 2*(_epnum))
		__le32 length;
		__le32 code;
		struct {
			u8 es_status;	// ESデバイスの設定有効無効
			u8 zone;	// ESデバイスのZone情報
		} info[];	/* max ep だけの配列としてデータを受けとる */
	} info;			//$SD xscoket機能仕様書の0001(H):ES情報通知で利用する応答データの構造を定義した内部構造体
	struct {
#define ES_DEV_COMMAND_SHARE_BUFFER_RES_LEN (8)
		__le32 length;
		__le32 code;
	} share_buffer;		//$SD xscoket機能仕様書の0002(H):共有バッファ登録で利用する応答データの構造を定義した内部構造体
	struct {
#define ES_DEV_COMMAND_UNSHARE_BUFFER_RES_LEN (8)
		__le32 length;
		__le32 code;
	} unshare_buffer;	//$SD xscoket機能仕様書の0004(H):共有バッファ削除で利用する応答データの構造を定義した内部構造体
	struct {
#define ES_DEV_COMMAND_START_TRACE_RES_LEN (8)
		__le32 length;
		__le32 code;
	} start_trace;		//$SD xscoket機能仕様書の0100(H):トレース開始で利用する応答データの構造を定義した内部構造体
	struct {
#define ES_DEV_COMMAND_STOP_TRACE_RES_LEN (8)
		__le32 length;
		__le32 code;
	} stop_trace;		//$SD xscoket機能仕様書の0200(H):トレース終了で利用する応答データの構造を定義した内部構造体
};

//$SD コマンド制御の要求種別
enum es_dev_command_request_type {
	ES_CMD_TYPE_INFO = 0x0001,	//$SD 0001(H):ES情報通知
	ES_CMD_TYPE_SHARE_BUFFER = 0x0002,	//$SD 0002(H):共有バッファ登録
	ES_CMD_TYPE_UNSHARE_BUFFER = 0x0004,	//$SD 0004(H):共有バッファ削除
	ES_CMD_TYPE_START_TRACE = 0x0100,	//$SD 0100(H):トレース開始
	ES_CMD_TYPE_STOP_TRACE = 0x0200,	//$SD 0200(H):トレース終了
};

//$SD コマンド制御で使うパラメタのセット
//$SD 要求長とアドレス、応答長とアドレス、共有ステータス領域アドレス
struct es_device_command_param {
	u32 req_len;		//$SD 要求データのバッファ長
	phys_addr_t req_start;	//$SD 要求データのバッファ物理アドレス

	u32 res_len;		//$SD 応答データのバッファ長
	phys_addr_t res_start;	//$SD 応答データのバッファ物理アドレス

	phys_addr_t share_start;	//$SD 共有データのバッファ物理アドレス
};				// コマンドパラメタ(esvmm)

///$SD コマンド制御の応答エラーコード
enum es_dev_command_response_e {
	ES_CMD_STATUS_UNKNOWN,	//$SD 不明なエラー
	ES_CMD_STATUS_NORMAL,	//$SD 正常
	ES_CMD_STATUS_TIMEOUT,	//$SD タイムアウト
	ES_CMD_STATUS_ERROR_PARAM,	//$SD パラメタエラー
	ES_CMD_STATUS_ERROR_STATUS,	//$SD 状態エラー
};

//$SD 送受信バッファに格納されるフレーム構造
//$SD 先頭4byteがサイズ、その後ろにetherフレームが入るバッファとなっている。
//$SD 利用にはこの型のポインタ変数にフレームのポインタをキャストして使う
//$SD sizeは直接参照、フレームはframe_data としてバッファのアドレスを獲得できる
struct esmem_frame_t {
	__le32 frame_size;	//$SD バッファに格納しているデータ長
	u8 frame_data[];	//$SD バッファ
};

/// フレーム構造とMTUの関係
/// |-- Frame size                    --|
/// |size|-- Buffer size(data)        --|
///      |header|vlan|-- MTU size --|FCS|
/// MTUから、1バッファのサイズを逆算(ETH_HLEN,VLAN_HLEN,FCS_LENを追加)
#define ES_MTU_TO_BUFFER_SIZE(_mtu) (ETH_HLEN + VLAN_HLEN + (_mtu) + ETH_FCS_LEN)
/// MTUから、1フレームサイズを逆算(バッファに変換後、sizeのサイズと合算)
#define ES_MTU_TO_FRAME_SIZE(_mtu) (sizeof(struct esmem_frame_t) + ES_MTU_TO_BUFFER_SIZE(_mtu))

/// MTU定義のためのフレーム構造サイズからMTU算出(sizeのサイズを減算後、ETH_HLEN,VLAN_HLEN,FCS_LENセット分を減算)
#define ES_MTU_DEFINE(_size) ((_size) - sizeof(struct esmem_frame_t) - (ETH_HLEN + VLAN_HLEN + ETH_FCS_LEN))

//$SD MTUリスト
static const u32 ep_support_mtu[] = {
	ES_MTU_DEFINE(8 * 1024),	//$SD   8K / MTU  8166 ( 8192 - 4 - (14 + 4 + 4))
	ES_MTU_DEFINE(16 * 1024),	//$SD  16K / MTU 16358 (16384 - 4 - (14 + 4 + 4))
	ES_MTU_DEFINE(32 * 1024),	//$SD  32K / MTU 32742 (32768 - 4 - (14 + 4 + 4))
	ES_MTU_DEFINE(64 * 1024),	//$SD  64K / MTU 65510 (65536 - 4 - (14 + 4 + 4))
	0			///< 番兵
};

//$SD 送受信での共有バッファ
//$SD バージョンでの定義を実施
union ep_buffer_info {
	u8 raw[MINIMUM_PAGE_SIZE];	// このデータは常にサイズ4096として処理できるようにする

	struct _ep_buffer_info_common_t {
		u32 version;	///<  バージョンを参照する共通ヘッダ
	} common;		//$SD バージョン番号を参照

	struct _ep_buffer_info_v1_t {
		u32 version;	///<  このデータでは1を設定、処理を固有にするなら分岐させること
		u32 info_size;	///<  このテーブルサイズを設定(基本、固定で4096になるが、ページサイズ基準にするかもしれない)

		u32 buffer_size;	///<  送受信バッファのサイズ EP_BUFFER_SIZE - sizeof(ep_buffer_info)になる
		u16 count_max;	///<  バッファの登録フレームサイズ(= ring index max)

		u16 _rsv_1;	///<  (rsv1)

		u32 frame_max;	///<  送受信のフレームのサイズ
		u8 mac_addr[ETH_ALEN];	///<  自分のMAC アドレス

		u16 _rsv_2;	///<  (rsv2)
		u32 _rsv_3;	///<  (rsv3)

		u16 tx_status;
		u16 rx_status;

		// リング制御、送信関数および受信関数のみが操作すること
		u32 head;	///<  リングのHead
		u32 tail;	///<  リングのTail

#define EP_BUFFER_SUPPORT_VLAN_MAX (4)
		u16 vlan_id[EP_BUFFER_SUPPORT_VLAN_MAX];	///<  VLAN ID 1～4

		// (rsv4)は残りで制御しているので処置なし
	} v1i;			//$SD バージョン1での定義(現在の定義)

	// 構造が変化したら、versionだけ同じ構造体を追加定義すること、変数名はv?iで

};

/// 送受信バッファのサイズ、算出式は機能仕様書参照
#define EP_BUFFER_SIZE (( (sizeof(union ep_buffer_info) + (128 * (64 * 1024) )) / MINIMUM_PAGE_SIZE) * MINIMUM_PAGE_SIZE)

/// ringの個数算出
#define EP_RING_NUM(_buffer_size,_frame_size) (u32)((_buffer_size) / (_frame_size))
/// 位置正規化 0 - (max - 1)
#define EP_RING_INDEX(_num,_max) (((_num) + (_max)) % (_max))

/// tailが回って、headまでくる tail == head (差が0) はフル
#define EP_RING_FULL(_head,_tail,_max)   (0 == EP_RING_INDEX(( (_tail) - (_head) ),(_max)))
/// tail > headとしての状態が 1差であるなら、空
#define EP_RING_EMPTY(_head,_tail,_max)  (1 == EP_RING_INDEX(( (_tail) - (_head) ),(_max)))

/// _numに副作用があるのはNG、単一の文として記述すること
#define EP_RING_INDEX_INC(_num,_max) do{(_num) = EP_RING_INDEX((_num) + 1,(_max));}while(0)

/**$SD
 リングバッファの動作説明

 初期状態(バッファ空)

 +0 +---+  head = 0
    |   |
 +1 +---+  tail = 1
    |   |
 +2 +---+
    :   :
 +x +---+

  空の条件は
  1 == ((tail - head ) % max)

 フレーム追加

 +0 +---+  head = 0
    |...|
 +1 +---+
    |   |
 +2 +---+  tail = 2
    :   :
 +x +---+

  tailが+1される
  データは更新前のtailがデータ末端となる位置(アドレス先頭はtail - 1)
  送信側はアドレスとしてtail-1を開始値として最大でheadまで書き込み可能

 フレーム取得

 +0 +---+
    |   |
 +1 +---+  head = 1
    |   |
 +2 +---+  tail = 2
    :   :
 +x +---+

  headが+1される
  データは更新前のheadがデータ先端となる位置(アドレス先頭はhead)
  受信側はアドレスとしてheadを開始値として最大でtail-1まで読み込み可能

 バッファフル

 +0 +---+
    |   |
 +1 +---+  head = 1,tail = 1
    |...|
 +2 +---+
    :...:
    :...:
 +x +---+

  フルの条件は
  0 == ((tail - head ) % max)
  つまり tail == head

  入るフレームはmax - 1

**/

// マクロ展開の都合でここで定義   
#define ES_DEV_REQ_BUF_SIZE(_maxep)     ES_DEV_COMMAND_SHARE_BUFFER_REQ_LEN(EP_BUFFER_SIZE,EP_BUFFER_SIZE)
#define ES_DEV_RES_BUF_SIZE(_maxep)     ES_DEV_COMMAND_INFO_RES_LEN(_maxep)

//$SD ESドライバ 各EP統計構造体
struct es_driver_ep_stats {
	u64 command_register_buffer_executed;	// バッファ共有登録回数                           バッファ共有登録を実行した際に1カウント。
	u64 command_unregister_buffer_executed;	// バッファ共有解除回数                           バッファ共有解除を実行した際に1カウント。
	u64 send_interrupts_rx;	// 受信要求割り込み送信数                         通信相手EPに受信要求割り込み要求を出した際に1カウント。
	u64 send_interrupts_unshare;	// 切断要求割り込み送信数                         通信相手EPに切断要求割り込み要求を出した際に1カウント。
	u64 send_interrupts_stop;	// 停止要求割り込み送信数                         通信相手EPに停止要求割り込み要求を出した際に1カウント。
	u64 send_interrupts_zoneupdate;	// ES情報更新割り込み送信数                       通信相手EPにES情報更新割り込み要求を出した際に1カウント。
	u64 receive_interrupts_rx;	// 受信要求割り込み受信数                         受信要求割り込みを受けて生成元が通信相手EPの場合に1カウント。
	u64 receive_interrupts_unshare;	// 切断要求割り込み受信数                         切断要求割り込みを受けて生成元が通信相手EPの場合に1カウント。
	u64 receive_interrupts_stop;	// 停止要求割り込み受信数                         停止要求割り込みを受けて生成元が通信相手EPの場合に1カウント。
	u64 receive_interrupts_zoneupdate;	// ES情報更新割り込み受信数                       ES情報更新り込みを受けて生成元が通信相手EPの場合に1カウント。
	u64 tx_buffer_full;	// 送信バッファフル発生回数                       バッファフルが発生してプロトコル層のキューを止めた際に1カウント。
	u64 tx_dropped_not_shared;	// 非共有中EPへの送信で破棄したフレーム数         送信の際に通信相手EPと共有中状態ではないためにフレームを破棄した際に1カウント。
	u64 tx_dropped_version_mismatch;	// ESドライババージョン不一致で破棄したフレーム数 送信の際に通信相手EPとESドライバのバージョンが一致せずにフレームを破棄した際に1カウント。
	u64 tx_dropped_buffer_size_discrepancy;	// バッファサイズ不一致で破棄したフレーム数       送信の際に通信相手EPと送受信バッファのサイズが一致せずにフレームを破棄した際に1カウント。
	u64 tx_dropped_vlan_id_mismatch;	// タグVLANのID不一致で破棄したフレーム数         送信の際に通信相手EPに付与されているタグVLANのIDの中に送信しようとしているフレームのタグVLANのIDとい一致するものが存在せずフレームを破棄した際に1カウント。
};

//$SD ESドライバ全体統計構造体
struct es_driver_stats {
	u64 open;		// オープン回数                                     オープンが成功した際に１カウントする。
	u64 close;		// クローズ回数                                     クローズが成功した際に１カウントする。
	u64 mtu_changed;	// MTU設定回数                                      MTU設定変更が成功した際に1カウントする。
	u64 vlan_added;		// VLAN設定回数                                     VLAN ID設定が成功した際に1カウントする。
	u64 vlan_killed;	// VLAN削除回数                                     VLAN ID削除が成功した際に1カウントする。
	u64 napi_scheduled;	// New API起動回数                                  全EPのNew APIポーリングスケジュールした際に1カウント。
	u64 napi_polled;	// New APIポーリング実行回数                        全EPの「New APIポーリング実行回数」を合計する。
	u64 napi_polled_empty;	// New APIポーリングでパケット0だった実行回数       全EPの「New APIポーリング実行回数(パケットが0)」を合計する。
#if !defined(ESDRV_ETHERTOOL_OFF)
	u64 napi_poll_maxpackets;	// (NAPIで処理した最大パケット数:ethertool用に拡張)
#endif				/* !defined(ESDRV_ETHERTOOL_OFF) */
	u64 device_reset_executed;	// リセット実施回数                                 デバイスのリセットを実行した際に1カウント。
	u64 device_force_reset_scheduled;	// 強制リセットスケジュール回数                     デバイスのリセットで強制リセットをスケジュールした際に1カウント。
	u64 command_register_buffer_executed;	// バッファ共有登録回数                             バッファ共有登録を実行した際に1カウント。
	u64 command_unregister_buffer_executed;	// バッファ共有解除回数                             バッファ共有解除を実行した際に1カウント。
	u64 command_request_info_executed;	// ES情報通知回数                                   ES情報通知を実行した際に1カウント。
	u64 command_start_trace_executed;	// トレース開始回数                                 トレース開始を実行した際に1カウント。
	u64 command_stop_trace_executed;	// トレース終了回数                                 トレース終了を実行した際に1カウント。
	u64 interrupts_workqueue_scheduled;	// 受信割り込み生成用ワークキューの起動回数         受信割り込み生成用ワークキューをスケジュールした際に1カウント。
	u64 tx_stall_workqueue_scheduled;	// 送信キュー再開用ワークキューの起動回数           送信キュー再開用ワークキューをスケジュールした際に1カウント。
	u64 unshare_workqueue_scheduled;	// バッファ共有解除処理用ワークキューの起動回数     バッファ共有解除処理用ワークキューをスケジュールした際に1カウント。
	u64 zoneupdate_workqueue_scheduled;	// ES情報更新処理用ワークキューの起動回数           ES情報更新処理用ワークキューの起動回数をスケジュールした際に1カウント。
	u64 unshare_watch_workqueue_scheduled;	// 共有解除監視ワークキューの起動回数               共有解除監視ワークキューをスケジュールした際に1カウント。
	u64 tx_dropped_unknown_destination;	// 該当する送り先無しで破棄したフレーム数           送信の際に該当する送り先のEPが存在せずにフレームを破棄した際に1カウント。
	u64 send_interrupts_rx;	// 受信要求割り込み送信総数                         全EPの「受信要求割り込み送信数」を合計する。
	u64 send_interrupts_unshare;	// 切断要求割り込み送信総数                         全EPの「切断要求割り込み送信数」を合計する。
	u64 send_interrupts_stop;	// 停止要求割り込み送信総数                         全EPの「停止要求割り込み送信数」を合計する。
	u64 send_interrupts_zoneupdate;	// ES情報更新割り込み送信数                         通信相手EPにES情報更新割り込み要求を出した際に1カウント。
	u64 receive_interrupts_rx;	// 受信要求割り込み受信総数                         全EPの「受信要求割り込み受信数」を合計する。
	u64 receive_interrupts_unshare;	// 切断要求割り込み受信総数                         全EPの「切断要求割り込み受信数」を合計する。
	u64 receive_interrupts_stop;	// 停止要求割り込み受信総数                         全EPの「停止要求割り込み受信数」を合計する。
	u64 receive_interrupts_zoneupdate;	// ES情報更新割り込み受信数                         ES情報更新り込みを受けて生成元が通信相手EPの場合に1カウント。
	u64 tx_buffer_full;	// 総送信バッファフル発生回数                       全EPの「送信バッファフル発生回数」を合計する。
	u64 tx_dropped_not_shared;	// 非共有中EPへの送信で破棄したフレーム総数         全EPの「非共有中EPへの送信で破棄したフレーム数」を合計する。
	u64 tx_dropped_version_mismatch;	// ESドライババージョン不一致で破棄したフレーム総数 全EPの「ESドライババージョン不一致で破棄したフレーム数」を合計する。
	u64 tx_dropped_buffer_size_discrepancy;	// バッファサイズ不一致で破棄したフレーム総数       全EPの「バッファサイズ不一致で破棄したフレーム数」を合計する。
	u64 tx_dropped_vlan_id_mismatch;	// タグVLANのID不一致で破棄したフレーム総数         全EPの「タグVLANのID不一致で破棄したフレーム数」を合計する。
};

//$SD EPバッファペア
//$SD １つのEP向けの送信と受信のバッファ
struct ep_share_mem_info {
	//$SD EPバッファハンドル
	//$SD １つのEP向けの送信または受信のバッファの管理ハンドル
	struct epbuf_handler {
		memptr_t buffer;	//$SD 共有領域ポインタ
		size_t size;	//$SD 共有領域のサイズ
		union ep_buffer_info *info;	//$SD 通信管理テーブル(共有バッファ) bufferの先頭のポインタを設定
		u8 *ring;	//$SD リングバッファ(byte管理) bufferのinfoの次の位置のポインタを設定
	} tx, rx;
	//$SD 送信バッファ
	//$SD 受信バッファ

	//$SD EPの統計(netdevice形式)
	struct rtnl_link_stats64 net_stats;

	//$SD EPの統計(独自形式)
	struct es_driver_ep_stats ep_stats;

	//$SD EPごとの送信突き離し処理ステータス
	u16 tx_status_work;

	//$SD EPとESデバイスの状態(0x00 無効、0x01 有効、0xFF 存在しない)とZone情報
	//$SD requestInfoで更新情報が取れる
	u8 es_status;
	u8 zone;

};

//$SD トレースバッファ構造体
struct es_device_trace {
	u32 record_num;		//$SD レコード数
	u32 current_record;	//$SD レコード最新現在位置
	u32 status_flag;	//$SD 状態フラグ
	u32 _rsv;		//$SD (リザーブ)

#define ESDRV_TRACE_RECORD_SIZE (16)
	struct {
		u16 epid;	//$SD EPID
		u16 dir_offset;	//$SD アクセス方向(bit15-12) 0:r,1:w とオフセット(bit11-0)
		u32 data;	//$SD アクセスデータ
		u64 tsc;	//$SD TSC値
	} record[];		//$SD トレースレコード(16byte)
};

//$SD ACPI情報テーブル
struct es_device_acpi_table {
	bool found;		// デバイス検出
	bool status;		// デバイス有効無効
	u64 start;		// 開始アドレス
	u64 size;		// メモリエリアサイズ
	int irq;		// irq
};

//$SD procの登録状態
struct es_device_proc_table {
	bool root;
	bool info;
	bool status;
	bool stats;
	bool log;
};

//$SD net_device構造体のプライベート領域(主にesdrv_で使用)
struct esdrv_priv {

	//リカバリ処理系
	struct work_struct force_close;	// 強制クローズ処理ワークキュー(これのみシステムのwqを使い、cancel_syncしないこと)
	bool force_reset;	// クローズ時の強制リセットフラグ(実行時に落すこと)
	bool open_guard;	// ドライバのオープンを失敗させるフラグ

	struct rtnl_link_stats64 stats64;	///< 統計情報(64bit版)
	struct napi_struct napi;	///< NAPIオブジェクト

	bool irq_vmm_registerd;	///< 割り込みハンドラの登録状況
	unsigned int tx_retry_count;	///< 送信リトライカウンタ(同一フレームの送信済回数)
	unsigned long tx_start_jiffy;	///< 同一フレームの最初の送信時刻(単位:jiffies)
	unsigned long rx_last_jiffy;	///< 最後の受信からの経過時間(単位:jiffies)
	bool unset_rx_last;	///< 最後の受信からの経過時間が設定されていないか

	struct workqueue_struct *txrx_wq;	//送受信系ワークキュー
	struct workqueue_struct *control_wq;	//制御系ワークキュー

	struct work_struct raise_intr_rxdata_work;	///< 受信要求割り込みを発生させるワークキューエントリ
	struct work_struct tx_stall_work;	///< 送信フル時のストール抑制ワークキューエントリ

	struct work_struct unshare_watch_work;	// 共有状態監視
	unsigned long unshare_watch_bit;	// 共有状態監視(EP毎のビット) 

	struct delayed_work interrupt_watch_work;	// 割り込みの遅延監視ワークキュー
	bool interrupt_watch_enable;	// 割り込みの遅延監視ワークキュー動作状態

	//$SD esif_device構造体
	//$SD ESデバイスとのインターフェイス用   主にesif_以上の階層で使う
	struct esif_device {
		struct net_device *container_dev;	//net_device逆引き用

		// 共有制御関連
		unsigned long txrx_stop_req_bit;	//他EPからの送受信中止要求(EP毎のビット)

		struct work_struct update_zone_work;	//Zone更新ワークキュー

		struct work_struct workqueue_epstop;	//送受信中止処理ワークキュー
		unsigned long epstop_req_bit;	//送受信中止処理ワークキューへの通知として使う(EP毎のビット)

		struct es_driver_stats drv_stats;	///< ドライバ独自統計

		epno_t epid_self;	//自EPID
		epno_t epnum_max;	//最大EP数
		struct ep_share_mem_info *ep_shm_info;	// 必要なEP分の配列として確保する EPバッファ配列(全EPのTX,RXバッファ)

		spinlock_t rx_status_lock;	// rx_statusのスピンロック


		struct es_device_acpi_table *config;

		es_device_register *base;	// VMM レジスタ(xsocketデバイスが確保した領域)
		//$SD ドライバ側で確保し、デバイス側に割り付ける領域
		struct share_pointers_s {
			struct es_device_shared_info *share;	// 共有ステータス領域
			union es_device_command_req *req_buf;	// 要求データ領域        要求に使うに十分なバッファが割り当てられていること
			u64 req_buf_size;
			union es_device_command_res *res_buf;	// 応答データ領域        応答に使うに十分なバッファが割り当てられていること
			u64 res_buf_size;

			epno_t *epid_self;	//自EPID(ポインタ)
			epno_t *epnum_max;	//最大EP数(ポインタ)

			struct es_device_trace *trace;
			u64 trace_size;

			struct mutex lock;	// コマンド実行時のバッファ排他ロック

			unsigned long buffer_share_bit;	//共有発行ビットマップ
			unsigned long buffer_unshare_reserve_bit;	//共有発行解除予約ビットマップ
		} share_ptrs;

		//$SD トレースの実行モード、開始前はtrace_startがfalse
		bool trace_start;
		u32 trace_mode;
	} xsdevice;
	
	//$SD ログ管理構造
	struct {
		u64 count; // ログ総数
		unsigned int max;    // ログの確保サイズ(行数) バイト数算出は ( max * ES_DRV_LOG_LINESIZE )
		atomic_t index;  // リングバッファの現在インデックス(最新ログ格納後インクリメント)
		
		u8 *buffer; // リングバッファ [0..max-1]
		u64 log_jiffies[ES_DRV_LOG_MSG_MAX]; // リングバッファへの格納時間
		phys_addr_t buffer_page_address[ES_DRV_LOG_MSG_PAGE_NUM]; // リングバッファ ページアドレス(物理)
		
		struct timespec start_time; // ロード時の時刻取得結果
		u64 start_jiffiy;
		struct tm start_tm; // tm構造体への変換結果
	}log;
};

// 統計をEPごとに持つ必要もあるので、次のマクロを定義する
#define ES_DRV_STATS_INC(_priv,_epid,_type,_inc_val) do{                      \
	(_priv)->stats64._type += (_inc_val);                                 \
	(_priv)->xsdevice.ep_shm_info[(_epid)].net_stats._type += (_inc_val); \
}while(0)

// ドライバ固有の統計のため、以下も定義       
#define ES_DRV_CHARACTERISTIC_STATS_INC(_priv,_epid,_type,_inc_val) do{      \
	(_priv)->xsdevice.drv_stats._type += (_inc_val);                     \
	(_priv)->xsdevice.ep_shm_info[(_epid)].ep_stats._type += (_inc_val); \
}while(0)

// ロギングマクロ
// もし、値の更新時にすでにindexが変更されていた場合は、cmpxchg処理は動かないが影響はない(次にincした処理側でcmpxchgする)
// ラインログなので、改行は入れないこと
#define ES_DRV_LOGGING(msg, ...) do{ if(NULL != esdrv_device) {                    \
	struct esdrv_priv *priv = netdev_priv(esdrv_device);                       \
	int index,old_val,new_val;                                                 \
	old_val = atomic_inc_return(&(priv->log.index));                           \
	new_val = old_val % priv->log.max;                                         \
	index = (old_val - 1) % priv->log.max;                                     \
	if(priv->log.buffer){                                                      \
		memset((void *)&(priv->log.buffer[index * ES_DRV_LOG_LINESIZE]),0, \
			ES_DRV_LOG_LINESIZE);                                      \
		snprintf((char *)&(priv->log.buffer[index * ES_DRV_LOG_LINESIZE]), \
			ES_DRV_LOG_MSG_LEN,msg , ##__VA_ARGS__);                   \
		priv->log.log_jiffies[index] =  get_jiffies_64();                  \
		priv->log.count++;                                                 \
		if(old_val != new_val){                                            \
			atomic_cmpxchg(&(priv->log.index),old_val,new_val);        \
		}                                                                  \
	}                                                                          \
}}while(0)



//-----------------------------------------------------------------------------
//関数プロトタイプ宣言

//先行の型宣言(定義は後)
struct esif_device;		//esデバイス(esif)
struct epbuf_handler;		// バッファハンドラ(esmem)
enum es_dev_command_response_e;
struct esdrv_priv;
enum INTREG_IRQMASK;
struct share_pointers_s;

FUNC_STATIC void esdrv_tx_stall(struct work_struct *work);

FUNC_STATIC __init int esdrv_module_init(void);
FUNC_STATIC __init int esdrv_module_init_device(struct net_device **pdev);
FUNC_STATIC void esdrv_netdev_setup(struct net_device *dev);
FUNC_STATIC void esdrv_module_exit(void);
FUNC_STATIC void esdrv_module_exit_device(struct net_device *);
FUNC_STATIC int esdrv_open(struct net_device *dev);
FUNC_STATIC int esdrv_stop(struct net_device *dev);
FUNC_STATIC netdev_tx_t esdrv_start_xmit(struct sk_buff *skb,
					 struct net_device *dev);
FUNC_STATIC void esdrv_tx_retry(struct net_device *dev);
FUNC_STATIC void esdrv_rx_irq(struct net_device *dev, epno_t src_epid);
FUNC_STATIC int esdrv_rx_napi_poll(struct napi_struct *napi, int budget);
FUNC_STATIC int esdrv_change_mtu(struct net_device *dev, int newMtu);
#if 0				/* 現状不要になった */
FUNC_STATIC struct net_device_stats *esdrv_get_stats(struct net_device *dev);
#endif				/* 0 */
FUNC_STATIC struct rtnl_link_stats64 *esdrv_get_stats64(struct net_device *dev,
							struct rtnl_link_stats64
							*storage);
FUNC_STATIC int esdrv_do_ioctl(struct net_device *dev, struct ifreq *rq,
			       int cmd);
#ifndef ESDRV_VLAN_OFF
//  Kernel 3.3から戻り値がvoidではなくintになる
FUNC_STATIC int esdrv_vlan_rx_add_vid(struct net_device *dev, __be16 proto,
				      unsigned short vid);
FUNC_STATIC int esdrv_vlan_rx_kill_vid(struct net_device *dev, __be16 proto,
				       unsigned short vid);
#endif				/* !ESDRV_VLAN_OFF */
#if 0				/* 現状不要になった */
FUNC_STATIC u16 esdrv_select_queue(struct net_device *dev, struct sk_buff *skb);
#endif				/* 0 */

FUNC_STATIC void esdrv_proc_create(struct net_device *dev);
FUNC_STATIC void esdrv_proc_remove(void);

FUNC_STATIC int esdrv_ioctl_trace_start(struct net_device *dev,
					struct ifreq *rq);
FUNC_STATIC int esdrv_ioctl_trace_stop(struct net_device *dev,
				       struct ifreq *rq);
FUNC_STATIC int esdrv_ioctl_trace_getcfg(struct net_device *dev,
					 struct ifreq *rq);
FUNC_STATIC int esdrv_ioctl_reg_read(struct net_device *dev, struct ifreq *rq);
FUNC_STATIC int esdrv_ioctl_reg_write(struct net_device *dev, struct ifreq *rq);

#if !defined(ESDRV_ETHERTOOL_OFF)
FUNC_STATIC void ethtool_get_ethtool_stats(struct net_device *netdev,
					   struct ethtool_stats __always_unused
					   * stats, u64 * data);
FUNC_STATIC void ethtool_get_strings(struct net_device __always_unused * netdev,
				     u32 stringset, u8 * data);
FUNC_STATIC int ethtool_get_sset_count(struct net_device __always_unused *
				       netdev, int sset);
FUNC_STATIC void ethtool_get_drvinfo(struct net_device *netdev,
				     struct ethtool_drvinfo *drvinfo);
FUNC_STATIC int ethtool_get_settings(struct net_device *netdev,
				     struct ethtool_cmd *ecmd);
#endif				/* !defined(ESDRV_ETHERTOOL_OFF) */

FUNC_STATIC void esdrv_force_close_work(struct work_struct *work);

// esmem
// 送受信のバッファサイズと確保、開放、初期化設定
#if 0				/* 現状不要になった */
FUNC_STATIC size_t esmem_epbuf_size(void);
#endif				/* 0 */
FUNC_STATIC int esmem_epbuf_alloc(struct epbuf_handler *epbh);
FUNC_STATIC void esmem_epbuf_free(struct epbuf_handler *epbh);

FUNC_STATIC void esmem_epbuf_setup(struct epbuf_handler *epbh, u8 * mac_addr,
				   u32 mtu);

// 送受信バッファを使う(tx,rx)
// rx_curpktは排他していないが、複数箇所が同時に操作することを許容していない。ポーリングと割り込みが平行動作しないことは上位層で保証すること
FUNC_STATIC void esmem_epbuf_rx_curpkt_drop(struct epbuf_handler *epbh);
FUNC_STATIC framebuf_t esmem_epbuf_rx_curpkt_get_addr(struct epbuf_handler
						      *epbh, size_t * size);

FUNC_STATIC int esmem_epbuf_tx_pkt_send(struct epbuf_handler *epbh,
					framebuf_t frame, size_t size);

// 共有ステータス領域
FUNC_STATIC int esmem_share_alloc(struct esif_device *xdev);
FUNC_STATIC void esmem_share_free(struct esif_device *xdev);
FUNC_STATIC int esmem_is_shared(struct es_device_shared_info *aShare,
				epno_t dest_epid);

FUNC_STATIC bool esmem_has_vid(struct epbuf_handler *epbh, u16 vlan_id);
FUNC_STATIC bool esmem_set_vid(struct epbuf_handler *epbh, u16 vlan_id);
FUNC_STATIC void esmem_del_vid(struct epbuf_handler *epbh, u16 vlan_id);

FUNC_STATIC bool esmem_same_mtu(struct epbuf_handler *epbh, u32 mtu);
FUNC_STATIC bool esmem_same_ver(struct epbuf_handler *epbh, u32 ver);
FUNC_STATIC bool esmem_same_zone(struct esif_device *xdev, epno_t epid);

// esvmm
// ACPIアクセスによるデバイスの存在チェック
FUNC_STATIC bool esvmm_exist_device(void);
FUNC_STATIC bool esvmm_is_device_valid(void);

FUNC_STATIC acpi_status esvmm_acpi_check(acpi_handle handle, u32 lvl,
					 void *context, void **rv);
FUNC_STATIC acpi_status esvmm_acpi_probe(acpi_handle handle,
					 struct es_device_acpi_table *table);
FUNC_STATIC acpi_status esvmm_acpi_resources(struct acpi_resource *res,
					     void *data);

// ACPIアクセスによるデバイス情報取得
FUNC_STATIC int esvmm_get_device_config(struct esif_device *xdev);
#if 0				/* 現状不要になった */
FUNC_STATIC int esvmm_free_device_config(struct esif_device *xdev);
#endif				/* 0 */

// ベースアドレス設定
FUNC_STATIC es_device_register *esvmm_setup_address(struct esif_device *xdev);
FUNC_STATIC void esvmm_release_address(struct esif_device *xdev);

// 情報取得
FUNC_STATIC epno_t esvmm_get_self_epid(es_device_register * base);
FUNC_STATIC epno_t esvmm_get_max_epnum(es_device_register * base);

// デバイス制御
FUNC_STATIC int esvmm_reset_device(es_device_register * base);
#if 0				/* 現状不要になった */
//$DD バージョン通知
FUNC_STATIC void esvmm_notify_dev_ver(es_device_register * base);
#endif				/* 0 */

// バッファ登録のセットアップ関数
FUNC_STATIC void esvmm_command_setup(es_device_register * base,
				     struct es_device_command_param *param);

// 汎用コマンド制御(内部で利用)
FUNC_STATIC enum es_dev_command_response_e
esvmm_command_request(es_device_register * base,
		      enum es_dev_command_request_type type);

// ES情報通知
FUNC_STATIC int esvmm_request_info(es_device_register * base,
				   struct share_pointers_s *ptrs);
// バッファ共有登録
FUNC_STATIC int esvmm_register_buff_addr(es_device_register * base,
					 epno_t dest_epid,
					 struct ep_share_mem_info *bufPair,
					 struct share_pointers_s *ptrs);
// バッファ共有解除
FUNC_STATIC int esvmm_unregister_buff_addr(es_device_register * base,
					   epno_t dest_epid,
					   struct share_pointers_s *ptrs);

// トレース開始、終了
FUNC_STATIC int esvmm_start_trace(es_device_register * base, int mode,
				  struct share_pointers_s *ptrs);
FUNC_STATIC int esvmm_stop_trace(es_device_register * base,
				 struct share_pointers_s *ptrs);

// 割り込み関連
FUNC_STATIC intreg_t esvmm_interrupt_capture(es_device_register * base);

// 割り込み通知
FUNC_STATIC int esvmm_raise_interrupt(es_device_register * base,
				      epno_t dest_epid,
				      enum INTREG_IRQMASK intr_mask);

// 割り込みマスク
FUNC_STATIC void esvmm_irqmask(es_device_register * base,
			       enum INTREG_IRQMASK intr_mask, bool mask);
#if 0				/* 現状不要になった */
// 自動割り込みマスク
FUNC_STATIC void esvmm_automask_set(es_device_register * base, bool status);
#endif				/* 0 */

FUNC_STATIC irqreturn_t esif_vmm_irq(int irq, memptr_t dev_id);

// 割り込みを通知
FUNC_STATIC void esif_epstop_raise(struct esif_device *xdev);
// 該当するEPの完了待ちをする(20msec wait)
FUNC_STATIC int esif_epstop_wait(struct esif_device *xdev);
// 受信完了ハンドラ
FUNC_STATIC void esif_txrx_stop_req_irq(struct net_device *dev,
					epno_t src_epid);
// 共有解除ワークキュー
FUNC_STATIC void esif_epstop_work(struct work_struct *work);
// 共有解除リクエストのステータス取得関数
FUNC_STATIC bool esif_epstop_requested(struct esdrv_priv *priv,
				       epno_t src_epid);

// 共有解除処理完了通知でのハンドラだが、現在は割り込みでの通知はなくなったので、基本利用しなくなった
//#define ES_DRV_USE_STOP_DONE_IRQ
#if defined(ES_DRV_USE_STOP_DONE_IRQ)
FUNC_STATIC void esif_txrx_stop_done_irq(struct net_device *dev,
					 epno_t src_epid);
#endif				/* ES_DRV_USE_STOP_DONE_IRQ */

// Txのrx_statusチェック用
FUNC_STATIC bool esif_epstop_process_done(struct esdrv_priv *priv,
					  epno_t src_epid);

// 停止要求ハンドラ
FUNC_STATIC void esif_dev_stop_req_irq(struct net_device *dev, epno_t src_epid);

// 共有情報更新ハンドラ
FUNC_STATIC void esif_update_zone_irq(struct net_device *dev, epno_t src_epid);

// 共有情報更新ワークキュー
FUNC_STATIC void esif_update_zone_work(struct work_struct *work);

// 共有解除監視ワークキュー 
FUNC_STATIC void esif_watch_unshare(struct work_struct *work);

//割り込みの遅延監視ワークキュー
FUNC_STATIC void esdrv_irq_status_work(struct work_struct *work);

// 状態のenumとその算出マクロ
enum ep_partner_status {
	// 状態が増えたことでEP_PARTNER_UNSHAREを次へ置き換えを検討
	//(ただ、unreg状態は個別にコマンドがopen/close時処理しているので、不要かもしれない)
	//EP_PARTNER_UNREGISTERED,
	//EP_PARTNER_REGISTERED,

	EP_PARTNER_UNSHARE,
	EP_PARTNER_SHARED,
	EP_PARTNER_WAITING,
	EP_PARTNER_COMPLETE,
	EP_PARTNER_STATUS_MAX
};
#define EP_PARTNER_STATUS(_priv,_epid)                                 \
	( !esmem_is_shared((_priv)->xsdevice.share_ptrs.share, _epid)  \
		? EP_PARTNER_UNSHARE                                   \
		: esif_epstop_requested(_priv,_epid)                   \
			? EP_PARTNER_WAITING                           \
			: esif_epstop_process_done(_priv,_epid)        \
				? EP_PARTNER_COMPLETE                  \
				: EP_PARTNER_SHARED )

// 改訂版
//#define EP_PARTNER_STATUS(_priv,_epid)
//(!test_bit(_epid,&(_priv)->xsdevice.share_ptrs.buffer_share_bit)
//      ? EP_PARTNER_UNREGISTERED
//      : !esmem_is_shared((_priv)->xsdevice.share_ptrs.share, _epid)
//              ? EP_PARTNER_REGISTERED
//              : esif_epstop_requested(_priv,_epid)
//                      ? EP_PARTNER_WAITING
//                      : esif_epstop_process_done(_priv,_epid)
//                              ? EP_PARTNER_COMPLETE
//                              : EP_PARTNER_SHARED )

// esdrv_netdev_setup()で参照される定数

// ネットワークデバイス機能定義
static const struct net_device_ops esdrv_netdev_ops = {
	.ndo_open = esdrv_open,
	.ndo_stop = esdrv_stop,
	.ndo_start_xmit = esdrv_start_xmit,
	.ndo_do_ioctl = esdrv_do_ioctl,
	.ndo_get_stats64 = esdrv_get_stats64,
	.ndo_change_mtu = esdrv_change_mtu,
	.ndo_tx_timeout = esdrv_tx_retry,

#ifndef ESDRV_VLAN_OFF
	.ndo_vlan_rx_add_vid = esdrv_vlan_rx_add_vid,
	.ndo_vlan_rx_kill_vid = esdrv_vlan_rx_kill_vid,
#endif				/* !ESDRV_VLAN_OFF */
#if 0				/* 現状不要になった */
	.ndo_select_queue = esdrv_select_queue,
#endif				/* 0 */
};

//デバイス
struct net_device *esdrv_device = NULL;
struct es_device_acpi_table esdrv_acpi_table;
struct es_device_proc_table esdrv_proc_table = {
	.root = false,
	.info = false,
	.status = false,
	.stats = false,
	.log = false,
};

/**$DD
  ２．  関数詳細

 Extended Socket Linux Driver モジュールの関数詳細を以下に記述する。
**/

//$DD ソース上の記述と章立て番号を調整、1章の番号準拠に整理しているため、詳細仕様書を起こす場合は2.x単位で順序入れ替えすること

//$DD=============================================================================
//$DD ２．１ ネットワークドライバ  カーネル部
//$DD Linuxカーネルのネットワークドライバ機能のため、記述なし

//$DD=============================================================================
//$DD ２．５ ESデバイス(VMM)インターフェイス
//$DD ハードウェア制御の処理集合

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．５．１  ESデバイスのベースアドレスを取得、設定する
	
  ACPIから取得しているベースアドレス、サイズからMMIOマッピングを行う。
	
  @param[out]	xdev	es_device情報
	
  @return  VMMのベースアドレス(レジスタアドレス)
  @retval  NULL		エラー(デバイスが存在しない)
**/
FUNC_STATIC es_device_register *esvmm_setup_address(struct esif_device *xdev)
{
	es_device_register *base_addr;

	PDEBUG(PD_ENT, "ENTER");

	PDEBUG(PD_COMMON, "phy_addr=%Lx", xdev->config->start);

	//$DD ベースアドレスをrequest_mem_regionで確保できるか確認     request_mem_region()
	if (!request_mem_region
	    (xdev->config->start, xdev->config->size,
	     ES_DEVICE_MEM_REGION_NAME)) {
		PDEBUG(PD_COMMON || PD_INF, "request_mem_region fail");
		return NULL;
	}
	//$DD IOメモリ割り当て   ioremap_nocache()
	base_addr =
	    (es_device_register *) ioremap_nocache(xdev->config->start,
						   xdev->config->size);
	PDEBUG(PD_COMMON || PD_INF, "base_addr=%p", base_addr);
	return base_addr;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．５．２  ESデバイスのベースアドレスを解放
	
  MMIOマッピングの解除を行う。
	
  @param[out]	xdev	es_device情報
**/
FUNC_STATIC void esvmm_release_address(struct esif_device *xdev)
{
	es_device_register *reg = xdev->base;

	PDEBUG(PD_ENT, "ENTER base_addr=%p", reg);
	//$DD IOメモリ割り当て解放     iounmap()
	iounmap(reg);
	//$DD ベースアドレスをrelease_mem_regionで解放       release_mem_region()
	release_mem_region(xdev->config->start, xdev->config->size);
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．５．３ ESデバイスの存在確認
	
  ACPIの探索を行い、ESデバイスが存在するか確認する。
  なお、合わせてACPI上のデバイスステータスやベースアドレス、サイズ、IRQの保存処理が
  行われる。

  @retval  true		存在する
  @retval  false	存在しない
**/
FUNC_STATIC bool esvmm_exist_device(void)
{
	PDEBUG(PD_ENT || PD_ACPI, "ENTER");

	//$DD ACPIテーブルを初期化
	memset(&esdrv_acpi_table, 0, sizeof(esdrv_acpi_table));
	esdrv_acpi_table.found = false;

	//$DD ACPIによるテーブル情報取得結果を返す、esvmm_acpi_check()をコールバックとしてacpi_get_devices()を呼び出し
	return
	    ACPI_SUCCESS(acpi_get_devices
			 (ES_DEVICE_ACPI_DEVICE, esvmm_acpi_check,
			  &esdrv_acpi_table, NULL)) && esdrv_acpi_table.found;
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．５．４ ACPIによるテーブルの検索
	
	acpi_get_devicesコールバックとして、ACPIデバイステーブルを走査する

  @param[in]	handle	ACPIハンドル
  @param[in]	lvl		ACPIアクセスレベル
  @param[in]	context	呼び出し時データ
  @param[in]	rv		戻り値用ポインタ
	
  @retval	AE_OK				成功、存在しないので継続
  @retval	AE_CTRL_TERMINATE	検出したので、探索停止
**/
FUNC_STATIC acpi_status esvmm_acpi_check(acpi_handle handle, u32 lvl,
					 void *context, void **rv)
{
	struct es_device_acpi_table *table =
	    (struct es_device_acpi_table *)context;

	PDEBUG(PD_ENT || PD_ACPI, "ENTER");

	//$DD ACPIにESデバイスが存在するか調査     esvmm_acpi_probe()
	if (ACPI_SUCCESS(esvmm_acpi_probe(handle, table))) {
		PDEBUG(PD_ACPI, "found");
		//$DD 検出したらfound=trueとし、探索を停止させるためAE_CTRL_TERMINATEを返す
		table->found = true;
		return AE_CTRL_TERMINATE;
	}
	//$DD 検出しなかった場合は探索継続としてAE_OKを返す
	return AE_OK;
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．５．５ ACPIテーブルの検索実体処理

	ACPIデバイステーブルから、ESデバイスを探し、内容保存を行う
	
	
  @param[in]	handle	ACPIハンドル
  @param[in]	table	ACPIテーブルデータ保存
	
  @retval	AE_OK			成功
  @retval	AE_NOT_FOUND	違うデバイスなので失敗
  @retval	他				APIのNGステータス
	
**/
FUNC_STATIC acpi_status esvmm_acpi_probe(acpi_handle handle,
					 struct es_device_acpi_table * table)
{
	acpi_status status;
	acpi_handle tmp;
	struct acpi_buffer buffer = { ACPI_ALLOCATE_BUFFER, NULL };
	union acpi_object *str;
	char str_buf[sizeof(ES_DEVICE_ACPI_SYMBOL) + 1];
	int result;
	unsigned long long status_val;

	PDEBUG(PD_ENT || PD_ACPI, "ENTER");

	//$DD 文字列を取得し、比較する
	//$DD 文字列ハンドラ取得による存在確認  acpi_get_handle()
	status = acpi_get_handle(handle, ES_DEVICE_ACPI_STRING, &tmp);
	if (ACPI_FAILURE(status)) {
		PDEBUG(PD_ACPI, "%s get handle fail:%d", ES_DEVICE_ACPI_STRING,
		       status);
		//$DD 失敗の場合、失敗ステータスを返す
		return status;
	}
	//$DD 親ハンドラで、文字列データを取得  acpi_evaluate_object()
	status =
	    acpi_evaluate_object(handle, ES_DEVICE_ACPI_STRING, NULL, &buffer);
	if (ACPI_FAILURE(status)) {
		PDEBUG(PD_ACPI, "%s get object fail:%d", ES_DEVICE_ACPI_STRING,
		       status);
		//$DD 失敗の場合、失敗ステータスを返す
		return status;
	}
	//$DD データのポインタを取得する、ここから文字列データ利用終了までについて、異常終了で、このポインタをkfreeすること
	str = buffer.pointer;

	//$DD UTF-16をUTF-8に変換し、文字列比較      utf16s_to_utf8s()
	result = utf16s_to_utf8s((wchar_t *) str->string.pointer,
				 str->string.length,
				 UTF16_LITTLE_ENDIAN, str_buf,
				 sizeof(str_buf) - 1);
	str_buf[result] = 0;	// Null Term.

	//$DD 文字列比較で同じ文字列でなければAE_NOT_FOUND
	if (strncmp
	    (ES_DEVICE_ACPI_SYMBOL, str_buf,
	     strlen(ES_DEVICE_ACPI_SYMBOL)) != 0) {
		PDEBUG(PD_ACPI, "%s str cmp fail:%s", ES_DEVICE_ACPI_STRING,
		       str_buf);

		kfree(buffer.pointer);
		return AE_NOT_FOUND;
	}
	//$DD 文字列データ利用が終了したので、ポインタをkfreeする
	kfree(buffer.pointer);

	//$DD ステータス取得
	//$DD ステータスハンドラ取得による存在確認    acpi_get_handle()
	status = acpi_get_handle(handle, ES_DEVICE_ACPI_STATUS, &tmp);
	if (ACPI_FAILURE(status)) {
		PDEBUG(PD_ACPI, "%s get handle fail:%d", ES_DEVICE_ACPI_STATUS,
		       status);
		//$DD 失敗の場合、失敗ステータスを返す
		return status;
	}
	//$DD 親ハンドラで、ステータスを取得     acpi_evaluate_integer()
	status =
	    acpi_evaluate_integer(handle, ES_DEVICE_ACPI_STATUS, NULL,
				  &status_val);
	if (ACPI_FAILURE(status)) {
		PDEBUG(PD_ACPI, "%s get object fail:%d", ES_DEVICE_ACPI_STATUS,
		       status);
		//$DD 失敗の場合、失敗ステータスを返す
		return status;
	}
	//$DD ステータスがEnableならtrueを保存、そうでないならfalseを保存
	//table->status = (status_val & ACPI_STA_DEVICE_ENABLED) ? true : false;
	table->status =
	    (status_val == ES_DEVICE_ACPI_STATUS_ES_ENABLE) ? true : false;

	//$DD esvmm_acpi_resources, table()をコールバックとしてacpi_walk_resources()でbaseアドレス、IRQを取得する
	status =
	    acpi_walk_resources(handle, ES_DEVICE_ACPI_RESOURCE,
				esvmm_acpi_resources, table);
	if (ACPI_FAILURE(status)) {
		PDEBUG(PD_ACPI, "%s get resources fail:%d",
		       ES_DEVICE_ACPI_RESOURCE, status);
		//$DD 失敗の場合、失敗ステータスを返す
		return status;
	}
	//$DD AE_OKで終了
	return AE_OK;
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．５．６ ACPIテーブルリソース調査

	ACPIデバイステーブルから、ESデバイスのリソースを調査し、内容保存を行う

  @param[in]	res		ACPIリソース
  @param[in]	data	ACPIテーブルデータ保存
	
  @retval	AE_OK			成功
  @retval	AE_ERROR		失敗
	
**/
FUNC_STATIC acpi_status esvmm_acpi_resources(struct acpi_resource * res,
					     void *data)
{
	acpi_status status;
	struct acpi_resource_address32 *addr;
	struct acpi_resource_irq *irqp;
	struct es_device_acpi_table *table =
	    (struct es_device_acpi_table *)data;

	PDEBUG(PD_ENT, "ENTER");

	switch (res->type) {
	case ACPI_RESOURCE_TYPE_ADDRESS32:
		//$DD ACPI_RESOURCE_TYPE_ADDRESS32:リソースのポインタを取得する
		addr = &res->data.address32;

		PDEBUG(PD_ACPI, "address route");
		//$DD 開始アドレス、長さ(u32)を取得、保存
		table->start = addr->minimum;
		table->size = addr->address_length;
		//table->size = ES_DEVICE_REGISTER_SIZE;
		PDEBUG(PD_ACPI, "address %016llx,size %016llx", table->start,
		       table->size);
		break;
	case ACPI_RESOURCE_TYPE_IRQ:
		//$DD ACPI_RESOURCE_TYPE_IRQ:IRQ情報の取得
		//int irq;

		PDEBUG(PD_ACPI, "IRQ route");

		//$DD IRQ登録情報を調べ、登録を行う
		irqp = &res->data.irq;

		//irq = acpi_register_gsi(NULL, irqp->interrupts[i],irqp->triggering, irqp->polarity);
		//if (irq < 0 || (1 != irqp->interrupt_count))
		//      return AE_ERROR;
		//的な処理が必要かどうか不明、この場合戻り値のirqのチェック、保存、最後にvoid acpi_unregister_gsi (u32 gsi); が必要になるはず
		//必要な場合はesvmm_free_device_configを復活させる?対称性を考えると、esvmm_get_device_configで登録するのがよい?

		//$DD 値が正くない場合処理終了
		if (1 != irqp->interrupt_count) {
			PDEBUG(PD_ACPI, "IRQ fail");
			//$DD AE_ERRORで終了
			return AE_ERROR;
		}
		//$DD IRQの保存
		table->irq = irqp->interrupts[0];
		//table->irq = irq;
		PDEBUG(PD_ACPI, "IRQ:%d", table->irq);
		break;
	default:
		//$DD default:AE_OKで調査継続
		break;
	}
	//$DD AE_OKで終了
	return AE_OK;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．５．７ ESデバイスが有効かチェックする

  ACPI情報取得時にtrue/false変換済みのデバイスステータスを返す
	
  @retval  true		自環境がEP
  @retval  false	自環境がEP以外
**/
FUNC_STATIC bool esvmm_is_device_valid(void)
{
	PDEBUG(PD_ENT || PD_ACPI, "ENTER");
	//$DD ACPIのデバイスステータスがEnableならtrueを返す(値取得時に変換済み)
	return esdrv_acpi_table.status;
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．５．８ ESデバイスの情報取得

  @param[out]	xdev	es_device情報
	
  @return		エラーステータス
  @retval	0	正常終了
  @retval	0以外	異常終了(タイムアウト)
**/
FUNC_STATIC int esvmm_get_device_config(struct esif_device *xdev)
{
	PDEBUG(PD_ENT || PD_ACPI, "ENTER");

	//$DD ACPI情報テーブルをxdevに登録
	xdev->config = &esdrv_acpi_table;

	//$DD 正常終了で0を返す
	return 0;
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．５．９ ESデバイスの情報解放
	
	欠番
	
	現状では欠番として詳細には記載すること

  @param[out]	xdev	es_device情報
	
  @return		エラーステータス
  @retval	0	正常終了
  @retval	0以外	異常終了(タイムアウト)
**/
#if 0				/* 現状不要になった */
FUNC_STATIC int esvmm_free_device_config(struct esif_device *xdev)
{
	PDEBUG(PD_ENT || PD_ACPI, "ENTER");

	//$DD ACPIデバイス情報を削除
	xdev->config = NULL;

	//$DD 正常終了で0を返す
	return 0;
}
#endif				/* 0 */

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．５．１０ 自EPIDを取得

  @param[in,out] base	ESデバイス レジスタベースアドレス

  @return  自EPID
**/
FUNC_STATIC epno_t esvmm_get_self_epid(es_device_register * base)
{
	//$DD INFO[0]から自EP IDを返す
	INFO0_REG info;

	PDEBUG(PD_ENT, "ENTER");
	info.Reg = esvmm_readl((void *)(base + XSCT_REG_OWNER_EPID));
	return info.Bits.epid;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．５．１１ 最大EP数を取得

  @param[in,out] base	ESデバイス レジスタベースアドレス

  @return  最大EP数
**/
FUNC_STATIC epno_t esvmm_get_max_epnum(es_device_register * base)
{
	//$DD INFO[1]から最大EP数を返す
	INFO1_REG info;

	PDEBUG(PD_ENT, "ENTER");
	info.Reg = esvmm_readl((void *)(base + XSCT_REG_MAX_EP));
	return info.Bits.maxep;
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．５．１２ デバイスへのバージョン通知
	
	欠番
	
	現状では欠番として詳細には記載すること
	
	実施後device_resetすること

  @param[in,out]	base	ESデバイス レジスタベースアドレス
**/
#if 0				/* 現状不要になった */
FUNC_STATIC void esvmm_notify_dev_ver(es_device_register * base)
{
	VERNTFY_REG verntfy;

	PDEBUG(PD_ENT, "ENTER");

	//$DD レジスタに通知設定
	verntfy.Reg = 0;
	verntfy.Bits.notify = 1;
	esvmm_writel(verntfy.Reg, ((void *)(base + XSCT_REG_VERNTFY)));

	//$DD 値のセットアップ
	//$DD シーケンス値設定
	esvmm_writel((__le32) (es_drv_version_magic),
		     ((void *)(base + XSCT_REG_VERSEQ)));

	//$DD アドレスをLo->hiで設定
	/// vmalloc系、そもそも利用ができない可能性あり
	esvmm_writel((__le32) (__pa(&es_drv_version) & GENMASK_ULL(31, 0)), ((void *)(base + XSCT_REG_VERAL)));	/* 下32bit */
	esvmm_writel((__le32) ((__pa(&es_drv_version) & GENMASK_ULL(63, 32)) >> 32), ((void *)(base + XSCT_REG_VERAH)));	/* 上32bit */

	return;			//正常終了
}
#endif				/* 0 */

//$DD-----------------------------------------------------------------------------
/**$DD
	２．５．１３ デバイスのリセット
	
	バッファの共有状態が解除されてしまうので、上位層でケアすること

  @param[in,out]	base	ESデバイス レジスタベースアドレス
  
  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esvmm_reset_device(es_device_register * base)
{
	//$DD リセットレジスタに1設定
	DCTL_REG dctl;
	int timeout;

	PDEBUG(PD_ENT, "ENTER");
	dctl.Reg = 0;
	dctl.Bits.reset = 1;
	esvmm_writel(dctl.Reg, ((void *)(base + XSCT_REG_DCTL)));

	//$DD 完了待ち
	timeout = 0;
	dctl.Reg = esvmm_readl((void *)(base + XSCT_REG_DCTL));
	while ((1 == dctl.Bits.reset)
	       && (timeout < ES_DEVICE_RESET_TIMEOUT_MAX_MSEC)) {
		msleep(ES_DEVICE_RESET_TIMEOUT_UNIT_MSEC);
		timeout += ES_DEVICE_RESET_TIMEOUT_UNIT_MSEC;
		dctl.Reg = esvmm_readl((void *)(base + XSCT_REG_DCTL));
	}

	//$DD タイムアウトした場合 -EIOで返す
	return (timeout < ES_DEVICE_RESET_TIMEOUT_MAX_MSEC) ? 0 : -EIO;
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．５．１４ コマンドのバッファ登録
	
	コマンド要求、応答および共有バッファをレジスタに登録
	メモリ確保後、実施する
	
	この関数はinsmod時のメモリ確保後、実行される
	一度アドレスを設定したら、以降変更はしない(rmmod時のメモリ解放時は共有解除してしまうので問題ない)
	
  @param[in,out]	base	ESデバイス レジスタベースアドレス
  @param[in,out]	param	コマンドパラメタ

**/
FUNC_STATIC void esvmm_command_setup(es_device_register * base,
				     struct es_device_command_param *param)
{

	PDEBUG(PD_ENT, "ENTER");

	//$DD xsocket機能仕様書の順に沿って、レジスタに値を設定

	//$DD 要求および応答のlength設定
	esvmm_writel((__le32) (param->req_len),
		     ((void *)(base + XSCT_REG_REQBL)));
	esvmm_writel((__le32) (param->res_len),
		     ((void *)(base + XSCT_REG_RESPBL)));

	//$DD 要求データのバッファアドレスをLo->hiで設定
	esvmm_writel((__le32) (param->req_start & GENMASK_ULL(31, 0)), ((void *)(base + XSCT_REG_REQBAL)));	/* 下32bit */
	esvmm_writel((__le32) ((param->req_start & GENMASK_ULL(63, 32)) >> 32), ((void *)(base + XSCT_REG_REQBAH)));	/* 上32bit */

	//$DD 応答データのバッファアドレスをLo->hiで設定
	esvmm_writel((__le32) (param->res_start & GENMASK_ULL(31, 0)), ((void *)(base + XSCT_REG_RESPBAL)));	/* 下32bit */
	esvmm_writel((__le32) ((param->res_start & GENMASK_ULL(63, 32)) >> 32), ((void *)(base + XSCT_REG_RESPBAH)));	/* 上32bit */

	//$DD 共有ステータスのバッファアドレスをLo->hiで設定
	esvmm_writel((__le32) (param->share_start & GENMASK_ULL(31, 0)), ((void *)(base + XSCT_REG_SHSTSAL)));	/* 下32bit */
	esvmm_writel((__le32) ((param->share_start & GENMASK_ULL(63, 32)) >> 32), ((void *)(base + XSCT_REG_SHSTSAH)));	/* 上32bit */

}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．５．１５ コマンドのリクエスト/レスポンス
	
  @brief
  パラメタとして struct es_device_command_param構造体に必要データを入れ、そのポインタを受けてコマンドを発行する

  コマンドの要求内容、および応答結果は構造体に入れて渡されているバッファに入る
  バッファはcommandSetup関数で事前にレジスタに登録済み

  @param[in,out] base	ESデバイス レジスタベースアドレス
  @param[in]     type	コマンド種別

  @retval  ES_CMD_STATUS_NORMAL		正常
  @retval  ES_CMD_STATUS_UNKNOWN	不明なエラー
  @retval  ES_CMD_STATUS_TIMEOUT	タイムアウト
  @retval  ES_CMD_STATUS_ERROR_PARAM	パラメタエラー
  @retval  ES_CMD_STATUS_ERROR_STATUS	状態エラー

**/
FUNC_STATIC enum es_dev_command_response_e
esvmm_command_request(es_device_register * base,
		      enum es_dev_command_request_type type)
{
	CR_REG cr;
	CS_REG cs;
	enum es_dev_command_response_e ret = ES_CMD_STATUS_UNKNOWN;
	int timeout = 0;

	PDEBUG(PD_ENT, "ENTER");

	cr.Reg = 0;
	cs.Reg = 0;

	cr.Bits.req_start = 1;
	cr.Bits.req_code = type;

	//$DD コマンド制御レジスタに変数を用いて、一度にフラグ、種別情報を書き込み
	esvmm_writel(cr.Reg, ((void *)(base + XSCT_REG_CR)));

	//$DD 正常か確認
	cr.Reg = esvmm_readl((void *)(base + XSCT_REG_CR));
	if (0 == cr.Bits.error) {
		//$DD completeの状態を監視して、完了待ちをする
		//$DD 現行では6秒程度のタイムアウトとなる
		timeout = 0;
		cs.Reg = esvmm_readl((void *)(base + XSCT_REG_CS));
		while ((1 != cs.Bits.complete)
		       && (timeout < ES_COMMAND_REQ_TIMEOUT_MAX_MSEC)) {
			msleep(ES_COMMAND_REQ_TIMEOUT_UNIT_MSEC);
			timeout += ES_COMMAND_REQ_TIMEOUT_UNIT_MSEC;
			cs.Reg = esvmm_readl((void *)(base + XSCT_REG_CS));
		}

		//$DD 結果の確認
		if (1 == cs.Bits.complete) {
			//$DD 正常に終了した場合、ES_CMD_STATUS_NORMALを返却値に設定
			PDEBUG(PD_INF, "ES_CMD_STATUS_NORMAL");
			ret = ES_CMD_STATUS_NORMAL;
		} else if (timeout >= ES_COMMAND_REQ_TIMEOUT_MAX_MSEC) {
			//$DD busyタイムアウトの場合、ES_CMD_STATUS_TIMEOUTを返却値に設定
			PDEBUG(PD_INF, "ES_CMD_STATUS_TIMEOUT");
			ret = ES_CMD_STATUS_TIMEOUT;
		}
	} else {
		//$DD エラー終了の場合、種別を確認する
		switch (cr.Bits.err_info) {
		case ES_COMMAND_REQ_ERR_INFO_PARAM:
			//$DD パラメタエラーの場合、ES_CMD_STATUS_ERROR_PARAMを返却値に設定
			PDEBUG(PD_INF, "ES_CMD_STATUS_ERROR_PARAM");
			ret = ES_CMD_STATUS_ERROR_PARAM;
			break;
		case ES_COMMAND_REQ_ERR_INFO_STATUS:
			//$DD 状態エラーの場合、ES_CMD_STATUS_ERROR_STATUSを返却値に設定
			PDEBUG(PD_INF, "ES_CMD_STATUS_ERROR_STATUS");
			ret = ES_CMD_STATUS_ERROR_STATUS;
			break;
		default:
			//$DD 未定義のエラーだった場合、ES_CMD_STATUS_UNKNOWNを返却値に設定
			PDEBUG(PD_INF, "ES_CMD_STATUS_UNKNOWN for status");
			ret = ES_CMD_STATUS_UNKNOWN;
			break;
		}
	}

	ES_DRV_LOGGING("%s cr=%08x(req=%04x,error=%d,err_info=%04x,req_start=%d) cs=%08x(req=%04x,busy=%d,complete=%d) timeout=%d",
		__func__,
		cr.Reg,cr.Bits.req_code,cr.Bits.error,cr.Bits.err_info,cr.Bits.req_start,
		cs.Reg,cs.Bits.req_code,cs.Bits.busy,cs.Bits.complete,
		timeout);
	
	PDEBUG(PD_COMMON, "ret=%d", ret);

	//$DD 返却値を返して終了
	return ret;
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．５．１６ トレースの開始要求
	
  @param[in,out] base	ESデバイス レジスタベースアドレス
  @param[in]     mode	トレースモード
  @param[out]    ptrs	各領域(要求バッファ,応答バッファ,共有ステータス)のアドレス

  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード

**/
FUNC_STATIC int esvmm_start_trace(es_device_register * base, int mode,
				  struct share_pointers_s *ptrs)
{
	union es_device_command_req *req_buf = ptrs->req_buf;
	union es_device_command_res *res_buf = ptrs->res_buf;
	struct es_device_shared_info *aShare = ptrs->share;
	enum es_dev_command_response_e ret;
	int i;
	int page_count;
	int result;
	void *addr;

	PDEBUG(PD_ENT, "ENTER");

	//$DD 要求バッファのクリア
	memset(req_buf, 0, ptrs->req_buf_size);
	//$DD 応答バッファのクリア
	memset(res_buf, 0, ptrs->res_buf_size);

	//$DD 要求バッファの設定
	req_buf->start_trace.length =
	    ES_DEV_COMMAND_START_TRACE_REQ_LEN(ptrs->trace_size);
	req_buf->start_trace.mode = mode;

	//$DD トレースバッファのサイズを格納
	req_buf->start_trace.buffer_len = ptrs->trace_size;

	page_count = ptrs->trace_size / MINIMUM_PAGE_SIZE;

	//$DD 次に、トレースバッファの物理アドレス(4K単位)を順次格納
	for (i = 0; i < page_count; i++) {
		addr = ((u8 *) (ptrs->trace)) + (i * MINIMUM_PAGE_SIZE);
		req_buf->start_trace.buffer[i] =
		    (__le64) (page_to_phys(vmalloc_to_page(addr)) +
			      offset_in_page(addr));
	}

	//$DD 応答バッファの設定
	res_buf->start_trace.length = 0;
	res_buf->start_trace.code = 0;

	{
		void *trace;
		phys_addr_t trace_addr;
		trace = (void *)ptrs->trace;
		trace_addr = (phys_addr_t) (page_to_phys(vmalloc_to_page(trace)) + offset_in_page(trace));
		ES_DRV_LOGGING("%s reqbuf length=%d,mode=%d trace size=%lld,phy addr=%p type=%d",
			__func__,req_buf->start_trace.length,req_buf->start_trace.mode,
			ptrs->trace_size,(void *)trace_addr,
			ES_CMD_TYPE_START_TRACE);
	}
	//$DD コマンドを設定してesvmm_command_requestを実行
	ret = esvmm_command_request(base, ES_CMD_TYPE_START_TRACE);

	result = 0;
	ES_DRV_LOGGING("%s resbuf ret=%d,length=%d,code=%d",
		__func__,ret,res_buf->start_trace.length,res_buf->start_trace.code);

	//$DD 正常な応答バッファか長さを確認
	if (ES_DEV_COMMAND_START_TRACE_RES_LEN != res_buf->start_trace.length) {
		//$DD インターフェイスエラー(ES_DRV_INTERNAL_ERR_LENGTH1 = 8)
		pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME, ES_DRV_INTERNAL_ERR_LENGTH1);
	
		ES_DRV_LOGGING("%s NotReady=%d,length=%d",
			__func__,ES_DRV_INTERNAL_ERR_LENGTH1,res_buf->start_trace.length);
		
		result = -ENOMSG;
	} else if (ret == ES_CMD_STATUS_NORMAL) {
		//$DD esvmm_command_requestの結果を評価し戻り値に変換する、正常0、タイムアウトEBUSY、それ以外EPERM
		PDEBUG(PD_INF, "cmd ok,code:%d", res_buf->start_trace.code);
		switch (res_buf->start_trace.code) {
		case ES_COMMAND_REQ_RES_CODE_NORMAL:
			result = 0;
			break;
		default:
			result = -EPERM;
			break;
		}
	} else {
		PDEBUG(PD_INF, "cmd ng,ret:%d", ret);
		switch (ret) {
		case ES_CMD_STATUS_UNKNOWN:
			result = -EPERM;
			break;
		case ES_CMD_STATUS_TIMEOUT:
			//$DD 未Completeタイムアウト(ES_DRV_INTERNAL_ERR_CMD_TIMEO1 = 15)
			pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME,
			       ES_DRV_INTERNAL_ERR_CMD_TIMEO1);
			
			ES_DRV_LOGGING("%s NotReady=%d,ret=%d",
				__func__,ES_DRV_INTERNAL_ERR_CMD_TIMEO1,ret);
			
			result = -EBUSY;
			break;
		case ES_CMD_STATUS_ERROR_PARAM:
			result = -EPERM;
			break;
		case ES_CMD_STATUS_ERROR_STATUS:
			result = -EPERM;
			break;
		default:
			result = -EPERM;
			break;
		}
	}

	PDEBUG(PD_INF, "result:%d", result);
	return result;		//正常終了
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．５．１７ トレースの終了要求

  @param[in,out] base	ESデバイス レジスタベースアドレス
  @param[out]    ptrs	各領域(要求バッファ,応答バッファ,共有ステータス)のアドレス

  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esvmm_stop_trace(es_device_register * base,
				 struct share_pointers_s *ptrs)
{
	union es_device_command_req *req_buf = ptrs->req_buf;
	union es_device_command_res *res_buf = ptrs->res_buf;
	struct es_device_shared_info *aShare = ptrs->share;
	enum es_dev_command_response_e ret;
	int i;
	int page_count;
	int result;
	void *addr;

	PDEBUG(PD_ENT, "ENTER");

	//$DD 要求バッファのクリア
	memset(req_buf, 0, ptrs->req_buf_size);
	//$DD 応答バッファのクリア
	memset(res_buf, 0, ptrs->res_buf_size);

	//$DD 要求バッファの設定
	req_buf->stop_trace.length = ES_DEV_COMMAND_STOP_TRACE_REQ_LEN;

	//$DD 応答バッファの設定
	res_buf->stop_trace.length = 0;
	res_buf->stop_trace.code = 0;

	ES_DRV_LOGGING("%s reqbuf length=%d,type=%d",
		__func__,req_buf->stop_trace.length,
		ES_CMD_TYPE_STOP_TRACE);
	//$DD コマンドを設定してesvmm_command_requestを実行
	ret = esvmm_command_request(base, ES_CMD_TYPE_STOP_TRACE);

	result = 0;
	ES_DRV_LOGGING("%s resbuf ret=%d,length=%d,code=%d",
		__func__,ret,res_buf->stop_trace.length,res_buf->stop_trace.code);

	//$DD 正常な応答バッファか長さを確認
	if (ES_DEV_COMMAND_STOP_TRACE_RES_LEN != res_buf->stop_trace.length) {
		//$DD インターフェイスエラー(ES_DRV_INTERNAL_ERR_LENGTH2 = 9)
		pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME, ES_DRV_INTERNAL_ERR_LENGTH2);

		ES_DRV_LOGGING("%s NotReady=%d,length=%d",
			__func__,ES_DRV_INTERNAL_ERR_LENGTH2,res_buf->stop_trace.length);
		
		result = -ENOMSG;
	} else if (ret == ES_CMD_STATUS_NORMAL) {
		//$DD esvmm_command_requestの結果を評価し戻り値に変換する、正常0、タイムアウトEBUSY、それ以外EPERM
		PDEBUG(PD_INF, "cmd ok,code:%d", res_buf->stop_trace.code);
		switch (res_buf->stop_trace.code) {
		case ES_COMMAND_REQ_RES_CODE_NORMAL:
			result = 0;
			break;
		default:
			result = -EPERM;
			break;
		}
	} else {
		PDEBUG(PD_INF, "cmd ng,ret:%d", ret);
		switch (ret) {
		case ES_CMD_STATUS_UNKNOWN:
			result = -EPERM;
			break;
		case ES_CMD_STATUS_TIMEOUT:
			//$DD 未Completeタイムアウト(ES_DRV_INTERNAL_ERR_CMD_TIMEO2 = 16)
			pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME,
			       ES_DRV_INTERNAL_ERR_CMD_TIMEO2);
			
			ES_DRV_LOGGING("%s NotReady=%d,ret=%d",
				__func__,ES_DRV_INTERNAL_ERR_CMD_TIMEO2,ret);
			
			result = -EBUSY;
			break;
		case ES_CMD_STATUS_ERROR_PARAM:
			result = -EPERM;
			break;
		case ES_CMD_STATUS_ERROR_STATUS:
			result = -EPERM;
			break;
		default:
			result = -EPERM;
			break;
		}
	}

	PDEBUG(PD_INF, "result:%d", result);
	return result;		//正常終了
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．５．１８ ES情報の取得要求
	
  @brief
  応答バッファに格納される結果を利用できる

  @param[in,out] base	ESデバイス レジスタベースアドレス
  @param[out]    ptrs	各領域(要求バッファ,応答バッファ,共有ステータス)のアドレス

  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esvmm_request_info(es_device_register * base,
				   struct share_pointers_s *ptrs)
{
	union es_device_command_req *req_buf = ptrs->req_buf;
	union es_device_command_res *res_buf = ptrs->res_buf;
	struct es_device_shared_info *aShare = ptrs->share;
	enum es_dev_command_response_e ret;
	int result;

	PDEBUG(PD_ENT, "ENTER");

	//$DD 要求バッファのクリア
	memset(req_buf, 0, ptrs->req_buf_size);
	//$DD 応答バッファのクリア
	memset(res_buf, 0, ptrs->res_buf_size);

	//$DD 要求バッファの設定
	req_buf->info.length = ES_DEV_COMMAND_INFO_REQ_LEN;

	//$DD 応答バッファの設定
	res_buf->info.length = 0;
	res_buf->info.code = 0;

	ES_DRV_LOGGING("%s reqbuf length=%d,type=%d",
		__func__,req_buf->info.length,
		ES_CMD_TYPE_INFO);
	//$DD コマンドを設定してesvmm_command_requestを実行
	ret = esvmm_command_request(base, ES_CMD_TYPE_INFO);

	result = 0;
	ES_DRV_LOGGING("%s resbuf ret=%d,length=%d,code=%d ES info(status,zone)=(%d,%d),(%d,%d),(%d,%d),(%d,%d),(%d,%d),(%d,%d),(%d,%d),(%d,%d)",
		__func__,ret,res_buf->info.length,res_buf->info.code,
		res_buf->info.info[0].es_status,res_buf->info.info[0].zone,
		res_buf->info.info[1].es_status,res_buf->info.info[1].zone,
		res_buf->info.info[2].es_status,res_buf->info.info[2].zone,
		res_buf->info.info[3].es_status,res_buf->info.info[3].zone,
		res_buf->info.info[4].es_status,res_buf->info.info[4].zone,
		res_buf->info.info[5].es_status,res_buf->info.info[5].zone,
		res_buf->info.info[6].es_status,res_buf->info.info[6].zone,
		res_buf->info.info[7].es_status,res_buf->info.info[7].zone);

	//$DD 正常な応答バッファか長さを確認
	if (ES_DEV_COMMAND_INFO_RES_LEN((*(ptrs->epnum_max))) !=
	    res_buf->info.length) {
		//$DD インターフェイスエラー(ES_DRV_INTERNAL_ERR_LENGTH3 = 10)
		pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME, ES_DRV_INTERNAL_ERR_LENGTH3);
	    	
		ES_DRV_LOGGING("%s NotReady=%d,length=%d",
			__func__,ES_DRV_INTERNAL_ERR_LENGTH3,res_buf->info.length);
	    	
		result = -ENOMSG;
	} else if (ret == ES_CMD_STATUS_NORMAL) {
		//$DD esvmm_command_requestの結果を評価し戻り値に変換する、正常0、タイムアウトEBUSY、それ以外EPERM
		PDEBUG(PD_INF, "cmd ok,code:%d", res_buf->info.code);
		switch (res_buf->info.code) {
		case ES_COMMAND_REQ_RES_CODE_NORMAL:
			result = 0;
			break;
		default:
			result = -EPERM;
			break;
		}
	} else {
		PDEBUG(PD_INF, "cmd ng,ret:%d", ret);
		switch (ret) {
		case ES_CMD_STATUS_UNKNOWN:
			result = -EPERM;
			break;
		case ES_CMD_STATUS_TIMEOUT:
			//$DD 未Completeタイムアウト(ES_DRV_INTERNAL_ERR_CMD_TIMEO3 = 17)
			pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME,
			       ES_DRV_INTERNAL_ERR_CMD_TIMEO3);
			
			ES_DRV_LOGGING("%s NotReady=%d,ret=%d",
				__func__,ES_DRV_INTERNAL_ERR_CMD_TIMEO3,ret);
			
			result = -EBUSY;
			break;
		case ES_CMD_STATUS_ERROR_PARAM:
			result = -EPERM;
			break;
		case ES_CMD_STATUS_ERROR_STATUS:
			result = -EPERM;
			break;
		default:
			result = -EPERM;
			break;
		}
	}

	PDEBUG(PD_INF, "result:%d", result);
	return result;		//正常終了
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．５．１９ バッファアドレスを登録

  @param[in,out] base		ESデバイス レジスタベースアドレス
  @param[in]     dest_epid	接続先EPID
  @param[in]     bufpair	EPバッファペアのアドレス
  @param[out]    ptrs		各領域(要求バッファ,応答バッファ,共有ステータス)のアドレス

  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esvmm_register_buff_addr(es_device_register * base,
					 epno_t dest_epid,
					 struct ep_share_mem_info *bufPair,
					 struct share_pointers_s *ptrs)
{
	union es_device_command_req *req_buf = ptrs->req_buf;
	union es_device_command_res *res_buf = ptrs->res_buf;
	struct es_device_shared_info *aShare = ptrs->share;
	enum es_dev_command_response_e ret;
	int idx, i;
	int page_count;
	int result;
	void *addr;
	int timeout;

	PDEBUG(PD_ENT, "ENTER");

	//$DD 共有済みかチェック
	if (test_bit(dest_epid, &ptrs->buffer_share_bit)) {
		PDEBUG(PD_INF
		       || PD_STATUS, "(EPID %d)buffer shared:skip", dest_epid);
		//$DD 共有してる場合、無処理で終了する
		result = 0;
		return result;
	}
	//$DD 要求バッファのクリア
	memset(req_buf, 0, ptrs->req_buf_size);
	//$DD 応答バッファのクリア
	memset(res_buf, 0, ptrs->res_buf_size);

	//$DD 要求バッファの設定
	req_buf->share_buffer.length =
	    ES_DEV_COMMAND_SHARE_BUFFER_REQ_LEN(bufPair->tx.size,
						bufPair->rx.size);
	req_buf->share_buffer.epid = dest_epid;

	idx = 0;
	//$DD まず、送信バッファのサイズを格納

	req_buf->share_buffer.buffer[idx++] = bufPair->tx.size;

	page_count = bufPair->tx.size / MINIMUM_PAGE_SIZE;
	//$DD 次に、送信バッファの物理アドレス(4K単位)を順次格納
	for (i = 0; i < page_count; i++) {
		addr = ((u8 *) (bufPair->tx.buffer)) + (i * MINIMUM_PAGE_SIZE);
		req_buf->share_buffer.buffer[idx++] =
		    (__le64) (page_to_phys(vmalloc_to_page(addr)) +
			      offset_in_page(addr));
	}

	//$DD 同じく、受信バッファのサイズを格納
	req_buf->share_buffer.buffer[idx++] = bufPair->rx.size;

	page_count = bufPair->rx.size / MINIMUM_PAGE_SIZE;

	//$DD 同様に受信バッファの物理アドレス(4K単位)を順次格納
	for (i = 0; i < page_count; i++) {
		addr = ((u8 *) (bufPair->rx.buffer)) + (i * MINIMUM_PAGE_SIZE);
		req_buf->share_buffer.buffer[idx++] =
		    (__le64) (page_to_phys(vmalloc_to_page(addr)) +
			      offset_in_page(addr));
	}

	//$DD 応答バッファの設定
	res_buf->share_buffer.length = 0;
	res_buf->share_buffer.code = 0;

	{
		void *tx,*rx;
		phys_addr_t tx_addr,rx_addr;
		tx = (void *)bufPair->tx.buffer;
		rx = (void *)bufPair->rx.buffer;
		tx_addr = (phys_addr_t) (page_to_phys(vmalloc_to_page(tx)) + offset_in_page(tx));
		rx_addr = (phys_addr_t) (page_to_phys(vmalloc_to_page(rx)) + offset_in_page(rx));
		ES_DRV_LOGGING("%s reqbuf length=%d,epid=%d tx size=%zu,phy addr=%p rx size=%zu,phy addr=%p,type=%d",
			__func__,req_buf->share_buffer.length,req_buf->share_buffer.epid,
			bufPair->tx.size,(void *)tx_addr,bufPair->rx.size,(void *)rx_addr,
			ES_CMD_TYPE_SHARE_BUFFER);
	}
	//$DD コマンドを設定してesvmm_command_requestを実行
	ret = esvmm_command_request(base, ES_CMD_TYPE_SHARE_BUFFER);

	//$DD バッファ登録/削除のみ応答コードがBUSYでさらに独自にリトライを行う
	timeout = 0;
	while ((ret == ES_CMD_STATUS_NORMAL)
	       && (ES_DEV_COMMAND_SHARE_BUFFER_RES_LEN ==
		   res_buf->share_buffer.length)
	       && (ES_COMMAND_REQ_RES_CODE_BUSY == res_buf->share_buffer.code)
	       && (timeout < ES_COMMAND_REQ_BUFF_TIMEOUT_MAX_MSEC)) {
		msleep(ES_COMMAND_REQ_BUFF_TIMEOUT_EP_UNIT_MSEC
		       ((*(ptrs->epid_self))));
		timeout +=
		    ES_COMMAND_REQ_BUFF_TIMEOUT_EP_UNIT_MSEC((*
							      (ptrs->
							       epid_self)));

		//$DD 応答バッファの設定
		res_buf->share_buffer.length = 0;
		res_buf->share_buffer.code = 0;
		//$DD コマンドを設定してesvmm_command_requestを実行
		ret = esvmm_command_request(base, ES_CMD_TYPE_SHARE_BUFFER);
	}

	result = 0;
	ES_DRV_LOGGING("%s resbuf ret=%d,length=%d,code=%d,timeout=%d",
		__func__,ret,res_buf->share_buffer.length,res_buf->share_buffer.code,timeout);

	//$DD 正常な応答バッファか長さを確認
	if (ES_DEV_COMMAND_SHARE_BUFFER_RES_LEN != res_buf->share_buffer.length) {
		//$DD インターフェイスエラー(ES_DRV_INTERNAL_ERR_LENGTH4 = 11)
		pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME, ES_DRV_INTERNAL_ERR_LENGTH4);
		
		ES_DRV_LOGGING("%s NotReady=%d,length=%d",
			__func__,ES_DRV_INTERNAL_ERR_LENGTH4,res_buf->start_trace.length);
		
		result = -ENOMSG;
	} else if (ret == ES_CMD_STATUS_NORMAL) {
		//$DD esvmm_command_requestの結果を評価し戻り値に変換する、正常0、タイムアウトかビジーでEBUSY、それ以外EPERM
		PDEBUG(PD_INF, "cmd ok,code:%d", res_buf->share_buffer.code);
		switch (res_buf->share_buffer.code) {
		case ES_COMMAND_REQ_RES_CODE_NORMAL:
			result = 0;
			//$DD 成功の場合、共有を設定
			set_bit(dest_epid, &ptrs->buffer_share_bit);
			PDEBUG(PD_INF
			       || PD_STATUS, "(EPID %d)buffer shared:success",
			       dest_epid);
			break;
		case ES_COMMAND_REQ_RES_CODE_BUSY:
			//$DD BUSY継続タイムアウト(ES_DRV_INTERNAL_ERR_CMD_BUSY1 = 13)
			pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME,
			       ES_DRV_INTERNAL_ERR_CMD_BUSY1);
			
			ES_DRV_LOGGING("%s NotReady=%d,ret=%d",
				__func__,ES_DRV_INTERNAL_ERR_CMD_BUSY1,ret);
			
			result = -EBUSY;
			break;
		default:
			result = -EPERM;
			break;
		}
	} else {
		PDEBUG(PD_INF, "cmd ng,ret:%d", ret);
		switch (ret) {
		case ES_CMD_STATUS_UNKNOWN:
			result = -EPERM;
			break;
		case ES_CMD_STATUS_TIMEOUT:
			//$DD 未Completeタイムアウト(ES_DRV_INTERNAL_ERR_CMD_TIMEO4 = 18)
			pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME,
			       ES_DRV_INTERNAL_ERR_CMD_TIMEO4);
			
			ES_DRV_LOGGING("%s NotReady=%d,ret=%d",
				__func__,ES_DRV_INTERNAL_ERR_CMD_TIMEO4,ret);
			
			result = -EBUSY;
			break;
		case ES_CMD_STATUS_ERROR_PARAM:
			result = -EPERM;
			break;
		case ES_CMD_STATUS_ERROR_STATUS:
			result = -EPERM;
			break;
		default:
			result = -EPERM;
			break;
		}
	}

	PDEBUG(PD_INF || PD_STATUS
	       || PD_BUFFER_REG, "result:%d (EPID:%d,status:%d)", result,
	       dest_epid, esmem_is_shared(aShare, dest_epid));
	return result;		//正常終了
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．５．２０ バッファアドレスの登録を解除

  @param[in,out] base		ESデバイス レジスタベースアドレス
  @param[in]     dest_epid	接続先EPID
  @param[out]    ptrs		各領域(要求バッファ,応答バッファ,共有ステータス)のアドレス

  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esvmm_unregister_buff_addr(es_device_register * base,
					   epno_t dest_epid,
					   struct share_pointers_s *ptrs)
{
	int result;
	int timeout;

	PDEBUG(PD_ENT, "ENTER");

	//$DD ポインタNULLチェック(1)
	if (NULL == base || NULL == ptrs)
		return -EPERM;
	else {
		union es_device_command_req *req_buf = ptrs->req_buf;
		union es_device_command_res *res_buf = ptrs->res_buf;
		struct es_device_shared_info *aShare = ptrs->share;
		enum es_dev_command_response_e ret;

		//$DD ポインタNULLチェック(2)
		if (NULL == req_buf || NULL == res_buf || NULL == aShare)
			return -EPERM;

		//$DD 共有済みかチェック
		if (!test_bit(dest_epid, &ptrs->buffer_share_bit)) {
			PDEBUG(PD_INF
			       || PD_STATUS, "(EPID %d)buffer unshared:skip",
			       dest_epid);
			//$DD 共有してない場合、無処理で終了する
			result = 0;
			return result;
		}
		//$DD 要求バッファのクリア
		memset(req_buf, 0, ptrs->req_buf_size);
		//$DD 応答バッファのクリア
		memset(res_buf, 0, ptrs->res_buf_size);

		//$DD 要求バッファの設定
		req_buf->unshare_buffer.length =
		    ES_DEV_COMMAND_UNSHARE_BUFFER_REQ_LEN;
		req_buf->unshare_buffer.epid = dest_epid;

		//$DD 応答バッファの設定
		res_buf->unshare_buffer.length = 0;
		res_buf->unshare_buffer.code = 0;

		ES_DRV_LOGGING("%s reqbuf length=%d,epid=%d,type=%d",
			__func__,req_buf->unshare_buffer.length,req_buf->unshare_buffer.epid,
			ES_CMD_TYPE_UNSHARE_BUFFER);
		//$DD コマンドを設定してesvmm_command_requestを実行
		ret = esvmm_command_request(base, ES_CMD_TYPE_UNSHARE_BUFFER);

		//$DD バッファ登録/削除のみ応答コードがBUSYでさらに独自にリトライを行う
		timeout = 0;
		while ((ret == ES_CMD_STATUS_NORMAL)
		       && (ES_DEV_COMMAND_UNSHARE_BUFFER_RES_LEN ==
			   res_buf->unshare_buffer.length)
		       && (ES_COMMAND_REQ_RES_CODE_BUSY ==
			   res_buf->unshare_buffer.code)
		       && (timeout < ES_COMMAND_REQ_BUFF_TIMEOUT_MAX_MSEC)) {
			msleep(ES_COMMAND_REQ_BUFF_TIMEOUT_EP_UNIT_MSEC
			       ((*(ptrs->epid_self))));
			timeout +=
			    ES_COMMAND_REQ_BUFF_TIMEOUT_EP_UNIT_MSEC((*
								      (ptrs->
								       epid_self)));

			//$DD 応答バッファの設定
			res_buf->unshare_buffer.length = 0;
			res_buf->unshare_buffer.code = 0;
			//$DD コマンドを設定してesvmm_command_requestを実行
			ret =
			    esvmm_command_request(base,
						  ES_CMD_TYPE_UNSHARE_BUFFER);
		}

		result = 0;
		ES_DRV_LOGGING("%s resbuf ret=%d,length=%d,code=%d,timeout=%d",
			__func__,ret,res_buf->unshare_buffer.length,res_buf->unshare_buffer.code,timeout);

		//$DD 正常な応答バッファか長さを確認
		if (ES_DEV_COMMAND_UNSHARE_BUFFER_RES_LEN !=
		    res_buf->unshare_buffer.length) {
			//$DD インターフェイスエラー(ES_DRV_INTERNAL_ERR_LENGTH5 = 12)
			pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME,
			       ES_DRV_INTERNAL_ERR_LENGTH5);

			ES_DRV_LOGGING("%s NotReady=%d,length=%d",
				__func__,ES_DRV_INTERNAL_ERR_LENGTH5,res_buf->unshare_buffer.length);

		    	result = -ENOMSG;
		} else if (ret == ES_CMD_STATUS_NORMAL) {
			//$DD esvmm_command_requestの結果を評価し戻り値に変換する、正常0、タイムアウトかビジーでEBUSY、それ以外EPERM
			PDEBUG(PD_INF, "cmd ok,code:%d",
			       res_buf->unshare_buffer.code);
			switch (res_buf->unshare_buffer.code) {
			case ES_COMMAND_REQ_RES_CODE_NORMAL:
				result = 0;
				//$DD 成功の場合、共有を解除
				clear_bit(dest_epid, &ptrs->buffer_share_bit);
				PDEBUG(PD_INF
				       || PD_STATUS,
				       "(EPID %d)buffer unshared:success",
				       dest_epid);
				break;
			case ES_COMMAND_REQ_RES_CODE_BUSY:
				//$DD BUSY継続タイムアウト(ES_DRV_INTERNAL_ERR_CMD_BUSY2 = 14)
				pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME,
				       ES_DRV_INTERNAL_ERR_CMD_BUSY2);
				
				ES_DRV_LOGGING("%s NotReady=%d,ret=%d",
					__func__,ES_DRV_INTERNAL_ERR_CMD_BUSY2,ret);
				
				result = -EBUSY;
				break;
			default:
				result = -EPERM;
				break;
			}
		} else {
			PDEBUG(PD_INF, "cmd ng,ret:%d", ret);
			switch (ret) {
			case ES_CMD_STATUS_UNKNOWN:
				result = -EPERM;
				break;
			case ES_CMD_STATUS_TIMEOUT:
				//$DD 未Completeタイムアウト(ES_DRV_INTERNAL_ERR_CMD_TIMEO5 = 19)
				pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME,
				       ES_DRV_INTERNAL_ERR_CMD_TIMEO5);

				ES_DRV_LOGGING("%s NotReady=%d,ret=%d",
					__func__,ES_DRV_INTERNAL_ERR_CMD_TIMEO5,ret);
				
				result = -EBUSY;
				break;
			case ES_CMD_STATUS_ERROR_PARAM:
				result = -EPERM;
				break;
			case ES_CMD_STATUS_ERROR_STATUS:
				result = -EPERM;
				break;
			default:
				result = -EPERM;
				break;
			}
		}

		PDEBUG(PD_INF || PD_STATUS
		       || PD_BUFFER_REG, "result:%d (EPID:%d,status:%d)",
		       result, dest_epid, esmem_is_shared(aShare, dest_epid));
	}

	return result;		//正常終了
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．５．２１ 他EPへ任意の割り込みを発生する

  @param[in,out] base		ESデバイス レジスタベースアドレス
  @param[in]     dest_epid	送信先EPID
  @param[in]     intr_mask	割り込み要因

  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esvmm_raise_interrupt(es_device_register * base,
				      epno_t dest_epid,
				      enum INTREG_IRQMASK intr_mask)
{
	//$DD 割り込みフラグの設定、およびEP IDの設定を行なう
	intreg_t ig = intr_mask | dest_epid;

	PDEBUG(PD_ENT, "ENTER dest epid=%d int=%x", dest_epid, intr_mask);

	esvmm_writel((__le32) ig, ((void *)(base + XSCT_REG_IG)));

	return 0;		//正常終了
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．５．２２ 割り込み結果をキャプチャする
	
  割り込み結果は自動でリードクリアされる
  @note
  現在の割り込みの状態を確認する前に必ず実施する
  呼出し元では割り込みが有効かどうかINTREG_MASK_IS_ASSERTビットをチェックすること

  @param[in,out] base	ESデバイス レジスタベースアドレス

  @return  割り込み発生(発生要因,発生元EPIDを各ビットに設定)
**/
FUNC_STATIC intreg_t esvmm_interrupt_capture(es_device_register * base)
{
	//$DD 割り込み状態をISから読み出して返す
	intreg_t cur_is;

	PDEBUG(PD_ENT, "ENTER");
	cur_is = esvmm_readl((void *)(base + XSCT_REG_IS));
	return cur_is;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．５．２３ 他EPからの割り込みのマスク設定
	

  @param[in,out] base		ESデバイス レジスタベースアドレス
  @param[in]     intr_mask	割り込み要因
  @param[in]     mask		マスク状態 true:割り込み禁止 false:割り込み許可
**/
FUNC_STATIC void esvmm_irqmask(es_device_register * base,
			       enum INTREG_IRQMASK intr_mask, bool mask)
{
	PDEBUG(PD_ENT, "ENTER msak=%d,value=%08x", mask, intr_mask);

	//$DD マスクの方向で設定種別を変える
	if (mask) {
		//$DD マスクを有効にするなら IMSへ値書き込み
		PDEBUG(PD_INF, "irq mask");
		esvmm_writel(intr_mask, ((void *)(base + XSCT_REG_IMS)));
	} else {
		//$DD マスクを無効にするなら IMCへ値書き込み
		PDEBUG(PD_INF, "irq unmask");
		esvmm_writel(intr_mask, ((void *)(base + XSCT_REG_IMC)));
	}
	PDEBUG(PD_INF, "END");
	return;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．５．２４ 割り込み時の自動マスクの設定変更
	
	欠番
	
	現状では欠番として詳細には記載すること

  @param[in,out] base	VMMレジスタベースアドレス
  @param[in]     mask	マスク状態
**/
#if 0				/* 現状不要になった */
FUNC_STATIC void esvmm_automask_set(es_device_register * base, bool mask)
{
	ICTL_REG automask;

	PDEBUG(PD_ENT, "ENTER");
	automask.Reg = 0;
	automask.Bits.automask = mask;

	//$DD ICTLレジスタにautomask状態を設定する
	esvmm_writel(automask.Reg, ((void *)(base + XSCT_REG_ICTL)));
	return;
}
#endif				/* 0 */

//$DD=============================================================================
//$DD ２．４ メモリ管理

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．４．１ EPバッファのサイズ取得

	欠番
	
	現状では欠番として詳細には記載すること
	
  @brief
  送信または受信用の１EPのバッファサイズを取得する

  @return  送信または受信用の１EPのバッファサイズ
**/
#if 0				/* 現状不要になった */
FUNC_STATIC size_t esmem_epbuf_size(void)
{
	PDEBUG(PD_ENT, "ENTER");
	//$DD 定義されている固定長を返す
	return EP_BUFFER_SIZE;
}
#endif				/* 0 */

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．４．２ EPバッファの確保・初期化

  @brief
  送信または受信用の１EPのバッファを取得する

  @param[in,out] epbh	EPバッファハンドラ

  @retval  0			正常終了
  @retval  負の値		<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esmem_epbuf_alloc(struct epbuf_handler *epbh)
{
	memptr_t mem;

	PDEBUG(PD_ENT, "ENTER");

	//$DD vmallocで連続リニアアドレスのバッファを獲得する、サイズはEP_BUFFER_SIZE
	mem = vmalloc(EP_BUFFER_SIZE);

	//$DD 確保に失敗したらENOMEMで終了
	if (mem == NULL) {
		PDEBUG(PD_INF, "no alloc");
		return -ENOMEM;
	}
	//$DD バッファの0初期化
	memset(mem, 0, EP_BUFFER_SIZE);

	//$DD 引数で渡されているハンドラへ設定
	epbh->buffer = mem;
	epbh->size = EP_BUFFER_SIZE;

	//$DD その他、構造体に沿ったアドレスを設定
	epbh->info = (union ep_buffer_info *)mem;
	epbh->ring = (u8 *) (mem + sizeof(union ep_buffer_info));

	PDEBUG(PD_INF, "vmalloc:%zu", epbh->size);

	PDEBUG(PD_COMMON, "epbuf alloc: virt addr info:%p", epbh->info);
	PDEBUG(PD_COMMON, "epbuf alloc: virt addr ring:%p", epbh->ring);

	//$DD 正常終了で0を返す
	return 0;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．４．３ EPバッファの解放

  @brief
  送信または受信用の１EPのバッファを解放する

  @param[in,out] epbh	EPバッファハンドラ
**/
FUNC_STATIC void esmem_epbuf_free(struct epbuf_handler *epbh)
{
	PDEBUG(PD_ENT, "ENTER");

	//$DD NULLでなければvfreeで開放
	if (NULL != epbh->buffer) {
		PDEBUG(PD_INF, "free memory");
		vfree(epbh->buffer);
	}
	//$DD ハンドラにある各種ポインタ、数値を初期化
	epbh->buffer = NULL;
	epbh->size = 0;

	epbh->info = NULL;
	epbh->ring = NULL;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．４．４ EPバッファのセットアップ

  @brief
  EPバッファの初期値を設定する
  rx_statusの変更があるので、rx_status_lockでの排他が設定されていること

  @param[in,out] epbh		EPバッファハンドラ(NULLなし、memset(0)済み)
  @param[in]     mac_addr	MACアドレス
  @param[in]     mtu		MTU値
**/
FUNC_STATIC void esmem_epbuf_setup(struct epbuf_handler *epbh, u8 * mac_addr,
				   u32 mtu)
{

	union ep_buffer_info *info = epbh->info;
	int i;
	u16 vlan_id[EP_BUFFER_SUPPORT_VLAN_MAX];	///<  VLAN ID 1～4

	PDEBUG(PD_ENT, "ENTER");

	//$DD VLAN IDをリザーブ
	for (i = 0; i < EP_BUFFER_SUPPORT_VLAN_MAX; i++) {
		vlan_id[i] = info->v1i.vlan_id[i];
	}

	//$DD 初期化
	// info_sizeで0クリアしたほうがよいかもしれない
	memset((void *)info, 0, sizeof(union ep_buffer_info));

	//$DD バージョンを設定
	info->v1i.version = ES_DRV_MAJOR_NO;

	//$DD MACアドレスを設定
	for (i = 0; i < ETH_ALEN; i++) {
		info->v1i.mac_addr[i] = mac_addr[i];
	}
	PDEBUG(PD_COMMON, "epbuf setup: mac addr:%02x:%02x:%02x:%02x:%02x:%02x",
	       info->v1i.mac_addr[0],
	       info->v1i.mac_addr[1],
	       info->v1i.mac_addr[2],
	       info->v1i.mac_addr[3],
	       info->v1i.mac_addr[4], info->v1i.mac_addr[5]
	    );

	//$DD バッファ管理のHead,Tailの初期値を設定
	info->v1i.head = EP_RING_INITIAL_HEAD;
	info->v1i.tail = EP_RING_INITIAL_TAIL;

	//$DD 管理ページサイズ、バッファサイズ、フレームサイズ、フレーム最大数を登録
	info->v1i.info_size = sizeof(union ep_buffer_info);
	info->v1i.buffer_size = epbh->size - info->v1i.info_size;

	info->v1i.frame_max = ES_MTU_TO_FRAME_SIZE(mtu);
	info->v1i.count_max =
	    EP_RING_NUM(info->v1i.buffer_size, info->v1i.frame_max);

	//$DD VLAN ID領域を復元
	for (i = 0; i < EP_BUFFER_SUPPORT_VLAN_MAX; i++) {
		info->v1i.vlan_id[i] = vlan_id[i];
	}

	//$DD rx_statusに初期値を設定(ES_RX_MTU_CHANGING_DONEのbitは1が初期値)
	info->v1i.rx_status |= ES_RX_MTU_CHANGING_DONE;
	
	PDEBUG(PD_COMMON, "epbuf setup: mtu:%d", mtu);
	PDEBUG(PD_COMMON, "epbuf setup: info size:%d", info->v1i.info_size);
	PDEBUG(PD_COMMON, "epbuf setup: buf size:%d", info->v1i.buffer_size);
	PDEBUG(PD_COMMON, "epbuf setup: frame_max:%d", info->v1i.frame_max);
	PDEBUG(PD_COMMON, "epbuf setup: count_max:%d", info->v1i.count_max);

}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．４．５ 共有ステータス領域の確保

  @brief
  共有ステータス領域用のバッファを取得する(kmallocもしくはalloc_pagesによる連続物理アドレスが必須)

  @param[out]    xdev	es_device情報の格納先

  @retval  0			正常終了
  @retval  負の値		<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esmem_share_alloc(struct esif_device *xdev)
{
	size_t size;
	PDEBUG(PD_ENT, "ENTER");

	//$DD 連続物理のアドレスのバッファを獲得し、０クリアする
	size =
	    sizeof(struct es_device_shared_info) +
	    (sizeof(u8) * xdev->epnum_max);
	xdev->share_ptrs.share =
	    (struct es_device_shared_info *)kzalloc(size, GFP_KERNEL);

	//$DD 確保に失敗したらENOMEMで終了
	if (xdev->share_ptrs.share == NULL) {
		PDEBUG(PD_INF, "no alloc");
		return -ENOMEM;
	}
	//$DD 引数で渡されているハンドラへ設定
	xdev->share_ptrs.share->epnum = xdev->epnum_max;

	PDEBUG(PD_INF, "kzalloc:%zu", size);

	//$DD 正常終了で0を返す
	return 0;

}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．４．６ 共有ステータス領域の解放

  @brief
  共有ステータス領域用のバッファを解放する

  @param[out]    xdev	es_device情報の格納先
**/
FUNC_STATIC void esmem_share_free(struct esif_device *xdev)
{
	PDEBUG(PD_ENT, "ENTER");

	//$DD NULLでなければkfreeで開放
	if (NULL != xdev->share_ptrs.share) {
		PDEBUG(PD_INF, "free memory");
		kfree(xdev->share_ptrs.share);
	}
	xdev->share_ptrs.share = NULL;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．４．７ 共有状態の取得

  @brief
  共有ステータス領域用の情報を取得する

  @param[in]     aShare		共有ステータス領域
  @param[in]     dest_epid	確認対象のEPID

  @retval  false	共有中ではない
  @retval  true		共有中
**/
FUNC_STATIC int esmem_is_shared(struct es_device_shared_info *aShare,
				epno_t dest_epid)
{
	int value = false;
	PDEBUG(PD_ENT, "ENTER");

	//$DD 最大EP未満であることを確認
	if (dest_epid < aShare->epnum) {
		//$DD 共有ステータス領域用の情報を取得する
		value = aShare->ep_status[dest_epid];
		PDEBUG(PD_COMMON, "ENTER status=%d", value);
	}
	//$DD ステータスを返す
	return value;
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．４．８ Zoneのチェック

  管理情報にあるZoneが引数のEP IDのZoneと等しいか返す
  この判定ではESが有効かの確認も含んでいる
  この情報は事前にrequestInfoで取得、priv->xsdevice.ep_shm_infoに保存しているものとする。
	
  @param[in]	xdev		es_device情報の格納先
  @param[in]	epid		確認対象のEP ID
	
  @retval	true		等しい
  @retval	false		等しくないもしくは利用できない
**/
FUNC_STATIC bool esmem_same_zone(struct esif_device * xdev, epno_t epid)
{
	PDEBUG(PD_ENT, "ENTER");

#ifndef ESDRV_ZONE_OFF
	//$DD 最大EP未満であることを確認
	if (epid < xdev->epnum_max) {
		//$DD 引数のEP IDが有効でないか自Zoneがnoneならfalse
		if ((xdev->ep_shm_info[epid].es_status !=
		     ES_ZONING_STATUS_ENABLE)
		    || (xdev->ep_shm_info[xdev->epid_self].zone ==
			ES_ZONING_ZONE_TYPE_NONE)
		    ) {
			PDEBUG(PD_INF, "status not enable:%d",
			       xdev->ep_shm_info[epid].es_status);
			return false;
		} else {
			PDEBUG(PD_INF, "self:%d,epid(%d):%d result:%d",
			       xdev->ep_shm_info[xdev->epid_self].zone,
			       epid,
			       xdev->ep_shm_info[epid].zone,
			       (xdev->ep_shm_info[epid].zone ==
				xdev->ep_shm_info[xdev->epid_self].zone));
			//$DD 引数のEP IDが有効なら、自EPのzoneと等しいかを返す
			return (xdev->ep_shm_info[epid].zone ==
				xdev->ep_shm_info[xdev->epid_self].zone);
		}
	}

	PDEBUG(PD_INF, "epid invalid:%d", epid);
	//$DD 最大EPIDより上だったらfalse
	return false;
#else				/* !ESDRV_ZONE_OFF */
	// 未実装状態だと無条件true
	return true;
#endif				/* !ESDRV_ZONE_OFF */
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．４．９ MTUのチェック

	受信バッファが渡され、その管理情報にあるMTU値が引数と同じか確認する
	確認には受信バッファ(相手)のframe_maxとES_MTU_TO_FRAME_SIZEで変換したMTUとで比較を行う
	
  @param[in]	epbh		EPバッファハンドラ(RX側のみ渡される)
  @param[in]	mtu		確認対象のMTU
	
  @retval	true		設定されている
  @retval	false		設定されていない
**/
FUNC_STATIC bool esmem_same_mtu(struct epbuf_handler * epbh, u32 mtu)
{
	union ep_buffer_info *info = epbh->info;
	int i;
	PDEBUG(PD_ENT, "ENTER");

	//$DD 以下のANDを返す
	//$DD 引数のepbuf_handlerのfrmame_maxとMTUをフレームサイズに変換したものの比較結果
	//$DD 対向のrx_statusでMTU変更中でないこと
	return ((info->v1i.frame_max == ES_MTU_TO_FRAME_SIZE(mtu)) && (ES_RX_MTU_CHANGING_DONE & info->v1i.rx_status));
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．４．１０ バージョンのチェック

	受信バッファが渡され、その管理情報にあるバージョンが引数と同じか確認する
	
  @param[in]	epbh	EPバッファハンドラ
  @param[in]	ver		確認対象のバージョン値
	
  @retval	true		設定されている
  @retval	false		設定されていない
**/
FUNC_STATIC bool esmem_same_ver(struct epbuf_handler * epbh, u32 ver)
{
	union ep_buffer_info *info = epbh->info;
	int i;
	PDEBUG(PD_ENT, "ENTER");

	//$DD バッファハンドラ内のversionと比較した結果を返す
	return (ver == info->common.version);
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．４．１１ VLAN IDのチェック

	受信バッファが渡され、その管理情報にVLAN IDが含まれているか確認する
	VLAN_IDの妥当性は上位で確認ずみとする
	
  @param[in]	epbh		EPバッファハンドラ
  @param[in]	vlan_id	確認対象のVLAN ID
	
  @retval	true		設定されている
  @retval	false		設定されていない
**/
FUNC_STATIC bool esmem_has_vid(struct epbuf_handler * epbh, u16 vlan_id)
{
	union ep_buffer_info *info = epbh->info;
	int i;
	PDEBUG(PD_ENT, "ENTER");

	//$DD VLAN ID=0で呼び出された場合はtrue
	if (0 == vlan_id) {
		PDEBUG(PD_VLAN, "vid 0 always true");
		return true;
	} else {
		//$DD それ以外についてチェック
		//$DD バッファハンドラ内のvlan_idを順次チェックする
		for (i = 0; i < EP_BUFFER_SUPPORT_VLAN_MAX; i++) {
			PDEBUG(PD_COMMON, "match check vid:%d,buffer_saved:%d",
			       vlan_id, info->v1i.vlan_id[i]);
			if (vlan_id == info->v1i.vlan_id[i]) {
				PDEBUG(PD_VLAN, "match found vlan:%d in %d",
				       vlan_id, i);
				//$DD 同値があったらtrue
				return true;
			}
		}
	}

	PDEBUG(PD_VLAN, "match not found:%d", vlan_id);
	//$DD 同値が見つからなかったらfalse
	return false;
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．４．１２ VLAN IDの追加

	送信バッファが渡され、その管理情報にVLAN IDを格納
	VLAN_IDの妥当性は上位で確認ずみとする
	
	同値の多重登録防止のため、先にhas_vidで存在しないことを確かめること
	
  @param[in]	epbh		EPバッファハンドラ(TX側のみ渡される)
  @param[in]	vlan_id	確認対象のVLAN ID
	
  @retval	true		格納できた
  @retval	false		格納できなかった
**/
FUNC_STATIC bool esmem_set_vid(struct epbuf_handler * epbh, u16 vlan_id)
{
	union ep_buffer_info *info = epbh->info;
	int i;
	PDEBUG(PD_ENT, "ENTER");

	//$DD バッファハンドラ内のvlan_idを順次チェックする
	for (i = 0; i < EP_BUFFER_SUPPORT_VLAN_MAX; i++) {
		if (0 == info->v1i.vlan_id[i]) {
			PDEBUG(PD_VLAN, "set VID:%d to buff:%d", vlan_id, i);
			//$DD 空きがあったら格納してtrueを返す
			info->v1i.vlan_id[i] = vlan_id;
			return true;
		}
	}

	PDEBUG(PD_VLAN, "not set:%d", vlan_id);
	//$DD 空きがなく格納できなかったらfalse
	return false;
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．４．１３ VLAN IDの削除

	送信バッファが渡され、その管理情報からVLAN IDを削除
	VLAN_IDの妥当性は上位で確認ずみとする
	
  @param[in]	epbh		EPバッファハンドラ(TX側のみ渡される)
  @param[in]	vlan_id	確認対象のVLAN ID
**/
FUNC_STATIC void esmem_del_vid(struct epbuf_handler *epbh, u16 vlan_id)
{
	union ep_buffer_info *info = epbh->info;
	int i;
	PDEBUG(PD_ENT, "ENTER");

	//$DD VLAN ID=0で呼び出された場合は無処理
	if (0 != vlan_id) {
		PDEBUG(PD_VLAN, "del vid %d", vlan_id);
		//$DD バッファハンドラ内のvlan_idを順次チェックする
		for (i = 0; i < EP_BUFFER_SUPPORT_VLAN_MAX; i++) {
			if (vlan_id == info->v1i.vlan_id[i]) {
				PDEBUG(PD_VLAN, "remove VID:%d from buff:%d",
				       vlan_id, i);
				//$DD 同値があったら0を設定
				//$DD 複数の存在のため、処理は継続させる
				info->v1i.vlan_id[i] = 0;

			}
		}
	}
	return;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．４．１４ EP受信バッファのデータの有無を確認

  @param[in,out] epbh	EPバッファハンドラ

  @retval  true		データ無し
  @retval  false	データ有り
**/
FUNC_STATIC bool esmem_epbuf_rx_is_empty(struct epbuf_handler * epbh)
{
	union ep_buffer_info *info;
	static bool log_output = false;

	info = epbh->info;

	//$DD MTU変更中を検出したら、empty
	if (!(info->v1i.rx_status & ES_RX_MTU_CHANGING_DONE)) {
		return true;
	}
	
	//$DD 0除算対策でcount_max確認
	if (info->v1i.count_max == 0) {
		//$DD 成功で返す(ES_DRV_INTERNAL_ERR_RX_0DIV = 5)
		if (false == log_output) {
			pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME,
			       ES_DRV_INTERNAL_ERR_RX_0DIV);
			
			ES_DRV_LOGGING("%s NotReady=%d",
				__func__,ES_DRV_INTERNAL_ERR_RX_0DIV);
			
			log_output = true;
		}
		return true;
	}
	//$DD バッファが空かを確認、空ならばフレームが取得できないとして、trueを返す
	return EP_RING_EMPTY(info->v1i.head, info->v1i.tail,
			     info->v1i.count_max);
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．４．１５ EP受信バッファから1フレーム取得

  @brief
  Headが操作されない、curpkt_dropで進めること
  バッファが空かどうかの確認は行わない。必要に応じesmem_epbuf_rx_is_empty()を実行しておくこと

  @note
  上位が複数動作禁止を保証する

  @param[in,out] epbh	EPバッファハンドラ
  @param[out]    psize	フレームサイズ

  @return  バッファアドレス
  @retval  NULL				読み込みできず もしくは sizeが正しくなかった。
  @retval  引数のフレームと同じ		読み込んだバッファ(引数で渡している)のポインタを返す
**/
FUNC_STATIC framebuf_t esmem_epbuf_rx_curpkt_get_addr(struct epbuf_handler *
						      epbh, size_t * psize)
{
	union ep_buffer_info *info = epbh->info;
	struct esmem_frame_t *ring_frame;
	framebuf_t frame;

	//$DD フレーム構造のポインタを取得 受信側なので、headを起点とする
	ring_frame =
	    (struct esmem_frame_t *)&(epbh->
				      ring[EP_RING_INDEX
					   (info->v1i.head,
					    info->v1i.count_max) *
					   info->v1i.frame_max]);

	//$DD サイズを導出
	*psize = (size_t) ring_frame->frame_size;

	//$DD フレームの先頭ポインタを取得し返す
	frame = (framebuf_t) (ring_frame->frame_data);

	return frame;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．４．１６ EP受信バッファのフレームの完了
	
  @brief
  1フレーム取得完了もしくは破棄でつかう
  位置を1つ進め、Headが操作される
	
  @note
  上位が複数動作禁止を保証する

  @param[in,out] epbh	EPバッファハンドラ
**/
FUNC_STATIC void esmem_epbuf_rx_curpkt_drop(struct epbuf_handler *epbh)
{
	union ep_buffer_info *info = epbh->info;

	//$DD バッファが空かを確認、空ならばフレームが取得できないとして終了
	if (esmem_epbuf_rx_is_empty(epbh))
		return;

	//$DD headを1すすめる
	EP_RING_INDEX_INC(epbh->info->v1i.head, info->v1i.count_max);

	return;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．４．１７ EP送信バッファへの1フレーム設定

  @brief
  送信バッファにフレームを設定、Tailが操作される。

  @note
  上位が複数動作禁止を保証する

  @param[in,out] epbh	EPバッファハンドラ
  @param[in]     frame	フレーム用バッファ
  @param[in]     size	フレームサイズ

  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
  @retval  ENOBUFS	バッファフル
 **/
FUNC_STATIC int esmem_epbuf_tx_pkt_send(struct epbuf_handler *epbh,
					framebuf_t frame, size_t size)
{
	union ep_buffer_info *info = epbh->info;
	struct esmem_frame_t *ring_frame;

	PDEBUG(PD_ENT, "ENTER");

	//$DD バッファがフルかを確認、フルならばフレームを入れることができないとしてENOBUFSを返して終了
	if (EP_RING_FULL(info->v1i.head, info->v1i.tail, info->v1i.count_max)) {
		PDEBUG(PD_COMMON || PD_INF, "full");
		return -ENOBUFS;
	}
	//$DD フレーム構造のポインタを取得 送信側なので、tail-1を起点とする
	ring_frame =
	    (struct esmem_frame_t *)&(epbh->
				      ring[EP_RING_INDEX
					   (info->v1i.tail - 1,
					    info->v1i.count_max) *
					   info->v1i.frame_max]);

	//$DD フレームのサイズ設定およびデータコピー
	ring_frame->frame_size = size;
	memcpy((void *)(ring_frame->frame_data), (void *)frame, size);

	//$DD tailを1すすめる
	EP_RING_INDEX_INC(epbh->info->v1i.tail, info->v1i.count_max);

	//$DD 正常終了で0を返す
	return 0;		//正常終了
}

//$DD=============================================================================
//$DD ２．３ ESインターフェイス
//$DD Extended Socketの動作仕様に強く依存する部分。

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．１ ESデバイスの初期化
	
	モジュールロード時(esdrv_netdev_setup)に呼び出される

  @param[out]    xdev	es_device情報の格納先

  @return  エラーコード
  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esif_init_device(struct esif_device *xdev)
{
	int ret;
	PDEBUG_XDEV(PD_ENT, "ENTER");

	//$DD ACPIからデバイス情報を取得する       esvmm_get_device_config()
	ret = esvmm_get_device_config(xdev);
	if (ret) {
		//$DD 取得に失敗、呼び出し結果のエラーコードで終了する
		return ret;
	}
	//$DD MMIOのマッピングセットアップ  esvmm_setup_address()
	xdev->base = esvmm_setup_address(xdev);
	if (NULL == xdev->base) {	//取得できなかった
		//$DD 取得に失敗、エラーコードEIOで終了する(ES_DRV_INTERNAL_ERR_ADDR_SETUP = 0)
		pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME,
		       ES_DRV_INTERNAL_ERR_ADDR_SETUP);
		ES_DRV_LOGGING("%s NotReady=%d,base=%p",
			__func__,ES_DRV_INTERNAL_ERR_ADDR_SETUP,xdev->base);
		return -EIO;	// エラー番号
	}
	//PDEBUG(PD_COMMON, "adr = %p", xdev->base);

	//$DD デバイスリセット  esvmm_reset_device()
	ret = esvmm_reset_device(xdev->base);
	//$DD ドライバ固有統計(device_reset_executed)更新
	xdev->drv_stats.device_reset_executed++;
	if (ret) {
		//$DD リセット、呼び出し結果のエラーコードで終了する(ES_DRV_INTERNAL_ERR_RESET_NG1 = 1)
		pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME, ES_DRV_INTERNAL_ERR_RESET_NG1);
		ES_DRV_LOGGING("%s NotReady=%d,ret=%d",
			__func__,ES_DRV_INTERNAL_ERR_RESET_NG1,ret);
		return ret;
	}
	//$DD 全割り込み禁止     esvmm_irqmask(true)
	esvmm_irqmask(xdev->base, INTREG_IRQMASK_ALL, true);

	//$DD 最大EP数を取得 esvmm_get_max_epnum()
	xdev->epnum_max = esvmm_get_max_epnum(xdev->base);
	//PDEBUG_XDEV(PD_COMMON,"epnum_max=%d",xdev->epnum_max);

	//$DD 自EPIDを取得 esvmm_get_self_epid()
	xdev->epid_self = esvmm_get_self_epid(xdev->base);
	//PDEBUG_XDEV(PD_COMMON,"epid_self=%d",xdev->epid_self);

	//$DD 最大EPが0(認証エラーで発生しうる)でない、かつ自EPIDが最大EP未満(0 - max-1)であるなら0、そうでないならENXIOを返す
	return ((xdev->epnum_max != 0)
		&& (xdev->epid_self < xdev->epnum_max)) ? 0 : -ENXIO;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．２ ESデバイスの解放

  確保したリソース類の解放を行う
	
  @param[out]    xdev	es_device情報の格納先
**/
FUNC_STATIC void esif_exit_device(struct esif_device *xdev)
{
	int ret;

	PDEBUG(PD_ENT, "ENTER");

	//$DD ベースアドレスの解放
	if (xdev->base) {
		PDEBUG(PD_INF, "exist base");

		// トレースの解放
		// およびトレース停止が必要
		//$DD トレース実行中の場合、停止させる
		if (xdev->trace_start) {
			//$DD トレース停止コマンドを発行する     esvmm_stop_trace()
			mutex_lock(&xdev->share_ptrs.lock);
			ret = esvmm_stop_trace(xdev->base, &xdev->share_ptrs);
			mutex_unlock(&xdev->share_ptrs.lock);
			// ret:処理 現状esif_exit_deviceでの登録削除失敗はケアしない
			// 終了時なので、リカバリなし

			//$DD ドライバ固有統計(command_stop_trace_executed)更新
			xdev->drv_stats.command_stop_trace_executed++;

			xdev->trace_start = false;
		}
		//$DD デバイスリセット  esvmm_reset_device()
		ret = esvmm_reset_device(xdev->base);
		//$DD ドライバ固有統計(device_reset_executed)更新
		xdev->drv_stats.device_reset_executed++;
		if (ret) {
			//$DD リセットエラー(ES_DRV_INTERNAL_ERR_RESET_NG2 = 6)
			pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME,
			       ES_DRV_INTERNAL_ERR_RESET_NG2);
			ES_DRV_LOGGING("%s NotReady=%d,ret=%d",
				__func__,ES_DRV_INTERNAL_ERR_RESET_NG2,ret);
		}
		//$DD NULLでないならesvmm_release_addressを呼ぶ
		esvmm_release_address(xdev);
		xdev->base = NULL;

		//$DD 終了のログ出力、insmod失敗でもこの処理がくるため、ベースアドレス取得後の場合だけ出す
		pr_info(ES_DRV_MSG_RMMOD, ES_DRV_MSG_TIME);
	}

}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．３ VMM割り込みハンドラ
	
  @param[in]     irq	IRQ番号
  @param[in]     dev_id	デバイス識別子(net_device構造体)

  @return  エラーコード
  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC irqreturn_t esif_vmm_irq(int irq, memptr_t dev_id)
{
	struct net_device *dev = dev_id;
	struct esdrv_priv *priv = netdev_priv(dev);
	intreg_t intr_cause;	//割り込み原因

	PDEBUG_DEV(PD_ENT, "ENTER");

	//$DD 割り込み結果をキャプチャする        esvmm_interrupt_capture()
	intr_cause = esvmm_interrupt_capture(priv->xsdevice.base);
	//$DD 割り込みハンドラ振り分け
	if (intr_cause & INTREG_MASK_IS_ASSERT) {
		if (intr_cause & INTREG_IRQMASK_RX_DATA) {
			//$DD 受信割り込みハンドラ呼出し   esdrv_rx_irq()
			esdrv_rx_irq(dev, intr_cause & INTREG_MASK_EPID);
			//$DD ドライバ固有統計(receive_interrupts_rx)更新
			ES_DRV_CHARACTERISTIC_STATS_INC(priv,
							intr_cause &
							INTREG_MASK_EPID,
							receive_interrupts_rx,
							1);
		}
		if (intr_cause & INTREG_IRQMASK_TXRX_STOP_REQ) {
			//$DD 受信中止要求割り込みハンドラ呼出し       esif_txrx_stop_req_irq()
			esif_txrx_stop_req_irq(dev,
					       intr_cause & INTREG_MASK_EPID);
			//$DD ドライバ固有統計(receive_interrupts_unshare)更新
			ES_DRV_CHARACTERISTIC_STATS_INC(priv,
							intr_cause &
							INTREG_MASK_EPID,
							receive_interrupts_unshare,
							1);
		}
		if (intr_cause & INTREG_IRQMASK_TXRX_STOP_DONE) {
			//$DD 受信中止完了割り込みハンドラ呼出し       esif_txrx_stop_done_irq()
			//$DD 現状利用しないため、マスクだけに置き換え
#if defined(ES_DRV_USE_STOP_DONE_IRQ)
			//esif_txrx_stop_done_irq(dev, intr_cause&INTREG_MASK_EPID);
#else				/* ES_DRV_USE_STOP_DONE_IRQ */
			esvmm_irqmask(priv->xsdevice.base,
				      INTREG_IRQMASK_TXRX_STOP_DONE, true);
#endif				/* ES_DRV_USE_STOP_DONE_IRQ */
		}
		if (intr_cause & INTREG_IRQMASK_DEV_STOP_REQ) {
			//$DD EPがダウンしたログ
			pr_err(ES_DRV_MSG_EP_SYSDOWN1, ES_DRV_MSG_TIME,
			       intr_cause & INTREG_MASK_EPID);
			//$DD 停止要求割り込み割り込みハンドラ呼出し esif_dev_stop_req_irq()
			esif_dev_stop_req_irq(dev,
					      intr_cause & INTREG_MASK_EPID);
			//$DD ドライバ固有統計(receive_interrupts_stop)更新
			ES_DRV_CHARACTERISTIC_STATS_INC(priv,
							intr_cause &
							INTREG_MASK_EPID,
							receive_interrupts_stop,
							1);
		}
		if (intr_cause & INTREG_IRQMASK_INFO_UPDATE) {
			//$DD ES情報更新割り込みハンドラ呼出し   esif_update_zone_irq
			esif_update_zone_irq(dev,
					     intr_cause & INTREG_MASK_EPID);
			//$DD ドライバ固有統計(receive_interrupts_zoneupdate)更新
			ES_DRV_CHARACTERISTIC_STATS_INC(priv,
							intr_cause &
							INTREG_MASK_EPID,
							receive_interrupts_zoneupdate,
							1);
		}
		//$DD 処理したのでIRQ_HANDLEDで終了
		return IRQ_HANDLED;
	} else {
		//$DD 割り込み無効だったのでIRQ_NONEで終了
		return IRQ_NONE;
	}
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．４ 割り込みハンドラの登録
	
	割り込みハンドラの登録を行う
	失敗・終了時のクリーンアップは別途行うこと

  @param[in,out] dev	net_device構造体

  @return  エラーコード
  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esif_epirq_init(struct net_device *dev)
{
	int result = -1;
	struct esdrv_priv *priv = netdev_priv(dev);

	//$DD 割り込みの遅延監視ワークキューを登録
	priv->interrupt_watch_enable = true;
	if (!delayed_work_pending(&priv->interrupt_watch_work)) {
		queue_delayed_work(priv->control_wq,
				   &priv->interrupt_watch_work,
				   ES_DRV_IRQ_WATCH_DELAY);
	}
	//$DD 未登録ならVMM割り込みハンドラを登録       request_irq()
	if (!priv->irq_vmm_registerd) {
		PDEBUG_DEV(PD_INF, "(esif_vmm_irq) irq_vmm_registerd is false");
		result =
		    request_irq(priv->xsdevice.config->irq, esif_vmm_irq,
				IRQF_SHARED, "esdrv", dev /*dev_id */ );
		if (result) {
			PDEBUG_DEV(PD_INF, "(esif_vmm_irq) failed");
			//$DD 失敗の場合request_irq()の戻り値を返す
			return result;
		}
		priv->irq_vmm_registerd = true;
	}

	PDEBUG_DEV(PD_INF, "success");

	//終了
	return result;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．５ 割り込みハンドラの終了処理

	(注意)free_irq()を実行した際、最後の割り込み処理が起動する
	割り込みハンドラの削除を行う

  @param[in,out] dev	net_device構造体
**/
FUNC_STATIC void esif_epirq_cleanup(struct net_device *dev)
{
	struct esdrv_priv *priv = netdev_priv(dev);

	PDEBUG_DEV(PD_ENT, "ENTER");

	//$DD 割り込みの遅延監視ワークキューを止める
	priv->interrupt_watch_enable = false;
	cancel_delayed_work_sync(&priv->interrupt_watch_work);

	//$DD 全割り込み禁止設定(念のため) esvmm_irqmask(true)
	esvmm_irqmask(priv->xsdevice.base, INTREG_IRQMASK_ALL, true);

	//$DD 登録済みなら割り込みハンドラ削除  free_irq()
	if (priv->irq_vmm_registerd) {
		free_irq(priv->xsdevice.config->irq, dev);
		priv->irq_vmm_registerd = false;
	}
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．６ 受信要求割り込み発生の突き放し処理

  ワークキューに登録する、送信時の割り込み生成処理の切り離し部分
	
  @param[in,out] work	ワークキューエントリの構造体
**/
FUNC_STATIC void esif_raise_intr_rxdata_work(struct work_struct *work)
{
	epno_t epid;
	epno_t maxepid, selfepid;
	struct esdrv_priv *priv =
	    container_of(work, struct esdrv_priv, raise_intr_rxdata_work);

	PDEBUG(PD_COMMON, "ENTER");

	selfepid = priv->xsdevice.epid_self;
	maxepid = priv->xsdevice.epnum_max;

	//$DD ステータスをクリア
	for (epid = 0; epid < maxepid; epid++) {
		priv->xsdevice.ep_shm_info[epid].tx_status_work = 0;
	}

	//$DD 現状の受信要求割り込み発生のペンディング状態を取得
	//$DD 全EPについてループ処理
	for (epid = 0; epid < maxepid; epid++) {
		if (selfepid == epid) {
			//$DD 自EPIDはスキップ
			continue;
		}
		//$DD 相手EPへ送信可能か確認(共有中で送受信中止要求が出ていない)
		if (EP_PARTNER_SHARED == EP_PARTNER_STATUS(priv, epid)) {

			//$DD 送信可能の場合

			//$DD 送信状態をローカル変数にコピーして、送信状態はクリア
			priv->xsdevice.ep_shm_info[epid].tx_status_work =
			    priv->xsdevice.ep_shm_info[epid].tx.info->v1i.
			    tx_status;
			if (priv->xsdevice.ep_shm_info[epid].tx_status_work ==
			    ES_TX_DELAY_SEND_PENDING) {
				priv->xsdevice.ep_shm_info[epid].tx.info->v1i.
				    tx_status = ES_TX_DELAY_SEND_NONE;
			}
			PDEBUG(PD_COMMON, "tx_status[%d] : 0x%x", epid,
			       priv->xsdevice.ep_shm_info[epid].tx_status_work);
		}
	}

	//$DD 現状の受信要求割り込み発生ペンディング状態のEPに対して受信要求割り込みを生成 esvmm_raise_interrupt()
	for (epid = 0; epid < maxepid; epid++) {
		if (selfepid == epid) {
			//$DD 自EPIDはスキップ
			continue;
		}
		//$DD PENDINGあり、共有有効、対向でPOLL_WORKが設定されていないなら割り込み生成
		if ((priv->xsdevice.ep_shm_info[epid].tx_status_work ==
		     ES_TX_DELAY_SEND_PENDING)
		    && (EP_PARTNER_SHARED == EP_PARTNER_STATUS(priv, epid))
		    && !(ES_RX_POLL_WORK & priv->xsdevice.ep_shm_info[epid].rx.
			 info->v1i.rx_status)) {
			esvmm_raise_interrupt(priv->xsdevice.base, epid,
					      INTREG_IRQMASK_RX_DATA);
			//$DD 対象EPへの受信割り込み送信でドライバ統計(send_interrupts_rx)を更新する
			ES_DRV_CHARACTERISTIC_STATS_INC(priv, epid,
							send_interrupts_rx, 1);
		}
	}

	usleep_range(500, 1000);

	PDEBUG(PD_COMMON, "DONE");
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．７ 送信バッファへの書き込み・送信

  @param[in]     dev	net_device構造体
  @param[in]     dest	送信先EPID
  @param[in]     data	データ
  @param[in]     len	データ長

  @return  エラーコード
  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
  @retval  ENOBUFS	バッファフル
**/
FUNC_STATIC int esif_tx_send(struct net_device *dev, epno_t dest,
			     framebuf_t data, size_t len)
{
	int retval;

	struct esdrv_priv *priv = netdev_priv(dev);

	//$DD 共有バッファへ書き込み(送信) バッファフルチェックを含む esmem_epbuf_tx_pkt_send()
	//$DD 異常値の場合、その値を返却値として終了
	retval =
	    esmem_epbuf_tx_pkt_send(&(priv->xsdevice.ep_shm_info[dest].tx),
				    data, len);
	if (retval)
		return retval;
	//$DD 受信要求割り込み発生のペンディング状態をセットして受信割り込み発生処理をワークキューに登録

	priv->xsdevice.ep_shm_info[dest].tx.info->v1i.tx_status =
	    ES_TX_DELAY_SEND_PENDING;
	if (!work_pending(&priv->raise_intr_rxdata_work)) {
		PDEBUG(PD_COMMON, "schedule work queue");
		queue_work(priv->txrx_wq, &priv->raise_intr_rxdata_work);
		//$DD ドライバ固有統計(interrupts_workqueue_scheduled)更新
		priv->xsdevice.drv_stats.interrupts_workqueue_scheduled++;
	} else {
		// デバッグログのみ
		PDEBUG(PD_COMMON, "already scheduled work queue");
	}
	retval = 0;

	return retval;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．８ 全EPの受信バッファのデータの検索

  データのあるEPを検索する
  複数EPのフレームを平等に処理するため、次フレーム取得をラウンドロビンさせる
	
  @param[in]     priv		net_device構造体のプライベート領域
  @param[in,out] start_epid	探し始めるEPID

  @retval  0以上	見つかったEPID
  @retval  0未満	受信データ無し

**/
FUNC_STATIC epno_t esif_rxframe_search_exist(struct esdrv_priv * priv,
					     epno_t start_epid)
{
	bool empty;
	epno_t cur_epid;
	epno_t maxepid;
	int i;

	PDEBUG(PD_ENT, "ENTER");

	//$DD 自EPID以外の列挙
	//$DD start_epidからend_epidまで検索する(最大値で折り返し)
	//$DD start_epid = 引数のstart_epid + 1
	//$DD end_epid   = 上記のstart_epid - 1(引数のstart_epid)
	//$DD 検索処理はcur_epid+1していく

	maxepid = priv->xsdevice.epnum_max;
	start_epid = (start_epid + 1 + maxepid) % maxepid;

	PDEBUG(PD_ENT, "current search recipe start:%d,max:%d", start_epid,
	       maxepid);

	for (i = 0; i < maxepid; i++) {
		cur_epid = (start_epid + i) % maxepid;
		//$DD 自EPID以外かつ共有ステータスが共有中
		if (cur_epid != (priv->xsdevice.epid_self)
		    && (EP_PARTNER_SHARED == EP_PARTNER_STATUS(priv, cur_epid))
		    ) {
			//$DD バッファが空でなければそのEPIDを返す      esmem_epbuf_rx_is_empty()
			if (!esmem_epbuf_rx_is_empty
			    (&(priv->xsdevice.ep_shm_info[cur_epid].rx)))
				return cur_epid;
		}

	}
	//$DD データ無しで終了
	return -1;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．９ 受信バッファからの１フレーム分のデータ取得

  @param[in,out] dev	net_device構造体
  @param[out]    size	データ長
  @param[in,out] cur_epid	作業対象EPID

  @return  フレームバッファ
  @retval  NULL		データ無し

**/
FUNC_STATIC framebuf_t esif_rxframe_get(struct net_device * dev, size_t * psize,
					epno_t * cur_epid)
{
	framebuf_t frameBuf;
	struct esdrv_priv *priv = netdev_priv(dev);

	//$DD 有効な受信データのあるEPIDを検索。無かったらNULL終了        esif_rxframe_search_exist()
	*cur_epid = esif_rxframe_search_exist(priv, *cur_epid);
	if (*cur_epid < 0)
		return NULL;

	//$DD 共有バッファからのデータとサイズを取得 esmem_epbuf_rx_curpkt_get_addr()
	frameBuf =
	    esmem_epbuf_rx_curpkt_get_addr(&
					   (priv->xsdevice.
					    ep_shm_info[*cur_epid].rx), psize);

	//$DD バッファアドレスを返す
	return frameBuf;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．１０ 受信バッファの１フレーム分のデータを解放

  @param[in]     dev	net_device構造体
  @param[in]     cur_epid	作業対象EPID
**/
FUNC_STATIC void esif_rxframe_release(struct net_device *dev, epno_t cur_epid)
{
	struct esdrv_priv *priv;

	priv = netdev_priv(dev);

	//$DD 受信ポインタ(Head)更新    esmem_epbuf_rx_curpkt_drop()
	esmem_epbuf_rx_curpkt_drop(&(priv->xsdevice.ep_shm_info[cur_epid].rx));
}

//$DD=============================================================================
//$DD 共有開始・終了関連

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．１１ es_deviceのメモリ確保処理
	
  モジュールロード時(esdrv_init)に呼び出される
  送受信バッファの確保を行う

  @note
  確保に失敗などで、エラーを返す場合。上位層が責任をもってesif_freeによる開放を行なうこと

  @param[in]	xdev		es_device情報
	
  @return	エラーコード
  @retval	0		正常終了
  @retval	負の値		<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esif_setup(struct esif_device *xdev)
{
	int epidx;
	memptr_t buf;
	struct ep_share_mem_info *bufPair;
	size_t memSize;
	int result;		//関数の戻り値
	struct es_device_command_param param;
	unsigned long flags;

	PDEBUG_XDEV(PD_ENT, "ENTER");

	//$DD 自EPID,最大EP数について、ポインタを繋ぐ
	xdev->share_ptrs.epnum_max = &(xdev->epnum_max);
	xdev->share_ptrs.epid_self = &(xdev->epid_self);

	//$DD   送受信バッファ配列を確保し０クリアする
	memSize = sizeof(struct ep_share_mem_info) * (xdev->epnum_max);
	buf = kzalloc(memSize, GFP_KERNEL);
	if (!buf) {
		//$DD 失敗の場合
		//$DD メモリ確保失敗ログ
		pr_err(ES_DRV_MSG_MEMFAIL1, ES_DRV_MSG_TIME, ES_DRV_MSG_MEMFAIL1_SHM_INFO);
		//$DD ENOMEMを返す
		return -ENOMEM;
	}
	xdev->ep_shm_info = (struct ep_share_mem_info *)buf;

	//$DD 要求バッファの確保し０クリアする
	memSize = ES_DEV_REQ_BUF_SIZE(xdev->epnum_max);
	xdev->share_ptrs.req_buf = kzalloc(memSize, GFP_KERNEL);
	if (!(xdev->share_ptrs.req_buf)) {
		//$DD 失敗の場合
		//$DD メモリ確保失敗ログ
		pr_err(ES_DRV_MSG_MEMFAIL1, ES_DRV_MSG_TIME, ES_DRV_MSG_MEMFAIL1_DEV_REQ);
		//$DD ENOMEMを返す
		return -ENOMEM;
	}
	xdev->share_ptrs.req_buf_size = memSize;

	//$DD 応答バッファを確保し０クリアする
	memSize = ES_DEV_RES_BUF_SIZE(xdev->epnum_max);
	xdev->share_ptrs.res_buf = kzalloc(memSize, GFP_KERNEL);
	if (!(xdev->share_ptrs.res_buf)) {
		//$DD 失敗の場合
		//$DD メモリ確保失敗ログ
		pr_err(ES_DRV_MSG_MEMFAIL1, ES_DRV_MSG_TIME, ES_DRV_MSG_MEMFAIL1_DEV_RES);
		//$DD ENOMEMを返す
		return -ENOMEM;
	}
	xdev->share_ptrs.res_buf_size = memSize;

	//$DD   共有ステータス領域の確保・初期化        esmem_share_alloc()
	//$DD   メモリ確保失敗時、関数終了(クリーンアップは呼出し元で行う)
	if ((result = esmem_share_alloc(xdev))) {
		//$DD 失敗の場合
		//$DD メモリ確保失敗ログ
		pr_err(ES_DRV_MSG_MEMFAIL1, ES_DRV_MSG_TIME, ES_DRV_MSG_MEMFAIL1_SHM_STATUS);
		//$DD esmem_share_alloc()の戻り値を返す
		return result;
	}
	//$DD 共有状態制御ビットマップクリア
	xdev->share_ptrs.buffer_share_bit = 0;
	xdev->share_ptrs.buffer_unshare_reserve_bit = 0;

	//$DD   EP毎のバッファの確保・初期化
	for (epidx = 0; epidx < (xdev->epnum_max); epidx++) {
		if (epidx != xdev->epid_self) {
			bufPair = &(xdev->ep_shm_info[epidx]);

			//$DD   送信バッファの確保     esmem_epbuf_alloc()
			//$DD   メモリ確保失敗時、関数終了(クリーンアップは呼出し元で行う)
			if ((result = esmem_epbuf_alloc(&(bufPair->tx)))) {
				//$DD メモリ確保失敗ログ
				pr_err(ES_DRV_MSG_MEMFAIL2, ES_DRV_MSG_TIME,
				       ES_DRV_MSG_MEMFAIL2_SHM_TO_EPx, epidx);
				return result;
			}
			//$DD   受信バッファの確保     esmem_epbuf_alloc();
			//$DD   メモリ確保失敗時、関数終了(クリーンアップは呼出し元で行う)
			if ((result = esmem_epbuf_alloc(&(bufPair->rx)))) {
				//$DD メモリ確保失敗ログ
				pr_err(ES_DRV_MSG_MEMFAIL2, ES_DRV_MSG_TIME,
				       ES_DRV_MSG_MEMFAIL2_SHM_FROM_EPx, epidx);
				return result;
			}
			//$DD rx_statusの排他設定	spin_lock_irqsave
			spin_lock_irqsave(&xdev->rx_status_lock, flags);
			//$DD   バッファの仮初期化     esmem_epbuf_setup()
			{
				u8 mac[ETH_ALEN] =
				    { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
				esmem_epbuf_setup(&(bufPair->tx), mac,
						  ep_support_mtu[0]);
				esmem_epbuf_setup(&(bufPair->rx), mac,
						  ep_support_mtu[0]);
			}
			//$DD rx_statusの排他解除	spin_unlock_irqrestore
			spin_unlock_irqrestore(&xdev->rx_status_lock, flags);
		}
	}

	//$DD バッファをレジスタに設定
	//$DD パラメタに設定
	memset((void *)&param, 0, sizeof(param));

	//$DD 要求/応答バッファの設定
	param.req_len = xdev->share_ptrs.req_buf_size;
	param.req_start = __pa(xdev->share_ptrs.req_buf);
	param.res_len = xdev->share_ptrs.res_buf_size;
	param.res_start = __pa(xdev->share_ptrs.res_buf);

	//$DD 共有ステータス領域の設定
	param.share_start = __pa(xdev->share_ptrs.share->ep_status);

	//$DD esvmm_command_setupを実行し、レジスタに反映
	esvmm_command_setup(xdev->base, &param);

	//$DD 正常終了で0を返す
	return 0;		//正常終了
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．１２ es_deviceのオープン処理(共有開始処理)

  デバイス起動時(esdrv_open)に呼び出される
  送受信バッファの確保と、バッファアドレスの通知を行う

  @note
  確保に失敗などで、エラーを返す場合。上位層が責任をもってesif_closeによる開放を行なうこと
  また、どこからesif_closeされても破綻しないようにすること

  @param[in]     xdev	es_device情報

  @return  エラーコード
  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esif_open(struct esif_device *xdev)
{
	epno_t epidx;
	memptr_t buf;
	struct ep_share_mem_info *bufPair;
	size_t memSize;
	int result;		//関数の戻り値
	int i;
	struct net_device *dev = xdev->container_dev;
	struct esdrv_priv *priv = netdev_priv(dev);
	unsigned long flags;

	PDEBUG_XDEV(PD_ENT, "ENTER");

#ifndef ESDRV_ZONE_OFF
	//$DD esvmm_request_infoによりZone情報を取得、保存(初回)を行う
	mutex_lock(&xdev->share_ptrs.lock);
	result = esvmm_request_info(xdev->base, &xdev->share_ptrs);

	switch (result) {
	case 0:
		//$DD 正常系 値保存
		for (epidx = 0; epidx < (xdev->epnum_max); epidx++) {
			xdev->ep_shm_info[epidx].es_status =
			    xdev->share_ptrs.res_buf->info.info[epidx].
			    es_status;
			xdev->ep_shm_info[epidx].zone =
			    xdev->share_ptrs.res_buf->info.info[epidx].zone;
		}
		break;
	default:
	case -ENOMSG:
	case -EBUSY:
		//$DD デフォルト
		//$DD インターフェイスエラー(バッファ長不正)
		//$DD タイムアウト
		//$DD 強制リセット付きでクローズ
		priv->force_reset = true;
		//$DD ドライバ固有統計(device_force_reset_scheduled)更新
		xdev->drv_stats.device_force_reset_scheduled++;

		//$DD アンロックしてエラー終了
		mutex_unlock(&xdev->share_ptrs.lock);
		return result;
		break;
	}
	mutex_unlock(&xdev->share_ptrs.lock);

	//$DD ドライバ固有統計(command_request_info_executed)更新
	xdev->drv_stats.command_request_info_executed++;

	//$DD 状態の変化確認のため、通知を他EPに送る
	for (epidx = 0; epidx < (xdev->epnum_max); epidx++) {
		if ((epidx != xdev->epid_self)
		    && (ES_ZONING_STATUS_ENABLE == xdev->ep_shm_info[epidx].es_status)) {
			//$DD 有効なEPへ状態更新の割り込みを通知
			esvmm_raise_interrupt(xdev->base, epidx, INTREG_IRQMASK_INFO_UPDATE);
			
			//$DD 対象EPへの受信割り込み送信でドライバ統計(send_interrupts_zoneupdate)を更新する
			ES_DRV_CHARACTERISTIC_STATS_INC(container_of(xdev, struct esdrv_priv, xsdevice),
							epidx, send_interrupts_zoneupdate, 1);
		}
	}

	//$DD 状態の安定化待ちのため、EPmax * 300 (共有解除監視時間の最大値)だけ待つ
	msleep(ES_DRV_OPEN_ZONE_UPDATE_WAIT_MSEC * xdev->epnum_max);

#endif				/* !ESDRV_ZONE_OFF */

	//$DD   EP毎のバッファの初期化
	for (epidx = 0; epidx < (xdev->epnum_max); epidx++) {
		if (epidx != xdev->epid_self) {
			bufPair = &(xdev->ep_shm_info[epidx]);

			//$DD rx_statusの排他設定	spin_lock_irqsave
			spin_lock_irqsave(&priv->xsdevice.rx_status_lock, flags);
			//$DD   送信バッファの初期化  esmem_epbuf_setup()
			esmem_epbuf_setup(&(bufPair->tx),
					  xdev->container_dev->dev_addr,
					  xdev->container_dev->mtu);
			//$DD rx_statusの排他解除	spin_unlock_irqrestore
			spin_unlock_irqrestore(&priv->xsdevice.rx_status_lock, flags);

			//$DD 同じゾーンならバッファ共有   esmem_same_zone()
			if (esmem_same_zone(xdev, epidx)) {
				PDEBUG_XDEV(PD_INF, "same zone,register:%d",
					    epidx);
				//$DD   送受信バッファのアドレスを登録   esvmm_register_buff_addr()
				mutex_lock(&xdev->share_ptrs.lock);
				result =
				    esvmm_register_buff_addr(xdev->base, epidx,
							     bufPair,
							     &xdev->share_ptrs);
				mutex_unlock(&xdev->share_ptrs.lock);

				//$DD ドライバ固有統計(command_register_buffer_executed)更新
				ES_DRV_CHARACTERISTIC_STATS_INC(container_of
								(xdev,
								 struct
								 esdrv_priv,
								 xsdevice),
								epidx,
								command_register_buffer_executed,
								1);

				//$DD エラー判定
				switch (result) {
				case 0:
					//$DD 正常系 無処理
					break;
				default:
				case -ENOMSG:
				case -EBUSY:
					//$DD デフォルト
					//$DD インターフェイスエラー(バッファ長不正)
					//$DD タイムアウト
					//$DD 強制リセット付きでクローズ
					priv->force_reset = true;

					//$DD ドライバ固有統計(device_force_reset_scheduled)更新
					xdev->drv_stats.
					    device_force_reset_scheduled++;
					//$DD エラー終了
					return result;
				}
			}

			PDEBUG_XDEV(PD_COMMON, "share(%d) check mac", epidx);
			PDEBUG_XDEV(PD_COMMON,
				    "epbuf tx: mac addr:%02x:%02x:%02x:%02x:%02x:%02x",
				    bufPair->tx.info->v1i.mac_addr[0],
				    bufPair->tx.info->v1i.mac_addr[1],
				    bufPair->tx.info->v1i.mac_addr[2],
				    bufPair->tx.info->v1i.mac_addr[3],
				    bufPair->tx.info->v1i.mac_addr[4],
				    bufPair->tx.info->v1i.mac_addr[5]
			    );
			PDEBUG_XDEV(PD_COMMON,
				    "epbuf rx: mac addr:%02x:%02x:%02x:%02x:%02x:%02x",
				    bufPair->rx.info->v1i.mac_addr[0],
				    bufPair->rx.info->v1i.mac_addr[1],
				    bufPair->rx.info->v1i.mac_addr[2],
				    bufPair->rx.info->v1i.mac_addr[3],
				    bufPair->rx.info->v1i.mac_addr[4],
				    bufPair->rx.info->v1i.mac_addr[5]
			    );
		}
	}

	return 0;		//正常終了
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．１３ es_deviceのフリー処理
	
  モジュールアンロード時(esdrv_exit)に呼び出される
  送受信バッファの解放を行う

  @param[in]	xdev	es_device情報
**/
FUNC_STATIC void esif_free(struct net_device *dev)
{
	struct esdrv_priv *priv = netdev_priv(dev);
	struct esif_device *xdev = &priv->xsdevice;
	epno_t epidx;

	PDEBUG_DEV(PD_ENT, "ENTER");

	if (xdev->ep_shm_info) {
		PDEBUG(PD_INF, "free start");

		//$DD トレースバッファを解放
		if (NULL != xdev->share_ptrs.trace) {
			PDEBUG(PD_INF, "free trace");
			vfree(xdev->share_ptrs.trace);
			xdev->share_ptrs.trace = NULL;
		}
		xdev->share_ptrs.trace_size = 0;
		xdev->trace_mode = 0;

		//$DD 共有バッファの開放       esmem_share_free()
		esmem_share_free(xdev);

		//$DD 要求と応答バッファ配列の解放
		if (NULL != xdev->share_ptrs.req_buf) {
			PDEBUG(PD_INF, "req free");
			kfree(xdev->share_ptrs.req_buf);
		}
		xdev->share_ptrs.req_buf = NULL;

		if (NULL != xdev->share_ptrs.res_buf) {
			PDEBUG(PD_INF, "res free");
			kfree(xdev->share_ptrs.res_buf);
		}
		xdev->share_ptrs.res_buf = NULL;

		// 要求応答の後にし、アクセスをなくす
		//$DD   EP毎の送受信(共有)バッファの解放(自EPID以外)
		//$DD   バッファ共有状態管理テーブルの解放     esmem_epbuf_free()
		for (epidx = 0; epidx < (xdev->epnum_max); epidx++) {
			if (epidx != xdev->epid_self) {	//自EPID以外
				esmem_epbuf_free(&xdev->ep_shm_info[epidx].tx);	//送信バッファの解放
				esmem_epbuf_free(&xdev->ep_shm_info[epidx].rx);	//受信バッファの解放
			}
		}

		//$DD 送受信バッファ配列の解放
		kfree(xdev->ep_shm_info);
		xdev->ep_shm_info = NULL;

	}
	PDEBUG_DEV(PD_COMMON || PD_INF, "DONE");
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．１４ es_deviceのクローズ処理(共有終了処理)
	
  デバイス停止時(esdrv_stop)に呼び出される
  送受信バッファの解放を行う

  @param[in]     xdev	es_device情報
**/
FUNC_STATIC void esif_close(struct net_device *dev)
{
	struct esdrv_priv *priv = netdev_priv(dev);
	struct esif_device *xdev = &priv->xsdevice;
	struct ep_share_mem_info *bufPair;
	epno_t epidx;
	int ret;
	bool reset_flag = false;
	struct es_device_command_param param;
	unsigned long flags;

	PDEBUG_DEV(PD_ENT, "ENTER");

	//$DD 共有解除待ちを含めすべてについて解除実施
	for (epidx = 0; epidx < (xdev->epnum_max); epidx++) {
		if (epidx != xdev->epid_self) {
			//$DD   送受信バッファのアドレスを削除(解除済みなら無処理なので問題なし) esvmm_unregister_buff_addr()
			mutex_lock(&xdev->share_ptrs.lock);
			ret =
			    esvmm_unregister_buff_addr(xdev->base, epidx,
						       &xdev->share_ptrs);
			mutex_unlock(&xdev->share_ptrs.lock);
			//$DD エラー時はリセットのリカバリ設定
			if (ret != 0) {
				reset_flag = true;

				//$DD ドライバ固有統計(device_force_reset_scheduled)更新
				xdev->drv_stats.device_force_reset_scheduled++;
			}
			//$DD ドライバ固有統計(command_unregister_buffer_executed)更新
			ES_DRV_CHARACTERISTIC_STATS_INC(container_of
							(xdev,
							 struct esdrv_priv,
							 xsdevice), epidx,
							command_unregister_buffer_executed,
							1);

			//$DD セットアップ関数で初期化をしておく
			bufPair = &(xdev->ep_shm_info[epidx]);

			//$DD rx_statusの排他設定	spin_lock_irqsave
			spin_lock_irqsave(&priv->xsdevice.rx_status_lock, flags);
			//$DD   送信バッファの初期化  esmem_epbuf_setup()
			esmem_epbuf_setup(&(bufPair->tx), dev->dev_addr,
					  dev->mtu);
			//$DD rx_statusの排他解除	spin_unlock_irqrestore
			spin_unlock_irqrestore(&priv->xsdevice.rx_status_lock, flags);

			//$DD 対象ＥＰの送受信停止要求をクリア
			clear_bit(epidx, &priv->xsdevice.txrx_stop_req_bit);
			PDEBUG_DEV(PD_COMMON
				   || PD_STATUS,
				   "close done (EPID %d)EP_PARTNER_STATUS:%d",
				   epidx, EP_PARTNER_STATUS(priv, epidx));
		}
	}

	//$DD もし、削除しきれないバッファがあった場合、もしくは強制リセットフラグでリセットの後再初期化する
	if (reset_flag || priv->force_reset) {
		//$DD デバイスリセット  esvmm_reset_device()
		ret = esvmm_reset_device(xdev->base);
		//$DD ドライバ固有統計(device_reset_executed)更新
		xdev->drv_stats.device_reset_executed++;

		//$DD 強制リセットフラグ解除
		priv->force_reset = false;

		if (ret) {
			//$DD リセットエラー(ES_DRV_INTERNAL_ERR_RESET_NG3 = 7)
			pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME,
			       ES_DRV_INTERNAL_ERR_RESET_NG3);
			ES_DRV_LOGGING("%s NotReady=%d,ret=%d",
				__func__,ES_DRV_INTERNAL_ERR_RESET_NG3,ret);

			//$DD 回復不可能なのでオープン抑制を行う
			priv->open_guard = true;
		}
		//$DD バッファの登録状態を初期化
		xdev->share_ptrs.buffer_share_bit = 0;

		//$DD バッファをレジスタに設定
		//$DD パラメタに設定
		memset((void *)&param, 0, sizeof(param));

		//$DD 要求/応答バッファの設定
		param.req_len = xdev->share_ptrs.req_buf_size;
		param.req_start = __pa(xdev->share_ptrs.req_buf);
		param.res_len = xdev->share_ptrs.res_buf_size;
		param.res_start = __pa(xdev->share_ptrs.res_buf);

		//$DD 共有ステータス領域の設定
		param.share_start = __pa(xdev->share_ptrs.share->ep_status);

		//$DD esvmm_command_setupを実行し、レジスタに反映
		esvmm_command_setup(xdev->base, &param);

	}

	PDEBUG_DEV(PD_COMMON, "DONE");
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．１５ 送受信中止要求を発生
  シーケンス再検討中

  (送受信中止要求  発生側)
  すべての共有中のEPに自EP向けの送受信中止を要求する(割り込みを発生させる)
  他EPの esif_txrx_stop_req_irq() が起動する

  @param[in]     xdev	es_device情報
**/
FUNC_STATIC void esif_epstop_raise(struct esif_device *xdev)
{
	epno_t epidx;
	unsigned long flags;

	PDEBUG_XDEV(PD_ENT, "ENTER");

	//$DD   EP毎のループ
	for (epidx = 0; epidx < (xdev->epnum_max); epidx++) {
		//$DD   自EPID以外で共有中のEPの場合
		if (epidx != (xdev->epid_self)) {	//自EPID以外
			PDEBUG_XDEV(PD_COMMON
				    || PD_STATUS,
				    "before raise (EPID %d)EP_PARTNER_STATUS:%d",
				    epidx,
				    EP_PARTNER_STATUS(container_of
						      (xdev, struct esdrv_priv,
						       xsdevice), epidx));
			switch (EP_PARTNER_STATUS
				(container_of
				 (xdev, struct esdrv_priv, xsdevice), epidx)) {
			default:
			case EP_PARTNER_UNSHARE:
				//$DD EP_PARTNER_UNSHARE
				//$DD 共有解除
				//$DD 予約先でまとめて解除するため、予約
				break;
			case EP_PARTNER_COMPLETE:
			case EP_PARTNER_WAITING:
				//$DD EP_PARTNER_COMPLETE
				//$DD EP_PARTNER_WAITING
				//$DD 共有解除予約
				break;
			case EP_PARTNER_SHARED:
				//$DD EP_PARTNER_SHARED
				//$DD 送受信停止要求(共有解除)の割り込みを発生する
				esvmm_raise_interrupt(xdev->base, epidx,
						      INTREG_IRQMASK_TXRX_STOP_REQ);
				//$DD 対象EPへの停止要求割り込み送信でドライバ統計(send_interrupts_unshare)を更新する
				ES_DRV_CHARACTERISTIC_STATS_INC(container_of
								(xdev,
								 struct
								 esdrv_priv,
								 xsdevice),
								epidx,
								send_interrupts_unshare,
								1);
				break;
			}
			//$DD すべての場合において共有解除予約をする(解除済みは無視されるだけで問題なし?)
			set_bit(epidx,
				&xdev->share_ptrs.buffer_unshare_reserve_bit);

			//$DD ＥＰ向け送受信を停止のため、フラグ設定
			set_bit(epidx, &xdev->txrx_stop_req_bit);
			//$DD rx_statusの排他設定	spin_lock_irqsave
			spin_lock_irqsave(&xdev->rx_status_lock, flags);
			xdev->ep_shm_info[epidx].tx.info->v1i.rx_status |=
			    ES_RX_STOP_REQ_REQUEST;
			//$DD rx_statusの排他解除	spin_unlock_irqrestore
			spin_unlock_irqrestore(&xdev->rx_status_lock, flags);

			PDEBUG_XDEV(PD_COMMON
				    || PD_STATUS,
				    "after raise (EPID %d)EP_PARTNER_STATUS:%d",
				    epidx,
				    EP_PARTNER_STATUS(container_of
						      (xdev, struct esdrv_priv,
						       xsdevice), epidx));
		}
	}

}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．１６ 送受信中止完了待ち
  シーケンス再検討中
	
  (送受信中止要求  発生側)
  すべての共有中のEPが送受信中止を完了するまで待つ

  @param[in]     xdev	es_device情報

  @return  エラーステータス
  @retval  0		正常終了
  @retval  0以外	異常終了(タイムアウト)
**/
FUNC_STATIC int esif_epstop_wait(struct esif_device *xdev)
{
	epno_t epidx;
	int wait_time = 0;

	PDEBUG_XDEV(PD_ENT, "ENTER");

	//$DD 共有解除の待ちがある場合に実施
	while (xdev->share_ptrs.buffer_unshare_reserve_bit
	       && (wait_time < ES_COMMAND_EPSTOP_WAIT_MAX_MSEC)) {
		//$DD 共有解除準備完了か共有が解除(シャットダウン)されて予約のあるEPについて、共有解除予約クリア
		for (epidx = 0; epidx < (xdev->epnum_max); epidx++) {
			if ((epidx != xdev->epid_self)
			    && (!esmem_is_shared(xdev->share_ptrs.share, epidx)
				|| (ES_RX_STOP_REQ_DONE & xdev->
				    ep_shm_info[epidx].rx.info->v1i.rx_status))
			    && test_bit(epidx,
					&xdev->share_ptrs.
					buffer_unshare_reserve_bit)) {
				//$DD 共有解除予約をクリア
				clear_bit(epidx,
					  &xdev->share_ptrs.
					  buffer_unshare_reserve_bit);
				PDEBUG_XDEV(PD_COMMON
					    || PD_STATUS,
					    "stop wait done (EPID %d)EP_PARTNER_STATUS:%d",
					    epidx,
					    EP_PARTNER_STATUS(container_of
							      (xdev,
							       struct
							       esdrv_priv,
							       xsdevice),
							      epidx));
			}
		}
		//$DD ディレイを入れる
		msleep(ES_COMMAND_EPSTOP_WAIT_UNIT_MSEC);
		wait_time += ES_COMMAND_EPSTOP_WAIT_UNIT_MSEC;
	}

	//$DD タイムアウトでの処理
	for (epidx = 0; epidx < (xdev->epnum_max); epidx++) {
		if ((epidx != xdev->epid_self)
		    && test_bit(epidx,
				&xdev->share_ptrs.buffer_unshare_reserve_bit)) {
			//$DD 共有解除予約をクリア
			clear_bit(epidx,
				  &xdev->share_ptrs.buffer_unshare_reserve_bit);
		}
	}

	//$DD タイムアウトせずに終了なら0、そうでないならEBUSYを返す
	PDEBUG_XDEV(PD_COMMON, "DONE");
	return (wait_time < ES_COMMAND_EPSTOP_WAIT_MAX_MSEC) ? 0 : -EBUSY;
}

#if defined(ES_DRV_USE_STOP_DONE_IRQ)
//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．１７ 受信中止完了割り込み
  シーケンス再検討中

	(送受信中止要求  発生側)
	IRQハンドラとして登録する
	他EPから受信中止完了を受信したら起動される

  @param[in,out] dev		net_device構造体
  @param[in]     src_epid	要求元のEPID
**/
FUNC_STATIC void esif_txrx_stop_done_irq(struct net_device *dev,
					 epno_t src_epid)
{
	//$DD 送受信中止完了待ちesif_epstop_wait() を終了する?
	PDEBUG_DEV(PD_ENT, "ENTER");
}
#endif				/* ES_DRV_USE_STOP_DONE_IRQ */

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．１８ 送受信中止要求割り込み
  シーケンス再検討中

	(送受信中止要求  受付側)
	IRQハンドラとして登録する
	他EPから受信中止要求を受信したら起動される
	他EPの esif_epstop_raise() により起動される

  @param[in,out] dev		net_device構造体
  @param[in]     src_epid	要求元のEPID
**/
FUNC_STATIC void esif_txrx_stop_req_irq(struct net_device *dev, epno_t src_epid)
{
	struct esdrv_priv *priv = netdev_priv(dev);

	PDEBUG_DEV(PD_ENT, "ENTER");

	PDEBUG_DEV(PD_COMMON || PD_INF
		   || PD_STATUS, "start (EPID %d)EP_PARTNER_STATUS:%d",
		   src_epid, EP_PARTNER_STATUS(priv, src_epid));
	
	ES_DRV_LOGGING("%s pre (EPID %d)EP_PARTNER_STATUS=%d(share=%d,stop_req=%d,stop_done=%d) tx.rx_status=%08x",
		__func__,
		src_epid, EP_PARTNER_STATUS(priv, src_epid),
		esmem_is_shared(priv->xsdevice.share_ptrs.share, src_epid),
		esif_epstop_requested(priv, src_epid),
		esif_epstop_process_done(priv, src_epid),
		priv->xsdevice.ep_shm_info[src_epid].tx.info->v1i.rx_status);
	//$DD 共有解除
	switch (EP_PARTNER_STATUS(priv, src_epid)) {
	default:
	case EP_PARTNER_UNSHARE:
		//$DD EP_PARTNER_UNSHARE
		//$DD ありえない
		break;
	case EP_PARTNER_COMPLETE:
		//$DD EP_PARTNER_COMPLETE
		//$DD 無処理でよい
		break;
	case EP_PARTNER_WAITING:
		//$DD EP_PARTNER_WAITING
		//$DD 相手EPIDより大きい場合にEP_PARTNER_COMPLETEに遷移しつつ、共有解除監視を起動
		if (src_epid < priv->xsdevice.epid_self) {
			//$DD rx_statusの排他設定(割り込みコンテキスト)	spin_lock
			spin_lock(&priv->xsdevice.rx_status_lock);
			//$DD 共有完了ステータスを設定(Txのrx_status)
			priv->xsdevice.ep_shm_info[src_epid].tx.info->v1i.
			    rx_status |= ES_RX_STOP_REQ_DONE;
			//$DD rx_statusの排他解除(割り込みコンテキスト)	spin_unlock
			spin_unlock(&priv->xsdevice.rx_status_lock);

			//$DD 対象ＥＰの送受信停止要求をクリア
			clear_bit(src_epid, &priv->xsdevice.txrx_stop_req_bit);

			//$DD 共有状態解除監視に切り替え、監視対象として情報設定
			set_bit(src_epid, &priv->unshare_watch_bit);
			//$DD 共有状態解除監視ワークキューを起動       queue_work(priv->control_wq)
			if (!work_pending(&priv->unshare_watch_work)) {
				queue_work(priv->control_wq,
					   &priv->unshare_watch_work);
				//$DD ドライバ固有統計(unshare_watch_workqueue_scheduled)更新
				priv->xsdevice.drv_stats.
				    unshare_watch_workqueue_scheduled++;
			}
		}
		break;
	case EP_PARTNER_SHARED:
		//$DD EP_PARTNER_SHARED
		//$DD 相手が送受信停止を立てているかを確認する
		if (ES_RX_STOP_REQ_REQUEST & priv->xsdevice.
		    ep_shm_info[src_epid].rx.info->v1i.rx_status) {
			//$DD 送受信中止処理ワークキューへの通知を設定
			set_bit(src_epid, &priv->xsdevice.epstop_req_bit);

			//$DD 共有解除処理をスケジュールする(queue_work(priv->control_wq)でesif_epstop_workの呼出し)
			if (!work_pending(&priv->xsdevice.workqueue_epstop)) {
				queue_work(priv->control_wq,
					   &priv->xsdevice.workqueue_epstop);
				//$DD ドライバ固有統計(unshare_workqueue_scheduled)更新
				priv->xsdevice.drv_stats.
				    unshare_workqueue_scheduled++;
			}
		}
		break;
	}
	PDEBUG_DEV(PD_COMMON || PD_INF
		   || PD_STATUS, "end (EPID %d)EP_PARTNER_STATUS:%d", src_epid,
		   EP_PARTNER_STATUS(priv, src_epid));

	PDEBUG_DEV(PD_COMMON
		   || PD_INF,
		   "END epid=%d stop_req_bit=%lx epstop_req_bit=%lx ", src_epid,
		   priv->xsdevice.txrx_stop_req_bit,
		   priv->xsdevice.epstop_req_bit);
	
	ES_DRV_LOGGING("%s post (EPID %d)EP_PARTNER_STATUS=%d(share=%d,stop_req=%d,stop_done=%d) tx.rx_status=%08x",
		__func__,
		src_epid, EP_PARTNER_STATUS(priv, src_epid),
		esmem_is_shared(priv->xsdevice.share_ptrs.share, src_epid),
		esif_epstop_requested(priv, src_epid),
		esif_epstop_process_done(priv, src_epid),
		priv->xsdevice.ep_shm_info[src_epid].tx.info->v1i.rx_status);
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．３．１９ 停止要求ハンドラ
  シーケンス再検討中

	IRQハンドラとして登録する
	他EPから停止要求を受信したら起動される
	VMM により起動される

  @param          dev		net_deviceオブジェクト
  @param          src_epid	要求元のEPID
**/
FUNC_STATIC void esif_dev_stop_req_irq(struct net_device *dev, epno_t src_epid)
{
	struct esdrv_priv *priv = netdev_priv(dev);

	PDEBUG_DEV(PD_ENT, "ENTER");

	PDEBUG_DEV(PD_COMMON || PD_INF
		   || PD_STATUS, "start (EPID %d)EP_PARTNER_STATUS:%d",
		   src_epid, EP_PARTNER_STATUS(priv, src_epid));

	//$DD 停止要求では強制解除を行うため、buffer_unshare_reserve_bitに値を設定
	set_bit(src_epid,
		&priv->xsdevice.share_ptrs.buffer_unshare_reserve_bit);

	ES_DRV_LOGGING("%s pre (EPID %d)EP_PARTNER_STATUS=%d(share=%d,stop_req=%d,stop_done=%d) tx.rx_status=%08x",
		__func__,
		src_epid, EP_PARTNER_STATUS(priv, src_epid),
		esmem_is_shared(priv->xsdevice.share_ptrs.share, src_epid),
		esif_epstop_requested(priv, src_epid),
		esif_epstop_process_done(priv, src_epid),
		priv->xsdevice.ep_shm_info[src_epid].tx.info->v1i.rx_status);
	//$DD 共有解除
	switch (EP_PARTNER_STATUS(priv, src_epid)) {
	case EP_PARTNER_WAITING:
		//$DD EP_PARTNER_WAITING
		//$DD rx_statusの排他設定(割り込みコンテキスト)	spin_lock
		spin_lock(&priv->xsdevice.rx_status_lock);
		//$DD 共有完了ステータスを設定(Txのrx_status)
		priv->xsdevice.ep_shm_info[src_epid].tx.info->v1i.rx_status |=
		    ES_RX_STOP_REQ_DONE;
		//$DD rx_statusの排他解除(割り込みコンテキスト)	spin_unlock
		spin_unlock(&priv->xsdevice.rx_status_lock);

		//$DD 対象ＥＰの送受信停止要求をクリア
		clear_bit(src_epid, &priv->xsdevice.txrx_stop_req_bit);
		//$DD 以降はUNSHARE,COMPLETEと同じなので fall through
	default:
	case EP_PARTNER_UNSHARE:
		//$DD EP_PARTNER_UNSHARE
	case EP_PARTNER_COMPLETE:
		//$DD EP_PARTNER_COMPLETE
		//$DD 強制解除のため、共有状態解除監視ワークキューを起動が必要

		//$DD 監視対象として情報設定
		set_bit(src_epid, &priv->unshare_watch_bit);
		//$DD 共有状態解除監視ワークキューを起動       queue_work(priv->control_wq)
		if (!work_pending(&priv->unshare_watch_work)) {
			queue_work(priv->control_wq, &priv->unshare_watch_work);
			//$DD ドライバ固有統計(unshare_watch_workqueue_scheduled)更新
			priv->xsdevice.drv_stats.
			    unshare_watch_workqueue_scheduled++;
		}
		break;
	case EP_PARTNER_SHARED:
		//$DD EP_PARTNER_SHARED
		//$DD 送受信中止処理ワークキューへの通知を設定
		set_bit(src_epid, &priv->xsdevice.epstop_req_bit);

		//$DD 共有解除処理をスケジュールする(queue_work(priv->control_wq)でesif_epstop_workの呼出し)
		if (!work_pending(&priv->xsdevice.workqueue_epstop)) {
			queue_work(priv->control_wq,
				   &priv->xsdevice.workqueue_epstop);
			//$DD ドライバ固有統計(unshare_workqueue_scheduled)更新
			priv->xsdevice.drv_stats.unshare_workqueue_scheduled++;
		}
		break;
	}
	PDEBUG_DEV(PD_COMMON || PD_INF
		   || PD_STATUS, "end (EPID %d)EP_PARTNER_STATUS:%d", src_epid,
		   EP_PARTNER_STATUS(priv, src_epid));

	PDEBUG_DEV(PD_COMMON
		   || PD_INF,
		   "END epid=%d stop_req_bit=%lx epstop_req_bit=%lx ", src_epid,
		   priv->xsdevice.txrx_stop_req_bit,
		   priv->xsdevice.epstop_req_bit);
	
	ES_DRV_LOGGING("%s post (EPID %d)EP_PARTNER_STATUS=%d(share=%d,stop_req=%d,stop_done=%d) tx.rx_status=%08x",
		__func__,
		src_epid, EP_PARTNER_STATUS(priv, src_epid),
		esmem_is_shared(priv->xsdevice.share_ptrs.share, src_epid),
		esif_epstop_requested(priv, src_epid),
		esif_epstop_process_done(priv, src_epid),
		priv->xsdevice.ep_shm_info[src_epid].tx.info->v1i.rx_status);
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．３．２０ ゾーン変更ハンドラ
  シーケンス再検討中

	IRQハンドラとして登録する
	VMMからZone変更が通知される際に起動される
	VMM により起動される

  @param          dev		net_deviceオブジェクト
  @param          src_epid	要求元のEPID
**/
FUNC_STATIC void esif_update_zone_irq(struct net_device *dev, epno_t src_epid)
{
	struct esdrv_priv *priv = netdev_priv(dev);

	PDEBUG_DEV(PD_ENT, "ENTER");

	if (!work_pending(&priv->xsdevice.update_zone_work)) {

		//$DD ゾーン情報更新をスケジュールする(queue_work(priv->control_wq)でesif_update_zone_workの呼出し)
		queue_work(priv->control_wq, &priv->xsdevice.update_zone_work);

		//$DD ドライバ固有統計(zoneupdate_workqueue_scheduled)更新
		priv->xsdevice.drv_stats.zoneupdate_workqueue_scheduled++;
	}
	PDEBUG_DEV(PD_COMMON || PD_INF
		   || PD_STATUS, "(EPID %d)EP_PARTNER_STATUS:%d", src_epid,
		   EP_PARTNER_STATUS(priv, src_epid));

	PDEBUG_DEV(PD_COMMON
		   || PD_INF, "END epid=%d stop_req_bit=%lx", src_epid,
		   priv->xsdevice.txrx_stop_req_bit);
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．２１ 他EPからの送受信中止要求処理

  (送受信中止要求  受付側)
  受信中止要求割り込み esif_txrx_stop_req_irq() でスケジュールされる
  送信バッファヘッダの受信ステータスを受信中止完了に設定する
	
  共有解除の確認はポーリングで行うが、割り込みによる通知の方が望ましい

  @param[int]     work	ワークキューエントリの構造体
**/
FUNC_STATIC void esif_epstop_work(struct work_struct *work)
{
	struct esif_device *xdev =
	    container_of(work, struct esif_device, workqueue_epstop);
	struct net_device *dev = xdev->container_dev;
	struct esdrv_priv *priv = netdev_priv(dev);
	epno_t epid_bit;	//対象EPID(ビット位置)
	unsigned long remain_bit;	//未処理の共有停止完了通知(EP毎のビット)
	unsigned long flags;

	//$DD 未処理の停止要求が無くなるまで繰り返す(停止対象のEPを保存)
	while ((remain_bit = priv->xsdevice.epstop_req_bit)) {
		PDEBUG_DEV(PD_COMMON, "while (stop_req_bit=%lx)", remain_bit);

		//$DD (注意)共有バッファの解放待ちをポーリングでする場合、取りこぼしの可能性を考慮(共有解除から、すぐに共有中に戻る可能性あり)

		//$DD 送受信を停止したＥＰ毎の処理
		for (epid_bit = 0; remain_bit; (remain_bit >>= 1), (epid_bit++))	//セットされたビットがある間
		{
			if (remain_bit & 1) {
				//$DD rx_statusの排他設定	spin_lock_irqsave
				spin_lock_irqsave(&priv->xsdevice.rx_status_lock, flags);
				//$DD 共有完了ステータスを設定(Txのrx_status)
				priv->xsdevice.ep_shm_info[epid_bit].tx.info->
				    v1i.rx_status |= ES_RX_STOP_REQ_DONE;
				//$DD rx_statusの排他解除	spin_unlock_irqrestore
				spin_unlock_irqrestore(&priv->xsdevice.rx_status_lock, flags);

				//$DD 対象ＥＰの通知をクリア
				clear_bit(epid_bit,
					  &priv->xsdevice.epstop_req_bit);
				PDEBUG_DEV(PD_COMMON,
					   "cleared epid=%d stop_req_bit=%lx remain=%lx",
					   epid_bit,
					   priv->xsdevice.epstop_req_bit,
					   remain_bit);

				//$DD 共有状態解除監視に切り替え、監視対象として情報設定
				set_bit(epid_bit, &priv->unshare_watch_bit);
				//$DD 共有状態解除監視ワークキューを起動       queue_work(priv->control_wq)
				if (!work_pending(&priv->unshare_watch_work)) {
					queue_work(priv->control_wq,
						   &priv->unshare_watch_work);
					//$DD ドライバ固有統計(unshare_watch_workqueue_scheduled)更新
					priv->xsdevice.drv_stats.
					    unshare_watch_workqueue_scheduled++;
				}
				PDEBUG_DEV(PD_COMMON
					   || PD_STATUS,
					   "state change (EPID %d)EP_PARTNER_STATUS:%d",
					   epid_bit, EP_PARTNER_STATUS(priv,
								       epid_bit));
			}
		}
	}
	PDEBUG_DEV(PD_COMMON, "END");
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．３．２２ ゾーン更新ワークキュー

	受信中止要求割り込み esif_update_zone_irq() でスケジュールされる
	新しいゾーン情報を取得し、変更に合わせた動作を行う

  @param[int]     work	ワークキューエントリの構造体
**/
FUNC_STATIC void esif_update_zone_work(struct work_struct *work)
{
	struct esif_device *xdev =
	    container_of(work, struct esif_device, update_zone_work);
	struct net_device *dev = xdev->container_dev;
	struct esdrv_priv *priv = netdev_priv(dev);
	epno_t epidx;
	unsigned long share_bit = 0;	//未処理の共有開始(EP毎のビット)
	unsigned long unshare_bit = 0;	//未処理の共有停止(EP毎のビット)
	unsigned long irq_bit = 0;	//未処理の共有停止完了通知(EP毎のビット)
	int ret;
	bool update = false;
	unsigned long flags;

	PDEBUG_DEV(PD_ENT, "ENTER");

	//$DD esvmm_request_infoによりZone情報を取得、保管する
	mutex_lock(&priv->xsdevice.share_ptrs.lock);
	ret =
	    esvmm_request_info(priv->xsdevice.base, &priv->xsdevice.share_ptrs);
	switch (ret) {
	default:
	case -ENOMSG:
	case -EBUSY:
		//$DD デフォルト
		//$DD インターフェイスエラー(バッファ長不正)
		//$DD タイムアウト
		//$DD エラーリカバリとして、強制リセット付きでクローズを行う。ワークキューへクローズ突き放しを登録
		if (!work_pending(&priv->force_close)) {
			PDEBUG(PD_COMMON, "schedule work queue");
			priv->force_reset = true;
			//$DD ドライバ固有統計(device_force_reset_scheduled)更新
			xdev->drv_stats.device_force_reset_scheduled++;

			schedule_work(&priv->force_close);
		}
		break;
	case 0:
		//$DD 正常系
		for (epidx = 0; epidx < (priv->xsdevice.epnum_max); epidx++) {
			if (epidx != priv->xsdevice.epid_self) {
				PDEBUG_DEV(PD_COMMON
					   || PD_STATUS,
					   "(EPID %d)EP_PARTNER_STATUS:%d",
					   epidx, EP_PARTNER_STATUS(priv,
								    epidx));
				//$DD 状態に合わせて処理
				switch (EP_PARTNER_STATUS(priv, epidx)) {
				default:
				case EP_PARTNER_UNSHARE:
					//$DD EP_PARTNER_UNSHARE
					//$DD 非共有の場合
					if ((priv->xsdevice.share_ptrs.res_buf->info.info[epidx].zone != ES_ZONING_ZONE_TYPE_NONE)	// 自分のZoneがnone以外
					    && (priv->xsdevice.share_ptrs.res_buf->info.info[epidx].es_status == ES_ZONING_STATUS_ENABLE)	// 相手の状態が有効
					    && (priv->xsdevice.share_ptrs.res_buf->info.info[epidx].zone == priv->xsdevice.share_ptrs.res_buf->info.info[priv->xsdevice.epid_self].zone)	// 相手と同じゾーン
					    ) {
						//$DD ゾーンがそろったのなら、共有を設定(フラグ保持)
						set_bit(epidx, &share_bit);
					} else {
						//$DD ゾーンがはずれたなら、共有を削除(フラグ保持)
						set_bit(epidx, &unshare_bit);
					}
					break;
				case EP_PARTNER_COMPLETE:
				case EP_PARTNER_WAITING:
					//$DD EP_PARTNER_COMPLETE
					//$DD EP_PARTNER_WAITING
					//$DD 共有解除準備待ち状態と共有解除準備完了状態は共有解除予約をする
					if ((priv->xsdevice.share_ptrs.res_buf->info.info[epidx].zone == ES_ZONING_ZONE_TYPE_NONE)	// 自分のZoneがnone
					    || (priv->xsdevice.share_ptrs.res_buf->info.info[epidx].es_status != ES_ZONING_STATUS_ENABLE)	// 相手の状態が有効でない
					    || (priv->xsdevice.share_ptrs.res_buf->info.info[epidx].zone != priv->xsdevice.share_ptrs.res_buf->info.info[priv->xsdevice.epid_self].zone)	// 相手と同じゾーンでない
					    ) {
						//$DD ゾーンがはずれたなら、共有監視とその共有削除を設定
						set_bit(epidx,
							&priv->
							unshare_watch_bit);
						set_bit(epidx,
							&priv->xsdevice.
							share_ptrs.
							buffer_unshare_reserve_bit);
					}
					break;
				case EP_PARTNER_SHARED:
					//$DD EP_PARTNER_SHARED
					if ((priv->xsdevice.share_ptrs.res_buf->info.info[epidx].zone == ES_ZONING_ZONE_TYPE_NONE)	// 自分のZoneがnone
					    || (priv->xsdevice.share_ptrs.res_buf->info.info[epidx].es_status != ES_ZONING_STATUS_ENABLE)	// 相手の状態が有効でない
					    || (priv->xsdevice.share_ptrs.res_buf->info.info[epidx].zone != priv->xsdevice.share_ptrs.res_buf->info.info[priv->xsdevice.epid_self].zone)	// 相手と同じゾーンでない
					    ) {
						//$DD ゾーンがはずれたなら、共有解除割り込み予約(フラグ保持)
						set_bit(epidx, &irq_bit);
					}
					break;
				}
			}
			//$DD 更新データを保存
			if (priv->xsdevice.ep_shm_info[epidx].zone !=
			    priv->xsdevice.share_ptrs.res_buf->info.info[epidx].
			    zone) {
				update = true;
			}
			priv->xsdevice.ep_shm_info[epidx].es_status =
			    priv->xsdevice.share_ptrs.res_buf->info.info[epidx].
			    es_status;
			priv->xsdevice.ep_shm_info[epidx].zone =
			    priv->xsdevice.share_ptrs.res_buf->info.info[epidx].
			    zone;
		}
		break;
	}
	mutex_unlock(&priv->xsdevice.share_ptrs.lock);

	//$DD ドライバ固有統計(command_request_info_executed)更新
	priv->xsdevice.drv_stats.command_request_info_executed++;

	//$DD ゾーン変更ログ(Zone情報に変化があったら)
	if (update) {
		pr_info(ES_DRV_MSG_ZONE_CHANGE, ES_DRV_MSG_TIME);
	}
	//$DD 変更をそれぞれ反映
	for (epidx = 0; epidx < (priv->xsdevice.epnum_max); epidx++) {
		// 共有開始or共有停止のフラグをチェックして、処理を行う
		if (priv->xsdevice.epid_self == epidx) {
			// スキップ
			continue;
		}
		//$DD 共有開始の記録あり
		if (test_bit(epidx, &share_bit)) {
			//$DD rx_statusの排他設定	spin_lock_irqsave
			spin_lock_irqsave(&priv->xsdevice.rx_status_lock, flags);
			//$DD バッファを再初期化       esmem_epbuf_setup()
			esmem_epbuf_setup(&
					  (priv->xsdevice.ep_shm_info[epidx].
					   tx),
					  priv->xsdevice.container_dev->
					  dev_addr,
					  priv->xsdevice.container_dev->mtu);
			//$DD rx_statusの排他解除	spin_unlock_irqrestore
			spin_unlock_irqrestore(&priv->xsdevice.rx_status_lock, flags);

			//$DD   送受信バッファのアドレスを登録   esvmm_register_buff_addr()
			mutex_lock(&priv->xsdevice.share_ptrs.lock);
			ret =
			    esvmm_register_buff_addr(priv->xsdevice.base, epidx,
						     &(priv->xsdevice.
						       ep_shm_info[epidx]),
						     &priv->xsdevice.
						     share_ptrs);
			switch (ret) {
			case 0:
				//$DD 正常系 無処理
				break;
			default:
			case -ENOMSG:
			case -EBUSY:
				//$DD デフォルト
				//$DD インターフェイスエラー(バッファ長不正)
				//$DD タイムアウト
				//$DD エラーリカバリとして、強制リセット付きでクローズを行う。ワークキューへクローズ突き放しを登録
				if (!work_pending(&priv->force_close)) {
					PDEBUG(PD_COMMON,
					       "schedule work queue");
					priv->force_reset = true;
					//$DD ドライバ固有統計(device_force_reset_scheduled)更新
					xdev->drv_stats.
					    device_force_reset_scheduled++;

					schedule_work(&priv->force_close);
				}
				break;
			}
			mutex_unlock(&priv->xsdevice.share_ptrs.lock);

			//$DD ドライバ固有統計(command_register_buffer_executed)更新
			ES_DRV_CHARACTERISTIC_STATS_INC(priv, epidx,
							command_register_buffer_executed,
							1);

			PDEBUG_DEV(PD_COMMON
				   || PD_STATUS,
				   "share (EPID %d)EP_PARTNER_STATUS:%d", epidx,
				   EP_PARTNER_STATUS(priv, epidx));
		}
		//$DD 共有停止の記録あり
		if (test_bit(epidx, &unshare_bit)) {
			//$DD   送受信バッファのアドレスを削除   esvmm_unregister_buff_addr()
			mutex_lock(&priv->xsdevice.share_ptrs.lock);
			ret =
			    esvmm_unregister_buff_addr(priv->xsdevice.base,
						       epidx,
						       &priv->xsdevice.
						       share_ptrs);
			switch (ret) {
			case 0:
				//$DD 正常系 無処理
				break;
			default:
			case -ENOMSG:
			case -EBUSY:
				//$DD デフォルト
				//$DD インターフェイスエラー(バッファ長不正)
				//$DD タイムアウト
				//$DD エラーリカバリとして、強制リセット付きでクローズを行う。ワークキューへクローズ突き放しを登録
				if (!work_pending(&priv->force_close)) {
					PDEBUG(PD_COMMON,
					       "schedule work queue");
					priv->force_reset = true;
					//$DD ドライバ固有統計(device_force_reset_scheduled)更新
					xdev->drv_stats.
					    device_force_reset_scheduled++;

					schedule_work(&priv->force_close);
				}
				break;
			}
			mutex_unlock(&priv->xsdevice.share_ptrs.lock);

			//$DD ドライバ固有統計(command_unregister_buffer_executed)更新
			ES_DRV_CHARACTERISTIC_STATS_INC(priv, epidx,
							command_unregister_buffer_executed,
							1);

			if (ret == 0) {
				//$DD rx_statusの排他設定	spin_lock_irqsave
				spin_lock_irqsave(&priv->xsdevice.rx_status_lock, flags);
				//$DD バッファを再初期化       esmem_epbuf_setup()
				esmem_epbuf_setup(&
						  (priv->xsdevice.
						   ep_shm_info[epidx].tx),
						  priv->xsdevice.container_dev->
						  dev_addr,
						  priv->xsdevice.container_dev->
						  mtu);
				//$DD rx_statusの排他解除	spin_unlock_irqrestore
				spin_unlock_irqrestore(&priv->xsdevice.rx_status_lock, flags);
			}

			PDEBUG_DEV(PD_COMMON
				   || PD_STATUS,
				   "unshare (EPID %d)EP_PARTNER_STATUS:%d",
				   epidx, EP_PARTNER_STATUS(priv, epidx));
		}
		//$DD 共有解除通知の記録あり
		if (test_bit(epidx, &irq_bit)) {
			//$DD 送受信停止要求(共有解除)の割り込みを発生する esvmm_raise_interrupt()
			esvmm_raise_interrupt(priv->xsdevice.base, epidx,
					      INTREG_IRQMASK_TXRX_STOP_REQ);
			//$DD 対象EPへの受信割り込み送信でドライバ統計(send_interrupts_unshare)を更新する
			ES_DRV_CHARACTERISTIC_STATS_INC(priv, epidx,
							send_interrupts_unshare,
							1);

			//$DD 送受信停止停止要求フラグを設定
			set_bit(epidx, &priv->xsdevice.txrx_stop_req_bit);
			//$DD rx_statusの排他設定	spin_lock_irqsave
			spin_lock_irqsave(&priv->xsdevice.rx_status_lock, flags);
			priv->xsdevice.ep_shm_info[epidx].tx.info->v1i.
			    rx_status |= ES_RX_STOP_REQ_REQUEST;
			//$DD rx_statusの排他解除	spin_unlock_irqrestore
			spin_unlock_irqrestore(&priv->xsdevice.rx_status_lock, flags);

			//$DD 共有削除を設定
			set_bit(epidx,
				&priv->xsdevice.share_ptrs.
				buffer_unshare_reserve_bit);

			PDEBUG_DEV(PD_COMMON
				   || PD_STATUS,
				   "stopreq (EPID %d)EP_PARTNER_STATUS:%d",
				   epidx, EP_PARTNER_STATUS(priv, epidx));
		}
	}

	//$DD 送受信停止要求があるor共有解除がある場合に共有状態解除監視ワークキューを起動
	if (irq_bit || priv->unshare_watch_bit) {
		//$DD 共有状態解除監視ワークキューを起動       queue_work(priv->control_wq)
		if (!work_pending(&priv->unshare_watch_work)) {
			queue_work(priv->control_wq, &priv->unshare_watch_work);
			//$DD ドライバ固有統計(unshare_watch_workqueue_scheduled)更新
			priv->xsdevice.drv_stats.
			    unshare_watch_workqueue_scheduled++;
		}
	}
	PDEBUG_DEV(PD_COMMON, "END");
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．２３ 共有解除準備待ち監視および共有解除監視ワークキュー
	
  共有解除準備待ちと共有状態変更を監視するワークキュー

  @param[int]     work	ワークキューエントリの構造体
**/
FUNC_STATIC void esif_watch_unshare(struct work_struct *work)
{
	struct esdrv_priv *priv =
	    container_of(work, struct esdrv_priv, unshare_watch_work);
	epno_t epidx;
	epno_t maxepid, selfepid;
	struct esif_device *xdev = &priv->xsdevice;
	int wait_time = 0;
	int ret;
	unsigned long unshare_watch_bit;	// 共有状態監視(EP毎のビット) 
	unsigned long flags;

	selfepid = priv->xsdevice.epid_self;
	maxepid = priv->xsdevice.epnum_max;

	//$DD もし対向が停止などしている場合で共有削除を実施できない場合について、その後のリカバリー処理で停止要求が動作する想定。
	//$DD なので、タイムアウトでの共有監視はクリアする。そのため、この時点の値を保持し0クリアしておく(複数同時動作対策)
	unshare_watch_bit = priv->unshare_watch_bit;
	priv->unshare_watch_bit = 0;

	PDEBUG_XDEV(PD_ENT, "ENTER");

	//$DD 共有解除準備待ちか共有解除の待ちがある場合に実施
	while ((unshare_watch_bit || xdev->txrx_stop_req_bit)
	       && (wait_time < ES_UNSHARE_WATCH_WAIT_MAX_MSEC)
	    ) {
		//$DD 共有が解除されて監視のあるEPについて、バッファ初期化
		for (epidx = 0; epidx < (xdev->epnum_max); epidx++) {
			//$DD 共有解除準備待ちの確認
			if ((epidx != xdev->epid_self)
			    && test_bit(epidx, &xdev->txrx_stop_req_bit)) {
				if (!esmem_is_shared
				    (xdev->share_ptrs.share, epidx)
				    ||
				    (esmem_is_shared
				     (xdev->share_ptrs.share, epidx)
				     && (ES_RX_STOP_REQ_DONE & xdev->
					 ep_shm_info[epidx].rx.info->v1i.
					 rx_status))
				    ) {
					//$DD 対向のEPの共有状態がなんらかの理由で消えた(共有がなくなる、おそらくシャットダウン)
					//$DD 対向のEPで準備完了状態が設定された(共有かつDONE設定)
					//$DD   送受信バッファのアドレスを削除(解除済みなら無処理なので問題なし) esvmm_unregister_buff_addr()
					mutex_lock(&xdev->share_ptrs.lock);
					ret =
					    esvmm_unregister_buff_addr(xdev->
								       base,
								       epidx,
								       &xdev->
								       share_ptrs);
					switch (ret) {
					case 0:
						//$DD 正常系 無処理
						break;
					default:
					case -ENOMSG:
					case -EBUSY:
						//$DD デフォルト
						//$DD インターフェイスエラー(バッファ長不正)
						//$DD タイムアウト
						//$DD エラーリカバリとして、強制リセット付きでクローズを行う。ワークキューへクローズ突き放しを登録
						if (!work_pending
						    (&priv->force_close)) {
							PDEBUG(PD_COMMON,
							       "schedule work queue");
							priv->force_reset =
							    true;
							//$DD ドライバ固有統計(device_force_reset_scheduled)更新
							xdev->drv_stats.
							    device_force_reset_scheduled++;

							schedule_work(&priv->
								      force_close);
						}
						break;
					}
					mutex_unlock(&xdev->share_ptrs.lock);

					//$DD ドライバ固有統計(command_unregister_buffer_executed)更新
					ES_DRV_CHARACTERISTIC_STATS_INC
					    (container_of
					     (xdev, struct esdrv_priv,
					      xsdevice), epidx,
					     command_unregister_buffer_executed,
					     1);

					//$DD rx_statusの排他設定	spin_lock_irqsave
					spin_lock_irqsave(&priv->xsdevice.rx_status_lock, flags);
					//$DD バッファを再初期化       esmem_epbuf_setup()
					esmem_epbuf_setup(&
							  (xdev->
							   ep_shm_info[epidx].
							   tx),
							  xdev->container_dev->
							  dev_addr,
							  xdev->container_dev->
							  mtu);
					//$DD rx_statusの排他解除	spin_unlock_irqrestore
					spin_unlock_irqrestore(&priv->xsdevice.rx_status_lock, flags);

					//$DD 共有解除予約をクリア
					clear_bit(epidx,
						  &xdev->txrx_stop_req_bit);
					//$DD 共有解除監視と共有削除予約とをクリア
					clear_bit(epidx, &unshare_watch_bit);
					clear_bit(epidx,
						  &priv->xsdevice.share_ptrs.
						  buffer_unshare_reserve_bit);

					PDEBUG_XDEV(PD_COMMON
						    || PD_STATUS,
						    "unreg done (EPID %d)EP_PARTNER_STATUS:%d",
						    epidx,
						    EP_PARTNER_STATUS(priv,
								      epidx));
				} else {
					// 対向のEPで準備完了がまだなので、監視継続
					PDEBUG_XDEV(PD_COMMON
						    || PD_STATUS,
						    "unreg undone (EPID %d)EP_PARTNER_STATUS:%d",
						    epidx,
						    EP_PARTNER_STATUS(priv,
								      epidx));
				}
			}
			//$DD 共有解除の待ちの確認
			if ((epidx != xdev->epid_self)
			    && !esmem_is_shared(xdev->share_ptrs.share, epidx)
			    && test_bit(epidx, &unshare_watch_bit)) {
				//$DD 共有監視による共有削除を設定しているなら
				if (test_bit
				    (epidx,
				     &priv->xsdevice.share_ptrs.
				     buffer_unshare_reserve_bit)) {
					//$DD   送受信バッファのアドレスを削除   esvmm_unregister_buff_addr()
					mutex_lock(&xdev->share_ptrs.lock);
					ret =
					    esvmm_unregister_buff_addr(xdev->
								       base,
								       epidx,
								       &xdev->
								       share_ptrs);
					switch (ret) {
					case 0:
						//$DD 正常系 無処理
						break;
					default:
					case -ENOMSG:
					case -EBUSY:
						//$DD デフォルト
						//$DD インターフェイスエラー(バッファ長不正)
						//$DD タイムアウト
						//$DD エラーリカバリとして、強制リセット付きでクローズを行う。ワークキューへクローズ突き放しを登録
						if (!work_pending
						    (&priv->force_close)) {
							PDEBUG(PD_COMMON,
							       "schedule work queue");
							priv->force_reset =
							    true;
							//$DD ドライバ固有統計(device_force_reset_scheduled)更新
							xdev->drv_stats.
							    device_force_reset_scheduled++;

							schedule_work(&priv->
								      force_close);
						}
						break;
					}
					mutex_unlock(&xdev->share_ptrs.lock);

					//$DD ドライバ固有統計(command_unregister_buffer_executed)更新
					ES_DRV_CHARACTERISTIC_STATS_INC
					    (container_of
					     (xdev, struct esdrv_priv,
					      xsdevice), epidx,
					     command_unregister_buffer_executed,
					     1);
				}

				//$DD rx_statusの排他設定	spin_lock_irqsave
				spin_lock_irqsave(&priv->xsdevice.rx_status_lock, flags);
				//$DD バッファを再初期化       esmem_epbuf_setup()
				esmem_epbuf_setup(&
						  (xdev->ep_shm_info[epidx].tx),
						  xdev->container_dev->dev_addr,
						  xdev->container_dev->mtu);
				//$DD rx_statusの排他解除	spin_unlock_irqrestore
				spin_unlock_irqrestore(&priv->xsdevice.rx_status_lock, flags);

				//$DD 共有解除予約をクリア
				clear_bit(epidx, &xdev->txrx_stop_req_bit);
				//$DD 共有解除監視と共有削除予約とをクリア
				clear_bit(epidx, &unshare_watch_bit);
				clear_bit(epidx,
					  &priv->xsdevice.share_ptrs.
					  buffer_unshare_reserve_bit);

				PDEBUG_XDEV(PD_COMMON
					    || PD_STATUS,
					    "unshare done (EPID %d)EP_PARTNER_STATUS:%d",
					    epidx, EP_PARTNER_STATUS(priv,
								     epidx));
			}
		}
		//$DD ディレイを入れる
		msleep(ES_UNSHARE_WATCH_WAIT_UNIT_MSEC);
		wait_time += ES_UNSHARE_WATCH_WAIT_UNIT_MSEC;
	}

	//$DD タイムアウト後共有解除のものが残っているなら、強制解除する
	if (priv->xsdevice.share_ptrs.buffer_unshare_reserve_bit) {
		for (epidx = 0; epidx < (xdev->epnum_max); epidx++) {
			//$DD 共有解除の確認
			if ((epidx != xdev->epid_self)
			    && test_bit(epidx,
					&priv->xsdevice.share_ptrs.
					buffer_unshare_reserve_bit)) {
				//$DD   送受信バッファのアドレスを削除   esvmm_unregister_buff_addr()
				mutex_lock(&xdev->share_ptrs.lock);
				ret =
				    esvmm_unregister_buff_addr(xdev->base,
							       epidx,
							       &xdev->
							       share_ptrs);
				switch (ret) {
				case 0:
					//$DD 正常系 無処理
					break;
				default:
				case -ENOMSG:
				case -EBUSY:
					//$DD デフォルト
					//$DD インターフェイスエラー(バッファ長不正)
					//$DD タイムアウト
					//$DD エラーリカバリとして、強制リセット付きでクローズを行う。ワークキューへクローズ突き放しを登録
					if (!work_pending(&priv->force_close)) {
						PDEBUG(PD_COMMON,
						       "schedule work queue");
						priv->force_reset = true;
						//$DD ドライバ固有統計(device_force_reset_scheduled)更新
						xdev->drv_stats.
						    device_force_reset_scheduled++;

						schedule_work(&priv->
							      force_close);
					}
					break;
				}
				mutex_unlock(&xdev->share_ptrs.lock);

				//$DD ドライバ固有統計(command_unregister_buffer_executed)更新
				ES_DRV_CHARACTERISTIC_STATS_INC(container_of
								(xdev,
								 struct
								 esdrv_priv,
								 xsdevice),
								epidx,
								command_unregister_buffer_executed,
								1);

				//$DD rx_statusの排他設定	spin_lock_irqsave
				spin_lock_irqsave(&priv->xsdevice.rx_status_lock, flags);
				//$DD バッファを再初期化       esmem_epbuf_setup()
				esmem_epbuf_setup(&
						  (xdev->ep_shm_info[epidx].tx),
						  xdev->container_dev->dev_addr,
						  xdev->container_dev->mtu);
				//$DD rx_statusの排他解除	spin_unlock_irqrestore
				spin_unlock_irqrestore(&priv->xsdevice.rx_status_lock, flags);

				//$DD 共有解除予約をクリア
				clear_bit(epidx, &xdev->txrx_stop_req_bit);
				//$DD 共有解除監視と共有削除予約とをクリア
				clear_bit(epidx, &unshare_watch_bit);
				clear_bit(epidx,
					  &priv->xsdevice.share_ptrs.
					  buffer_unshare_reserve_bit);

				PDEBUG_XDEV(PD_COMMON
					    || PD_STATUS,
					    "unshare force done (EPID %d)EP_PARTNER_STATUS:%d",
					    epidx, EP_PARTNER_STATUS(priv,
								     epidx));
			}
			//$DD 送受信停止完了状態での監視タイムアウトで共有ありに戻す(ただし強制共有解除以外)
			if ((epidx != xdev->epid_self)
			    && test_bit(epidx, &unshare_watch_bit)) {
				//$DD rx_statusの排他設定	spin_lock_irqsave
				spin_lock_irqsave(&priv->xsdevice.rx_status_lock, flags);
				priv->xsdevice.ep_shm_info[epidx].tx.info->v1i.
				    rx_status &= ~ES_RX_STOP_REQ_DONE;
				//$DD rx_statusの排他解除	spin_unlock_irqrestore
				spin_unlock_irqrestore(&priv->xsdevice.rx_status_lock, flags);

				PDEBUG_XDEV(PD_COMMON
					    || PD_STATUS,
					    "unshare timeout COMPLETE recoverd (EPID %d)EP_PARTNER_STATUS:%d",
					    epidx, EP_PARTNER_STATUS(priv,
								     epidx));
			}
		}
	}
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．３．２４ 送受信中止要求があったか確認する
  シーケンス再検討中

  (送受信中止要求  受付側)
  送信処理、受信処理内で中止要求を確認する

  @param[in]     priv		net_device構造体のプライベート領域
  @param[in]     src_epid	要求元のEPID

  @return  送受信中止要求の有無
  @retval  true		有り
  @retval  false	無し
**/
FUNC_STATIC bool esif_epstop_requested(struct esdrv_priv *priv, epno_t src_epid)
{
	//$DD 全EPの送受信中止要求から該当EPを抜き出す
	return test_bit(src_epid, &priv->xsdevice.txrx_stop_req_bit);
}

//$DD-----------------------------------------------------------------------------
/**$DD
	２．３．２５ 対向への送受信停止完了通知状態確認
	
	送受信中止処理で、相手へ完了を出しているか確認する

	rx_statusチェック用

  @param	src_epid	要求先の相手のEPID

  @return  送受信中止完了状態の有無
  @retval  true		有り
  @retval  false	無し
**/
FUNC_STATIC bool esif_epstop_process_done(struct esdrv_priv * priv,
					  epno_t src_epid)
{
	//$DD 対象EP向けの共有メモリのTxリングでのrx_statusが完了状態かを確認し、返す
	return (ES_RX_STOP_REQ_DONE & priv->xsdevice.ep_shm_info[src_epid].tx.
		info->v1i.rx_status);
}

//$DD=============================================================================
//$DD ２．２ ドライバインターフェイス
//$DD Linuxネットワークドライバとして必要な機能

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．１ モジュールのカーネルへの登録

  insmodによりモジュールがカーネルにロードされる直前の処理
  ロード可能かのチェック、net_device構造体の初期化、デバイス登録を行う

  @return  エラーコード
  @retval  0		正常終了(モジュールロード成功)
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC __init int esdrv_module_init(void)
{
	int result = -EPERM;	//関数戻り値

	PDEBUG(PD_ENT, "ENTER");

	//$DD 名称とコピーライトのログ出力
	pr_info(ES_DRV_MSG_INTRO, ES_DRV_MSG_TIME);
	pr_info(ES_DRV_MSG_COPYRIGHT, ES_DRV_MSG_TIME);

	//$DD モジュールロード可能か確認する
	{
		//$DD ESデバイスが存在するか  esvmm_exist_device()
		if (!esvmm_exist_device()) {
			PDEBUG(PD_INF, "exist fail");
			//$DD 存在しなければエラーコード(ENODEV)を設定してエラー終了処理へ移る
			result = -ENODEV;
			//$DD デバイスが存在しないログ
			pr_info(ES_DRV_MSG_DEV_NOTFOUND, ES_DRV_MSG_TIME);
			//printk(KERN_ERR "ESDRV: error %d es_device is not exist\n", result);
			goto out;
		}
		//$DD ESデバイスが有効かどうか       esvmm_is_device_valid()
		if (!esvmm_is_device_valid()) {
			PDEBUG(PD_INF, "valid fail");
			//$DD EPからでなければエラーコード(EACCES)を設定してエラー終了処理へ移る
			result = -EACCES;
			//$DD デバイスが無効化されているログ
			pr_info(ES_DRV_MSG_DEV_NOTFOUND, ES_DRV_MSG_TIME);
			//printk(KERN_ERR "ESDRV: error %d not EP\n", result);
			goto out;
		}
	}

	//$DD net_device構造体の初期化・デバイスの登録を行う  esdrv_module_init_device()
	//$DD 登録失敗ならばエラー終了処理へ移る、エラーコードはesdrv_module_init_device()の戻り値
	if ((result = esdrv_module_init_device(&esdrv_device)))
		goto out;
	
	//$DD logのページ情報を内部情報ログとして出力する
	{
		struct esdrv_priv *priv = netdev_priv(esdrv_device);
		phys_addr_t buff_page_addr = virt_to_phys(priv->log.buffer_page_address);
		pr_info(ES_DRV_MSG("alloc param : addr:%p(phys:%pa,[0]:%pa),num:%ld,buffer:%p"),ES_DRV_MSG_TIME,
			priv->log.buffer_page_address,
			&(buff_page_addr),
			&(priv->log.buffer_page_address[0]),
			ES_DRV_LOG_MSG_PAGE_NUM,
			priv->log.buffer);
	}

	//$DD 関数正常終了
	return 0;

	//$DD エラー終了処理
 out:
	//$DD 終了処理(モジュールのカーネルからの削除処理)を呼出す     esdrv_module_exit()
	esdrv_module_exit();
	//$DD エラーで終了
	//$DD 失敗のログ出力
	pr_err(ES_DRV_MSG_INSMOD_FAIL, ES_DRV_MSG_TIME);
	return result;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．２ デバイスの初期化(登録)

  insmodによりモジュールがカーネルにロードされる直前の処理
  net_device構造体の初期化、es_device情報の設定、デバイスの登録を行う

  @param[out]    pdev		作成されるnet_device構造体

  @return  エラーコード
  @retval  0		正常終了(モジュールロード成功)
  @retval  負の値	linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC __init int esdrv_module_init_device(struct net_device **pdev)
{
	struct net_device *dev;
	struct esdrv_priv *priv;
	int result = -EPERM;	//関数戻り値
	char *device_name = DEVICE_NAME;

	PDEBUG(PD_ENT, "ENTER");

	//$DD net_device構造体のメモリ割り当てと初期化を行う  alloc_netdev()
	//$DD 初期化はモジュールの初期化ハンドラ内で行う
	*pdev =
	    alloc_netdev_mq(sizeof(struct esdrv_priv), device_name,
			    esdrv_netdev_setup, NUM_QUEUE);
	dev = *pdev;

	//$DD 割り当てに失敗したらエラー(ENOMEM)で終了する

	if (*pdev == NULL) {
		//$DD メモリ確保失敗ログ
		pr_err(ES_DRV_MSG_MEMFAIL1, ES_DRV_MSG_TIME, ES_DRV_MSG_MEMFAIL1_NETDEV);
		return -ENOMEM;
	}

	priv = netdev_priv(*pdev);

	//$DD ワークキュー作成に失敗
	if ((priv->txrx_wq == NULL) || (priv->control_wq == NULL)) {
		//$DD メモリ確保失敗ログ
		//pr_err(ES_DRV_MSG_MEMFAIL1, ES_DRV_MSG_TIME,);
		//pr_err(ES_DRV_MSG_INTERNAL_ERR1, ES_DRV_MSG_TIME,);
		ES_DRV_LOGGING("%s Intarnal=workqueue,txrx=%p,control=%p",
			__func__, priv->txrx_wq ,priv->control_wq);
		return -ENOMEM;
	}
	//$DD es_device情報を設定する    esif_init_device()
	//$DD (es_deviceの領域確保・０クリアはesdrv_netdev_setup()で実行済)
	if ((result = esif_init_device(&priv->xsdevice))) {
		//$DD 設定に失敗したらエラーで終了する、エラーコードはesdrv_netdev_setup()の戻り値
		PDEBUG(PD_INF, "init device fail");
		//printk(KERN_ERR "ESDRV: error %d esif_init_device device '%s'\n", result, (*pdev)->name);
		return result;
	}
	//アドレスは先に設定する     
	//$DD MACアドレスを設定する   dev->dev_addr 02:00:00:00:00:[epid]
	dev->dev_addr[0] = 2;
	dev->dev_addr[1] = 0;
	dev->dev_addr[2] = 0;
	dev->dev_addr[3] = 0;
	dev->dev_addr[4] = 0;
	dev->dev_addr[5] = priv->xsdevice.epid_self;	//最下位バイトはEPID

	//$DD メモリ確保を行う  esif_setup()
	if ((result = esif_setup(&priv->xsdevice))) {
		//$DD 登録に失敗したらエラーで終了する、エラーコードはesif_setup()の戻り値
		PDEBUG(PD_INF, "esif setup fail");
		//printk(KERN_ERR "ESDRV: error %d allocation memory '%s'\n",
		//              result, (*pdev)->name);
		return result;
	}
	//$DD ネットワークデバイスを登録する     register_netdev()
	if ((result = register_netdev(*pdev))) {
		//$DD 登録に失敗したらエラーで終了する、エラーコードはregister_netdev()の戻り値(ES_DRV_INTERNAL_ERR_NETDEV_REG = 2)
		//pr_err(ES_DRV_MSG_INTERNAL_ERR1, ES_DRV_MSG_TIME, ES_DRV_INTERNAL_ERR_NETDEV_REG);
		ES_DRV_LOGGING("%s Intarnal=%d,result=%d",
			__func__, ES_DRV_INTERNAL_ERR_NETDEV_REG, result);
		PDEBUG(PD_INF, "device register fail");
		//printk(KERN_ERR "ESDRV: error %d registering device '%s'\n",
		//              result, (*pdev)->name);
		return result;
	}
	//$DD 初期状態としてcarrier_off  netif_carrier_off()
	netif_carrier_off(dev);

	//$DD procファイルシステム作成        esdrv_proc_create()
	esdrv_proc_create(*pdev);

	//$DD 正常終了
	return 0;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．３  net_device構造体の初期化

  alloc_netdev()から呼ばれる
  net_device構造体のメンバーの初期化を行う(変数、ハンドラ関数)

  @param[in,out] dev	net_device構造体
**/
FUNC_STATIC void esdrv_netdev_setup(struct net_device *dev)
{
#if !defined(ESDRV_ETHERTOOL_OFF)
	extern const struct ethtool_ops esdrv_ethtool_ops;	//ethtoolのセクションで定義
#endif				/* !defined(ESDRV_ETHERTOOL_OFF) */
	struct esdrv_priv *priv = netdev_priv(dev);
	int i;

	PDEBUG(PD_ENT, "ENTER");

	//$DD net_device構造体のEthernet用の基本設定をする    ether_setup()
	ether_setup(dev);

	//$DD net_device構造体の初期化
	//$DD 送信リトライ時間 dev->watchdog_timeoに初期値の10を設定
	dev->watchdog_timeo = ES_TX_RETRY_INTERVAL;
	PDEBUG_DEV(PD_COMMON, "[jiffy] HZ=%d , retry = %d , timeout = %d", HZ,
		   ES_TX_RETRY_INTERVAL, ES_TX_RETRY_TIMEOUT);

	//$DD ハンドラ関数 dev->netdev_opsにesdrv_netdev_opsを設定
	dev->netdev_ops = &esdrv_netdev_ops;
	//$DD MTU値 dev->mtuに初期MTU値として8Kを設定
	dev->mtu = ep_support_mtu[0];
	// broadcastは常に扱えないといけない(ex ARPなど)
	dev->flags |= IFF_BROADCAST;

#if !defined(ESDRV_ETHERTOOL_OFF)
	//$DD   ethtoolのサポート
	dev->ethtool_ops = &esdrv_ethtool_ops;
#endif				/* !defined(ESDRV_ETHERTOOL_OFF) */

#ifndef ESDRV_VLAN_OFF
	//$DD VLAN設定 dev->featuresにNETIF_F_HW_VLAN_CTAG_FILTER(NETIF_F_HW_VLAN_FILTER)を追加
	//$DD LinuxのVLAN設定API ndo_vlan_rx_add_vid/ndo_vlan_rx_kill_vidは受信のためのフィルタ設定をハードに行うためのインターフェイスで、これを有効にするNETIF_F_HW_VLAN_STAG_FILTER(NETIF_F_HW_VLAN_FILTER)が必要。
	//$DD このドライバではVLANは受信側として設定を行い、送信側が「相手が受け取れるVLAN_IDの有無」を見て送信をフィルタする構造となっているため、この設定を有効として処理を実装している
	//$DD またSTAGについては対応していないので設定しないとする。
	dev->features |= NETIF_F_HW_VLAN_CTAG_FILTER;
#endif				/* !ESDRV_VLAN_OFF */

	//$DD プライベート領域の初期化
	//$DD プライベート領域の初期化(0クリア)
	memset(priv, 0, sizeof(struct esdrv_priv));

	//$DD ロックの初期化
	mutex_init(&priv->xsdevice.share_ptrs.lock);	// コマンドの排他ロック

	spin_lock_init(&priv->xsdevice.rx_status_lock); // rx_status

	//$DD net_device逆引きの設定
	priv->xsdevice.container_dev = dev;

	//$DD トレースの開始状態初期化
	priv->xsdevice.trace_start = false;

	//$DD 強制クローズ処理ワークキューエントリへのタスク登録       INIT_WORK()でesdrv_force_close_work初期化
	INIT_WORK(&priv->force_close, esdrv_force_close_work);
	//$DD 強制リセット、オープン抑制フラグを初期化(false)
	priv->force_reset = false;
	priv->open_guard = false;

	//$DD ワークキュー構造体初期化      create_workqueue(name)
	priv->txrx_wq = create_workqueue(ES_DRV_MODULE_NAME "/txrx");
	priv->control_wq = create_workqueue(ES_DRV_MODULE_NAME "/control");

	//$DD 送信まとめ ワークキューエントリへのタスク登録       INIT_WORK()でesdrv_tx_stall初期化
	INIT_WORK(&priv->tx_stall_work, esdrv_tx_stall);
	//$DD 受信監視 ワークキューエントリへのタスク登録  INIT_WORK()でesif_raise_intr_rxdata_work初期化
	INIT_WORK(&priv->raise_intr_rxdata_work, esif_raise_intr_rxdata_work);
	//$DD 共有状態監視 ワークキュー初期化    INIT_WORK()でesif_watch_unshare初期化
	INIT_WORK(&priv->unshare_watch_work, esif_watch_unshare);
	priv->unshare_watch_bit = 0;

	//$DD Zone更新処理 ワークキュー初期化      INIT_WORK()でesif_update_zone_work初期化
	INIT_WORK(&priv->xsdevice.update_zone_work, esif_update_zone_work);
	//$DD 共有中止処理ワークキューの初期化  INIT_WORK()でesif_epstop_work初期化
	INIT_WORK(&priv->xsdevice.workqueue_epstop, esif_epstop_work);

	//$DD 割り込みの遅延監視ワークキュー初期化    INIT_DELAYED_WORK()でesdrv_irq_status_work初期化
	INIT_DELAYED_WORK(&priv->interrupt_watch_work, esdrv_irq_status_work);
	priv->interrupt_watch_enable = false;

	//$DD NAPIポーリング関数を登録する    netif_napi_add()
	netif_napi_add(dev, &priv->napi, esdrv_rx_napi_poll,
		       ES_DRV_NAPI_POLLING_BUDGET);
	//PDEBUG_DEV(PD_INF,"netif_napi_add  state=%lx", priv->napi.state );
	
	memset(&(priv->log),0,sizeof(priv->log));
	//$DD logの初期値
	priv->log.max = ES_DRV_LOG_MSG_MAX;
	atomic_set(&(priv->log.index),0);
	//$DD log用のロード時間保持
	priv->log.start_time = current_kernel_time();
	time_to_tm(priv->log.start_time.tv_sec,0,&(priv->log.start_tm));
	priv->log.start_jiffiy = get_jiffies_64();

	//$DD log領域確保(バッファ)
	priv->log.buffer = (u8 *)vzalloc(priv->log.max * ES_DRV_LOG_LINESIZE);
	
	//$DD logのバッファのページアドレス保持(先頭からページ単位で取得)
	for (i = 0; i < ES_DRV_LOG_MSG_PAGE_NUM; i++) {
		void *addr = &(priv->log.buffer[i * PAGE_SIZE]);
		priv->log.buffer_page_address[i] =
		    (phys_addr_t) (page_to_phys(vmalloc_to_page(addr)) +
			      offset_in_page(addr));
	}

	
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．４  モジュールのカーネルからの削除処理

  rmmodによりモジュールがカーネルから削除される直前の処理
  net_device構造体等のメモリ解放
  デバイス削除
  
  @note
  	esdrv_module_init()から参照されるので__exitは付加しない
**/
FUNC_STATIC void esdrv_module_exit(void)
{
	PDEBUG(PD_ENT, "ENTER");

	//$DD デバイス毎の削除  esdrv_module_exit_device()
	esdrv_module_exit_device(esdrv_device);
	PDEBUG(PD_INF, "done");

	return;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．５  デバイスの削除

  モジュールのカーネルからの削除で呼び出され、デバイスの削除を行う

  @param[in,out] dev	net_device構造体
**/
FUNC_STATIC void esdrv_module_exit_device(struct net_device *dev)
{
	struct esdrv_priv *priv;

	PDEBUG(PD_ENT, "ENTER");

	CHKPOINT();		//チェックポイントのテスト
	if (dev) {
		PDEBUG(PD_INF, "exit device");

		priv = netdev_priv(dev);

		//$DD procファイルシステム削除        esdrv_proc_remove()
		esdrv_proc_remove();

		//$DD ネットワークデバイスが登録されていれば削除、このとき、unregister_netdev->rollback_registeredの経路でcloseがここから呼ばれるので、close時に必要なリソースの解放はこれ以降に行う
		if (dev->reg_state == NETREG_REGISTERED) {
			PDEBUG(PD_INF, "unregister device");
			CHKPOINT();	//チェックポイントのテスト
			//$DD ネットワークデバイスを削除する      unregister_netdev()
			unregister_netdev(dev);
		}
		//$DD ワークキュー構造体破棄 destroy_workqueue(*wq)
		if (priv->txrx_wq != NULL) {
			destroy_workqueue(priv->txrx_wq);
			priv->txrx_wq = NULL;
		}
		if (priv->control_wq != NULL) {
			destroy_workqueue(priv->control_wq);
			priv->control_wq = NULL;
		}
		//$DD NAPI終了処理  netif_napi_del()
		netif_napi_del(&priv->napi);

		//$DD デバイスとして終了処理 esif_exit_device()
		esif_exit_device(&priv->xsdevice);

		//$DD バッファメモリの解放を行う   esif_free()
		esif_free(dev);
		
		//$DD log のバッファ解放
		if(priv->log.buffer){
			vfree(priv->log.buffer);
			priv->log.buffer = NULL;
		}

		//$DD net_device構造体のメモリからの解放を行う    free_netdev()
		free_netdev(dev);
	}

	return;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．６  デバイス起動

  net_device.netdev_ops.ndo_openに本ハンドラ関数のポインタを登録する
  ifconfigによりデバイスを起動する直前の処理

  この呼出しにより、アプリケーションでソケットが使用可能になる
  es_deviceとの通信初期化処理を行う

  @param[in,out] dev	net_device構造体

  @return  エラーコード
  @retval  0		正常終了(モジュールロード成功)
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esdrv_open(struct net_device *dev)
{
	int result;
	struct esdrv_priv *priv = netdev_priv(dev);

	//PDEBUG_DEV(PD_ENT,"ENTER name=%s, irq=%u flags=(%lx)%s", dev->name, g_irqno_rx, irqFlags, prmIrqFlags);
	PDEBUG(PD_ENT, "ENTER");

	//$DD 0.オープン禁止フラグが有効な場合、エラーログを出力して終了(ES_DRV_INTERNAL_ERR_OPEN_GUARD = 20)
	if (priv->open_guard) {
		pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME,
		       ES_DRV_INTERNAL_ERR_OPEN_GUARD);
		ES_DRV_LOGGING("%s NotReady=%d",
			__func__,ES_DRV_INTERNAL_ERR_OPEN_GUARD);
		return -ENXIO;
	}
	//$DD 1.内部向けの初期値設定

	//$DD es_deviceの使用開始処理、NGでエラー終了へ  esif_open() (ES_DRV_INTERNAL_ERR_ESIF_OPEN  = 3)
	result = esif_open(&priv->xsdevice);
	if (result) {
		pr_err(ES_DRV_MSG_NOTREADY_ERR1, ES_DRV_MSG_TIME, ES_DRV_INTERNAL_ERR_ESIF_OPEN);
		ES_DRV_LOGGING("%s NotReady=%d,result=%d",
			__func__,ES_DRV_INTERNAL_ERR_ESIF_OPEN,result);
		goto out1;
	}
	//$DD 2.送受信系の動作開始

	//$DD 送受信停止停止要求フラグをクリア
	priv->xsdevice.txrx_stop_req_bit = 0;
	//$DD 合わせて中止処理ワークキュー通知ビットマップ初期化
	priv->xsdevice.epstop_req_bit = 0;

	//$DD NAPI機能を有効化する      napi_enable()
	napi_enable(&priv->napi);

	//$DD ステータスを読み捨て    esvmm_interrupt_capture()
	esvmm_interrupt_capture(priv->xsdevice.base);

	//$DD 割り込みハンドラ登録、NGでエラー終了へ  esif_epirq_init() (ES_DRV_INTERNAL_ERR_IRQ_INIT = 4)
	result = esif_epirq_init(dev);
	if (result) {
		//pr_err(ES_DRV_MSG_INTERNAL_ERR1, ES_DRV_MSG_TIME, ES_DRV_INTERNAL_ERR_IRQ_INIT);
		ES_DRV_LOGGING("%s Intarnal=%d,result=%d",
			__func__, ES_DRV_INTERNAL_ERR_IRQ_INIT, result);
		goto out2;	//エラー処理して終了
	}
	//$DD 3.割り込みマスク解除で全体の動作開始

	//$DD 全割り込み発生を許可する      esvmm_irqmask(false)
	esvmm_irqmask(priv->xsdevice.base, INTREG_IRQMASK_ALL, false);

	//$DD 上位層から本ドライバへの送信フレーム引き渡しを許可する      netif_tx_start_all_queues()とnetif_carrier_on()
	netif_tx_start_all_queues(dev);
	netif_carrier_on(dev);

	//$DD ドライバ固有統計(open)更新
	priv->xsdevice.drv_stats.open++;

	//$DD ifupのログ出力
	pr_info(ES_DRV_MSG_LINKUP, ES_DRV_MSG_TIME);

	//$DD 関数終了(正常)
	return 0;

	//$DD エラー終了処理。エラーコードは、エラー終了移動元の関数の返却値
 out2:
	//$DD 割り込みの終了処理(内部で登録状態チェックはしているため、不安定な場合でも実行可能)        esif_epirq_cleanup()
	esif_epirq_cleanup(dev);

	//$DD NAPIポーリングを停止する        napi_disable()
	PDEBUG_DEV(PD_COMMON || PD_DEV_STOP, "napi disable start");
	napi_disable(&priv->napi);

	//$DD 送受信の解除系は不要(まだ通信していないため)

 out1:
	//$DD es_deviceの共有終了処理    esif_close()
	esif_close(dev);

	return result;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．７  デバイス停止

  net_device.netdev_ops.ndo_stopに本ハンドラ関数のポインタを登録する
  ifconfig downによりデバイスを停止する直前の処理

  この呼出しにより、アプリケーションでソケットが使用不可能になる
  es_deviceとの通信終了処理を行う
	
  @param[int,out] dev	net_device構造体

  @return  エラーコード
  @retval  0		正常終了(モジュールロード成功)
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esdrv_stop(struct net_device *dev)
{
	//タイムアウトの考慮が必要

	struct esdrv_priv *priv = netdev_priv(dev);
	epno_t epidx;
	unsigned long flags;

	PDEBUG_DEV(PD_ENT || PD_COMMON || PD_DEV_STOP, "ENTER");

	//$DD 上位プロトコル層からの送信キュー送出を停止する      netif_tx_stop_all_queues()と netif_carrier_off()
	PDEBUG_DEV(PD_COMMON || PD_DEV_STOP, "netif_tx_stop_all_queues start");
	netif_tx_stop_all_queues(dev);
	netif_carrier_off(dev);

	//$DD 1.共有解除開始を通知

	//$DD 共有中のEPに対して受信中止(共有解除)要求割り込みを発生する     esif_epstop_raise()
	PDEBUG_DEV(PD_COMMON || PD_DEV_STOP, "esif_epstop_raise()");
	esif_epstop_raise(&priv->xsdevice);

	//$DD 2.送受信系の停止

	//$DD NAPIポーリングを停止する        napi_disable()
	PDEBUG_DEV(PD_COMMON || PD_DEV_STOP, "napi disable start");
	napi_disable(&priv->napi);
	//$DD 全他EP向けtxのrx_statusから(ES_RX_POLL_WORK)を解除 (NAPI中断がありえるので)
	//$DD rx_statusの排他設定	spin_lock_irqsave
	spin_lock_irqsave(&priv->xsdevice.rx_status_lock, flags);
	for (epidx = 0; epidx < priv->xsdevice.epnum_max; epidx++) {
		//$DD 自分のEP IDは処理しない
		if (epidx == priv->xsdevice.epid_self) {
			continue;
		}
		if (EP_PARTNER_SHARED == EP_PARTNER_STATUS(priv, epidx)) {
			priv->xsdevice.ep_shm_info[epidx].tx.info->v1i.rx_status &=
			    ~ES_RX_POLL_WORK;
		}
	}
	//$DD rx_statusの排他解除	spin_unlock_irqrestore
	spin_unlock_irqrestore(&priv->xsdevice.rx_status_lock, flags);

	//$DD 割り込みの終了処理       esif_epirq_cleanup()
	esif_epirq_cleanup(dev);

	//$DD ペンディング中の物はキャンセルし、実行中の物は処理の終了まで待つ
	//$DD 呼出し元の可能性があり、強制クローズ処理ワークキューエントリはキャンセルしない(システムのワークキューかつデバイスの処理とは関係しないため安全)
	//$DD 送信まとめ ワークキュー停止
	cancel_work_sync(&priv->tx_stall_work);
	//$DD 受信監視 ワークキュー停止
	cancel_work_sync(&priv->raise_intr_rxdata_work);
	//$DD 共有状態監視 ワークキュー停止
	cancel_work_sync(&priv->unshare_watch_work);
	priv->unshare_watch_bit = 0;

	//$DD Zone更新処理 ワークキュー停止
	cancel_work_sync(&priv->xsdevice.update_zone_work);
	//$DD 共有中止処理 ワークキュー停止
	cancel_work_sync(&priv->xsdevice.workqueue_epstop);

	//$DD 3.共有解除

	//$DD 共有中の全EPの受信ステータスが受信中止完了になるまで待つ       esif_epstop_wait()
	PDEBUG_DEV(PD_COMMON || PD_DEV_STOP, "esif_epstop_wait()");
	if (esif_epstop_wait(&priv->xsdevice)) {
		PDEBUG(PD_INF, "epstop timeout");
		//タイムアウト
		//現状処理無し
	}
	//$DD es_deviceの共有終了処理    esif_close()
	esif_close(dev);

	//$DD ドライバ固有統計(close)更新
	priv->xsdevice.drv_stats.close++;

	//$DD ifdownのログ出力
	pr_info(ES_DRV_MSG_LINKDOWN, ES_DRV_MSG_TIME);

	//$DD 関数終了(正常)
	PDEBUG_DEV(PD_COMMON || PD_DEV_STOP, "closed");
	return 0;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．８  送信キューの振り分け

	欠番
	
	現状では欠番として詳細には記載すること
	
  net_device.netdev_ops.ndo_select_queueに本ハンドラ関数のポインタを登録する
  送信先EPによりキューを振り分ける

  @param[in,out] dev	net_device構造体
  @param[in]     skb	ソケットバッファ

  @return  キュー番号
**/
#if 0				/* 現状不要になった */
FUNC_STATIC u16 esdrv_select_queue(struct net_device * dev,
				   struct sk_buff * skb)
{
	//$DD 送信キュー番号を決定する
	//$DD ユニキャスト時：送信先EPIDより変換
	//$DD ブロードキャスト・マルチキャスト時：専用のキュー番号
}
#endif				/* 0 */

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．９  フレーム送信

  net_device.netdev_ops.ndo_start_xmitに本ハンドラ関数のポインタを登録する
  es_deviceにフレーム(Ethernetフレーム)を送信する
  
  送信先が停止中の場合、ソケットバッファを捨てて正常終了する(短時間で再送信できる見込みが薄い為)
  送信バッファ(送信先の受信バッファ)フルの場合リトライを要求して終了する

  @param[in]     skb	ソケットバッファ
  @param[in,out] dev	net_device構造体

  @return  送信処理結果
  @retval  NETDEV_TX_OK		正常終了(又は送信先が停止中)
  @retval  NETDEV_TX_BUSY	ビジー、カーネルにリトライ要求(バッファフル)
**/
FUNC_STATIC netdev_tx_t esdrv_start_xmit(struct sk_buff *skb,
					 struct net_device *dev)
{
	static int dbgCnt = 0;
	int len;
	char *data, shortpkt[VLAN_ETH_HLEN];
	struct esdrv_priv *priv = netdev_priv(dev);
//      u16 queueNo = skb_get_queue_mapping(skb);       //キュー番号取得
	u16 queueNo = 0;
	struct netdev_queue *curQueue = netdev_get_tx_queue(dev, queueNo);
	netdev_tx_t retStatus = NETDEV_TX_OK;	//関数の終了ステータス
	struct ethhdr *eth;	//ethernetヘッダ
	epno_t dest_epid;	//送信先EP
	// 07/27追加
	epno_t maxepid, selfepid;
	u16 vlan_id = 0;
	bool vlan = false;
	bool is_multi = false;

//      PDEBUG_DEV(PD_ENT||PD_COMMON||PD_START_XMIT,"ENTER queueNo=%d",queueNo);
	PDEBUG_DEV(PD_ENT || PD_COMMON || PD_START_XMIT, "ENTER %d", dbgCnt++);

	//$DD 送信先イーサネットアドレスをEPIDに変換
	eth = (struct ethhdr *)skb->data;	//ethernetヘッダ
	selfepid = priv->xsdevice.epid_self;

	//$DD VLANのチェックとVLAN ID取得       vlan_get_tag()
	vlan = (0 == vlan_get_tag(skb, &vlan_id)) ? true : false;

	data = skb->data;
	len = skb->len;

	//$DD フレーム種別チェック
	if (is_multicast_ether_addr(eth->h_dest)) {
		//$DD マルチキャスト/ブロードキャストなので全EPを送信対象に設定      is_multicast_ether_addr()
		PDEBUG(PD_INF, "xmit multicast");

		dest_epid = 0;
		maxepid = priv->xsdevice.epnum_max;
		is_multi = true;
	} else if (is_local_ether_addr(eth->h_dest)) {
		//$DD 送信できるユニキャストなので対象EPのみ送信対象に設定     is_local_ether_addr()
		PDEBUG(PD_INF, "xmit local(uni)cast");

		dest_epid = eth->h_dest[ETH_ALEN - 1];	//送信先EP
		maxepid = dest_epid + 1;
		if ((0x02 == eth->h_dest[0])
		    && (0x00 == (eth->h_dest[1] | eth->h_dest[2] | eth->h_dest[3] | eth->h_dest[4]))	// 1-4 all 0
		    && (dest_epid < priv->xsdevice.epnum_max)) {
			// support epid
		} else {
			//$DD EPIDが範囲外の場合送信処理しないため送信対象設定を削除
			PDEBUG(PD_INF, "xmit local(uni)cast:drop");
			// unsupport epid
			dest_epid = 0;
			maxepid = 0;
			retStatus = NETDEV_TX_OK;

			//$DD 対象EPが存在しないでdropさせる場合ドライバ統計(tx_dropped_unknown_destination)を更新する
			priv->xsdevice.drv_stats.
			    tx_dropped_unknown_destination++;

			//$DD 統計情報(tx_packets,tx_bytes)を更新
			ES_DRV_STATS_INC(priv, selfepid, tx_packets, 1);
			ES_DRV_STATS_INC(priv, selfepid, tx_bytes, len);
		}
	} else {
		//$DD 送信できないユニキャストは送信処理しないため送信対象設定を削除
		PDEBUG(PD_INF, "xmit other:drop");
		// other nothing
		dest_epid = 0;
		maxepid = 0;
		retStatus = NETDEV_TX_OK;

		//$DD 対象EPが存在しないでdropさせる場合ドライバ統計(tx_dropped_unknown_destination)を更新する
		priv->xsdevice.drv_stats.tx_dropped_unknown_destination++;

		//$DD 統計情報(tx_packets,tx_bytes)を更新
		ES_DRV_STATS_INC(priv, selfepid, tx_packets, 1);
		ES_DRV_STATS_INC(priv, selfepid, tx_bytes, len);
	}

	//$DD 送信対象の各EPに送信処理実施
	for (; dest_epid < maxepid; dest_epid++) {
		//$DD 複数宛先処理のために、自EPID宛が発生した場合はスキップする
		if (selfepid == dest_epid) {
			// スキップ
			continue;
		}
		PDEBUG_DEV(PD_COMMON
			   || PD_START_XMIT, "dest_epid=%d", dest_epid);

		//$DD 以下の複数のチェックを行い、送信可能か判定する：
		//$DD 相手EPへ送信可能か確認(共有中か?、送受信中止要求が出ていないか?)
		//$DD バージョンのチェック(同じか?)
		//$DD MTUが同値か確認(同じか?)
		//$DD VLANフレームだった場合はesmem_has_vidに受信バッファを渡して、送信可能か確認(VLANが存在するか?)
		if (EP_PARTNER_SHARED != EP_PARTNER_STATUS(priv, dest_epid)) {
			//$DD 相手EPへ送信可能かチェックし、送信不能の場合
			PDEBUG_DEV(PD_COMMON
				   || PD_START_XMIT, "can't send:Don't Exist");
			//$DD 統計情報を更新
			if (!is_multi) {
				//$DD 対象EPが共有でない送信失敗でドライバ統計(tx_dropped_not_shared)を更新する(マルチキャスト/ブロードキャスト時は除外)
				ES_DRV_CHARACTERISTIC_STATS_INC(priv, dest_epid,
								tx_dropped_not_shared,
								1);
			}
			//$DD プロトコル層からの送信キューを捨てる(関数の戻り値を正常に設定する)
			retStatus = NETDEV_TX_OK;
		} else
		    if (!esmem_same_ver
			(&(priv->xsdevice.ep_shm_info[dest_epid].rx),
			 ES_DRV_MAJOR_NO)) {
			//$DD バージョンのチェックで送信不能の場合    esmem_same_ver()
			PDEBUG_DEV(PD_COMMON
				   || PD_START_XMIT, "can't send:Version");
			//$DD 統計情報を更新
			//$DD 通信可能状態ではない(共有状態になっていない、受信中止要求、バージョン違い)はtx_carrier_errors
			ES_DRV_STATS_INC(priv, dest_epid, tx_carrier_errors, 1);
			//$DD 対象EPがバージョン違いでの送信失敗でドライバ統計(tx_dropped_version_mismatch)を更新する
			ES_DRV_CHARACTERISTIC_STATS_INC(priv, dest_epid,
							tx_dropped_version_mismatch,
							1);

			//$DD プロトコル層からの送信キューを捨てる(関数の戻り値を正常に設定する)
			retStatus = NETDEV_TX_OK;
		} else
		    if (!esmem_same_mtu
			(&(priv->xsdevice.ep_shm_info[dest_epid].rx),
			 dev->mtu)) {
			//$DD MTUが同値か確認の結果、送信不能の場合    esmem_same_mtu()
			PDEBUG_DEV(PD_COMMON
				   || PD_START_XMIT, "can't send:MTU");
			//$DD 統計情報を更新
			//$DD MTU違いはtx_dropped、tx_errors
			ES_DRV_STATS_INC(priv, dest_epid, tx_dropped, 1);
			ES_DRV_STATS_INC(priv, dest_epid, tx_errors, 1);
			//$DD 対象EPとMTU違いでの送信失敗でドライバ統計(tx_dropped_buffer_size_discrepancy)を更新する
			ES_DRV_CHARACTERISTIC_STATS_INC(priv, dest_epid,
							tx_dropped_buffer_size_discrepancy,
							1);

			//$DD プロトコル層からの送信キューを捨てる(関数の戻り値を正常に設定する)
			retStatus = NETDEV_TX_OK;
		} else if ((true == vlan)
			   &&
			   !esmem_has_vid(&
					  (priv->xsdevice.
					   ep_shm_info[dest_epid].rx),
					  vlan_id)) {
			//$DD VLANフレームでVLAN IDがなく、送信不能の場合       !esmem_has_vid()
			PDEBUG_DEV(PD_COMMON
				   || PD_START_XMIT, "can't send:VLAN");
			PDEBUG(PD_COMMON, "drop unmatch epid:%d,vid:%d",
			       dest_epid, vlan_id);

			//$DD 統計情報を更新
			//$DD 対象EPとVLANが合わない送信失敗でドライバ統計(tx_dropped_vlan_id_mismatch)を更新する
			ES_DRV_CHARACTERISTIC_STATS_INC(priv, dest_epid,
							tx_dropped_vlan_id_mismatch,
							1);

			//$DD プロトコル層からの送信キューを捨てる(関数の戻り値を正常に設定する)
			retStatus = NETDEV_TX_OK;
		} else {
			//$DD 送信可能の場合
			PDEBUG_DEV(PD_COMMON
				   || PD_START_XMIT, "tx start epid=%d",
				   dest_epid);

			//$DD 送信フレームが当該ドライバがサポートする最小Ethernetフレームサイズ(src,dest,vlan_tag,type = 6+6+4+2 = 18)より小さい場合の処理
			//$DD 送信フレームを当該ドライバがサポートする最小Ethernetフレームサイズのバッファにコピー
			if (len < VLAN_ETH_HLEN) {
				memset(shortpkt, 0, VLAN_ETH_HLEN);
				memcpy(shortpkt, skb->data, skb->len);
				len = VLAN_ETH_HLEN;
				data = shortpkt;
			}
			//$DD 初めての送信か？ (送信済回数が１回以上ならばリトライ中)
			if (priv->tx_retry_count == 0) {
				//$DD 初回
				//$DD 送信開始時刻設定
				priv->tx_start_jiffy = jiffies;
				//$DD 送信済回数を１回に設定する
				priv->tx_retry_count = 1;
			} else {
				//$DD 2回目以降
				//$DD 送信済回数をカウントアップする
				priv->tx_retry_count++;
			}
			PDEBUG_DEV(PD_COMMON
				   || PD_START_XMIT, "tx start retry=%d",
				   priv->tx_retry_count);

			//$DD 送信先EP用の送信バッファに転送データを書き込み・送信     esif_tx_send()
			if (esif_tx_send(dev, dest_epid, data, len)) {
				//$DD エラー時の処理
				//$DD マルチキャスト/ブロードキャスト、もしくは送信タイムアウトか確認する 閾値100
				if (is_multi) {
					//$DD マルチキャスト/ブロードキャストで送信NGは関数の戻り値を正常に設定して次へ(単純無視される)
					retStatus = NETDEV_TX_OK;
				} else if (((long)jiffies - (long)priv->tx_start_jiffy) >= ES_TX_RETRY_TIMEOUT) {	//jiffiesのラップを考慮
					//$DD タイムアウト、プロトコル層からの送信フレームをキューから捨てる(関数の戻り値を正常に設定する)
					//$DD リトライタイムアウトはバッファフルでのtx_fifo_errors,tx_errors
					ES_DRV_STATS_INC(priv, dest_epid,
							 tx_fifo_errors, 1);
					ES_DRV_STATS_INC(priv, dest_epid,
							 tx_errors, 1);
					retStatus = NETDEV_TX_OK;
				} else {
					//$DD (タイムアウトしていない)バッファフルが発生した。リトライの遅延実行の設定を行う
					dev->trans_start = jiffies;	// save the timestamp
					PDEBUG_DEV(PD_COMMON
						   || PD_START_XMIT,
						   "buffer full retry (%ld)",
						   jiffies);

					//$DD 統計情報を更新
					//$DD 対象EPがバッファフルでドライバ統計(tx_buffer_full)を更新する
					ES_DRV_CHARACTERISTIC_STATS_INC(priv,
									dest_epid,
									tx_buffer_full,
									1);

					//$DD 送信キューを停止する
					//$DD 送信キューの再開は、一定時間後、フレーム送信リトライ関数 esdrv_tx_retry() で行う
					netif_tx_stop_queue(curQueue);

					//$DD 送信再開監視ワークキュー起動
					if (!work_pending(&priv->tx_stall_work)) {
						queue_work(priv->txrx_wq,
							   &priv->
							   tx_stall_work);
						//$DD ドライバ固有統計(tx_stall_workqueue_scheduled)更新
						priv->xsdevice.drv_stats.
						    tx_stall_workqueue_scheduled++;
					}
					//$DD 関数戻り値をビジーに設定する
					retStatus = NETDEV_TX_BUSY;
				}
			} else {
				//$DD 送信が正常終了した
				PDEBUG_DEV(PD_COMMON
					   || PD_START_XMIT, "send done");

				//$DD 統計情報(tx_packets,tx_bytes)を更新(マルチキャスト/ブロードキャスト以外)
				if (!is_multi) {
					ES_DRV_STATS_INC(priv, dest_epid,
							 tx_packets, 1);
					ES_DRV_STATS_INC(priv, dest_epid,
							 tx_bytes, len);
				}
				//$DD フレーム送信済回数をリセットする
				priv->tx_retry_count = 0;

				//$DD 関数戻り値を正常に設定する
				retStatus = NETDEV_TX_OK;
			}
		}

	}			// for

	//$DD 終了処理
	//$DD 関数戻り値が正常終了の場合はソケットバッファを解放する
	//$DD 関数戻り値がをビジーの場合は解放しない
	if (retStatus == NETDEV_TX_OK) {
		PDEBUG_DEV(PD_COMMON || PD_START_XMIT, "dev_kfree_skb");
		dev_kfree_skb(skb);

		//$DD 統計情報(tx_packets,tx_bytes)を更新(マルチキャスト/ブロードキャスト、自分のEPIDを指定して実施)
		if (is_multi) {
			ES_DRV_STATS_INC(priv, selfepid, tx_packets, 1);
			ES_DRV_STATS_INC(priv, selfepid, tx_bytes, len);
		}

	} else {
		PDEBUG_DEV(PD_COMMON || PD_START_XMIT, "retStatus!=OK");
	}

	PDEBUG_DEV(PD_COMMON || PD_START_XMIT, "end");
	return retStatus;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．１０ 送信バッファ滞留軽減タスク

  状態取得でTx側と齟齬が発生する可能性はあるが、
   * デッドロック回避
   * 対向EPでも数値操作する
   * キューは停止済み、再開できるかチェックすればよい
   ので複数回試行とそれによるキュー再開で問題ない

  @param[int]     work	ワークキューエントリの構造体
**/
FUNC_STATIC void esdrv_tx_stall(struct work_struct *work)
{
	struct esdrv_priv *priv =
	    container_of(work, struct esdrv_priv, tx_stall_work);
	epno_t epid;
	epno_t maxepid, selfepid;
	union ep_buffer_info *info;
	int all_queue_available;
	int i;
	int sendable;

	//$DD 動作時間の超過チェック(dev->trans_startを開始時刻として使う)
	if (((long)jiffies -
	     (long)(priv->xsdevice.container_dev->trans_start)) >
	    ES_TX_TX_STALL_TIMEOUT) {
		netif_wake_queue(priv->xsdevice.container_dev);
		return;
	}

	selfepid = priv->xsdevice.epid_self;
	maxepid = priv->xsdevice.epnum_max;

	//$DD チェック処理を複数回実施(再スケジュールによる遅延防止)
	for (i = 0; i < ES_TX_STALL_CHECK_MAX; i++) {
		all_queue_available = 1;

		//$DD 全EPについて、送信バッファがいっぱいのEPが存在しないか確認
		for (epid = 0; epid < maxepid; epid++) {
			//$DD 複数宛先処理のために、自EPID宛が発生した場合はスキップする
			if (selfepid == epid) {
				// スキップ
				continue;
			}
			//$DD 相手EPへ送信可能か確認(共有中で送受信中止要求が出ていない)
			//$DD 共有状態取得
			sendable =
			    (EP_PARTNER_SHARED ==
			     EP_PARTNER_STATUS(priv, epid));
			//$DD 送信可能なら処理継続
			if (!sendable) {
				continue;
			}

			info = priv->xsdevice.ep_shm_info[epid].tx.info;

			//$DD MTU変更中を検出したら、即座に終了(回復処置もエンキューもしない)
			//$DD MTU変更ではcancel_syncと十分な時間の待ち、およびその後のwakeがあるため
			if (!(info->v1i.rx_status & ES_RX_MTU_CHANGING_DONE)) {
				return;
			}
			
			//$DD EPの送信バッファが有効か(フルでないか)を確認
			if (EP_RING_FULL
			    (info->v1i.head, info->v1i.tail,
			     info->v1i.count_max)) {
				all_queue_available = 0;
				break;
			}
		}

		//$DD 全EPの送信バッファが有効ならnetif_wake_queueして終了する
		if (all_queue_available) {
			netif_wake_queue(priv->xsdevice.container_dev);
			return;
		}
	}

	//$ 2回目以降の繰り返しチェックは遅延させて登録する
	usleep_range(50, 100);

	//$DD 再スケジュールによる遅延防止
	queue_work(priv->txrx_wq, &priv->tx_stall_work);

	//$DD ドライバ固有統計(tx_stall_workqueue_scheduled)更新
	priv->xsdevice.drv_stats.tx_stall_workqueue_scheduled++;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．１１ フレーム送信リトライ

  net_device.netdev_ops.ndo_tx_timeoutに本ハンドラ関数のポインタを登録する
  送信キュー停止状態が一定時間以上続くと呼び出される

  規定時間内に送信できなかった場合の処理を行う
  バッファフルが一定時間続いた場合を想定する。
  共有状態を確認して送信しているので、送信相手が異常終了していると考えられる。
  
  @param[in,out] dev	net_device構造体

**/
FUNC_STATIC void esdrv_tx_retry(struct net_device *dev)
{

	struct esdrv_priv *priv = netdev_priv(dev);
	struct netdev_queue *curQueue = netdev_get_tx_queue(dev, 0);	//キュー番号取得

	//$DD 送信キューを再開する    netif_tx_wake_queue()
	PDEBUG_DEV(PD_ENT, "ENTER timeout at %ld, latency %ld\n", jiffies,
		   jiffies - dev->trans_start);
	netif_tx_wake_queue(curQueue);
	return;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．１２ 受信  割り込み

  IRQハンドラとして登録する
  受信バッファに受信できるデータが揃ったら呼ばれる

  受信処理はLinux NAPI(割り込み＋ポーリング)方式で行う
  受信割り込みの発生を停止して、受信ポーリングをスケジュールする。
  （ここで受信は行わない)

  @param[in,out] dev		net_device構造体
  @param[in]     src_epid	送信元のEPID(今のところ未使用)
**/
FUNC_STATIC void esdrv_rx_irq(struct net_device *dev, epno_t src_epid)
{
	struct esdrv_priv *priv;

	PDEBUG_DEV(PD_ENT, "ENTER");

	priv = netdev_priv(dev);

	//$DD 受信割り込み発生を禁止する   esvmm_irqmask(true)
	esvmm_irqmask(priv->xsdevice.base, INTREG_IRQMASK_RX_DATA, true);

	//$DD 受信ポーリング esdrv_rx_napi_poll() をスケジュールする    napi_schedule()

	PDEBUG_DEV(PD_INF, "NAPI RX_INTR %d",
		   test_bit(NAPI_STATE_SCHED, &priv->napi.state));
	priv->unset_rx_last = true;
	napi_schedule(&priv->napi);

	//$DD ドライバ固有統計(napi_scheduled:全体統計のみ)更新
	priv->xsdevice.drv_stats.napi_scheduled++;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．１３ NAPI受信ポーリング

  カーネル関数により本ハンドラ関数を登録する。
  受信割り込みでスケジュールされ、受信ポーリングを行う。

  @param[in,out] pnapi		NAPIオブジェクト
  @param[in]     budget		本関数の呼出しで処理可能なフレームの最大数

  @retval  0		処理すべきフレームは残っていない
  @retval  0以外	処理すべきフレームが残っている(今回処理フレーム数を返す)
**/
FUNC_STATIC int esdrv_rx_napi_poll(struct napi_struct *pnapi, int budget)
{
	int npackets = 0;
	struct sk_buff *skb;
	struct esdrv_priv *priv = container_of(pnapi, struct esdrv_priv, napi);
	struct net_device *dev = pnapi->dev;
	framebuf_t frameData;
	size_t frameLen;
	unsigned long flags;	//irqsave/restore用
	epno_t cur_epid = 0;
	epno_t epidx = 0;

//      PDEBUG_DEV(PD_ENT,"ENTER rx_queue=%d", priv->rx_queue!=NULL);
	PDEBUG_DEV(PD_ENT, "ENTER");

	//$DD 全他EP向けtxのrx_statusに(ES_RX_POLL_WORK)を設定
	//$DD rx_statusの排他設定(割り込みコンテキスト)	spin_lock
	spin_lock(&priv->xsdevice.rx_status_lock);
	for (epidx = 0; epidx < priv->xsdevice.epnum_max; epidx++) {
		//$DD 自分のEP IDは処理しない
		if (epidx == priv->xsdevice.epid_self) {
			continue;
		}
		if (EP_PARTNER_SHARED == EP_PARTNER_STATUS(priv, epidx)) {
			priv->xsdevice.ep_shm_info[epidx].tx.info->v1i.rx_status |=
			    ES_RX_POLL_WORK;
		}
	}
	//$DD rx_statusの排他解除(割り込みコンテキスト)	spin_unlock
	spin_unlock(&priv->xsdevice.rx_status_lock);

	//$DD ドライバ固有統計(napi_polled:全体統計のみ)更新
	priv->xsdevice.drv_stats.napi_polled++;

	//$DD 処理したフレーム数が処理可能なフレーム数(budget)より少ない、
	//$DD かつ受信バッファにフレームが残っている間、次の処理を繰り返す
	while (npackets < budget) {
		//$DD xsocket deviceからフレームを受信する   esif_rxframe_get()
		//$DD 共有中・送受信中止要求の確認は esif_rxframe_get()に含まれる
		prefetch(&priv->xsdevice);
		frameData = esif_rxframe_get(dev, &frameLen, &cur_epid);

		if (frameData) {
			PDEBUG_DEV(PD_INF, "exist");
			//$DD データあり、格納するskbを確保
			skb = netdev_alloc_skb_ip_align(dev, frameLen);
			if (!skb) {
				//$DD 確保できない場合、統計値(rx_dropped,rx_errors)上昇
				PDEBUG_LIMIT(PD_INF, "ESDRV: packet dropped\n");
				ES_DRV_STATS_INC(priv, cur_epid, rx_dropped, 1);
				ES_DRV_STATS_INC(priv, cur_epid, rx_errors, 1);
			} else {
				//$DD ethernetフレームの生成
				memcpy(skb_put(skb, frameLen), frameData,
				       frameLen);
				skb->dev = dev;
				skb->protocol = eth_type_trans(skb, dev);
				skb->ip_summed = CHECKSUM_UNNECESSARY;	/* don't check it */

				//$DD 受信したフレームをカーネル(プロトコル層)に渡す    netif_receive_skb()
				netif_receive_skb(skb);

				//$DD 統計情報(rx_packets,rx_bytes)を更新する
				npackets++;
				ES_DRV_STATS_INC(priv, cur_epid, rx_packets, 1);
				ES_DRV_STATS_INC(priv, cur_epid, rx_bytes,
						 frameLen);

				//$DD rx_length_errorsは送信側が最小値を補正しているため、発生しない
				//$DD rx_frame_errorsは最大値を超えて入れないため、発生しない
				//$DD multicastはframeDataからチェックできる  is_multicast_ether_addr()
				if (is_multicast_ether_addr
				    (((struct ethhdr *)frameData)->h_dest)) {
					ES_DRV_STATS_INC(priv, cur_epid,
							 multicast, 1);
				}

			}

			//$DD 受信バッファを解放       esif_rxframe_release()
			PDEBUG_DEV(PD_INF, "release buffer");
			esif_rxframe_release(dev, cur_epid);
			priv->unset_rx_last = true;
		} else {
			PDEBUG_DEV(PD_INF, "empty");
			break;
		}
	}

#if !defined(ESDRV_ETHERTOOL_OFF)
	//$DD Ethertool用統計(napi_poll_maxpackets)更新
	if (priv->xsdevice.drv_stats.napi_poll_maxpackets < npackets) {
		priv->xsdevice.drv_stats.napi_poll_maxpackets = npackets;	// ポーリング処理最大パケット数(回数ではなくパケット)
	}
#endif				/* !defined(ESDRV_ETHERTOOL_OFF) */

	//$DD NAPIでの基本動作
	//$DD 受信バッファにフレームが残っていなければ
	//$DD         ポーリングを終了を通知する  napi_complete()
	//$DD         受信割り込み発生を許可する
	//$DD         処理すべきフレームは残っていないとして終了(0を返す)
	//$DD 受信バッファにフレームが残っていれば
	//$DD         処理すべきフレームが残っているとして終了(処理フレーム数を返す)

	PDEBUG_DEV(PD_INF, "done %d", npackets);

	//$DD 残りフレーム数のチェック
	if (npackets) {
		//$DD 処理すべきフレームが残っているため、NAPI継続なので(ES_RX_POLL_WORK)を解除しないで維持する
		PDEBUG_DEV(PD_INF, "poll continue");
	} else {
		//$DD 処理すべきフレームが残っていない、NAPI動作完了       napi_complete()
		PDEBUG_DEV(PD_INF, "complete");
		napi_complete(pnapi);

		//$DD ドライバ固有統計(napi_polled_empty:全体統計のみ)更新
		priv->xsdevice.drv_stats.napi_polled_empty++;

		//$DD 処理すべきフレームが残っていない状態
		if (priv->unset_rx_last) {
			//$DD 初回状態 rx_last_jiffyが設定されていない、現在時間を設定
			priv->rx_last_jiffy = jiffies;
			priv->unset_rx_last = false;
		}
		//$DD rx_last_jiffyから受信なしが所定時間経過したかを確認 閾値3
		if (((long)jiffies - (long)priv->rx_last_jiffy) < ES_RX_NAPI_MARGIN) {	//jiffiesのラップを考慮
			//$DD 達してないので、再スケジュール     napi_reschedule()
			napi_reschedule(pnapi);
		} else {
			//$DD 全他EP向けtxのrx_statusから(ES_RX_POLL_WORK)を解除
			//$DD rx_statusの排他設定(割り込みコンテキスト)	spin_lock
			spin_lock(&priv->xsdevice.rx_status_lock);
			for (epidx = 0; epidx < priv->xsdevice.epnum_max;
			     epidx++) {
				//$DD 自分のEP IDは処理しない
				if (epidx == priv->xsdevice.epid_self) {
					continue;
				}
				if (EP_PARTNER_SHARED == EP_PARTNER_STATUS(priv, epidx)) {
					priv->xsdevice.ep_shm_info[epidx].tx.info->v1i.
					    rx_status &= ~ES_RX_POLL_WORK;
				}
			}
			//$DD rx_statusの排他解除(割り込みコンテキスト)	spin_unlock
			spin_unlock(&priv->xsdevice.rx_status_lock);

			//$DD 受信割り込み許可  esvmm_irqmask(false)
			esvmm_irqmask(priv->xsdevice.base,
				      INTREG_IRQMASK_RX_DATA, false);
		}
	}
	//$DD 戻り値=処理パケット数で終了する
	return npackets;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．１４ MTU変更通知

  net_device.netdev_ops.ndo_change_mtuに本ハンドラ関数のポインタを登録する
  MTU値の範囲チェックと変更を行う

  @param[in,out] dev		net_device構造体
  @param[in]     newMtu		新しいMTU値の要求値

  @return  エラーコード
  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esdrv_change_mtu(struct net_device *dev, int newMtu)
{
	struct esdrv_priv *priv = netdev_priv(dev);
	int idx,epidx;
	bool running = netif_running(dev);	//オープン中
	int ret = -EINVAL;
	unsigned long flags;

	PDEBUG_DEV(PD_ENT, "ENTER");

	//$DD 設定可能な値を探す(要求値以上の設定可能な値にする)
	for (idx = 0; ep_support_mtu[idx] != 0; idx++) {
		if (newMtu <= ep_support_mtu[idx]) {
			//$DD 設定可能な値が見つかった
			newMtu = ep_support_mtu[idx];

			//$DD 現在値と発見値を比較し、変更不要なら処理終了
			if (newMtu == dev->mtu) {
				PDEBUG_DEV(PD_MTU, "same MTU:%d", newMtu);
				//$DD 正常終了
				return 0;	/* success */
			}
			
			//$DD 検出値を設定することになるので、戻り値に正常を設定しループ脱出
			ret = 0;
			break;
		}
	}
	
	ES_DRV_LOGGING("%s MTU old=%d,new=%d,running=%d",
		__func__,dev->mtu,newMtu,
		running);
	//$DD 戻り値が正常値であり、MTUの変更を行う場合
	if (0 == ret) {
		//$DD インタフェースがrunningの場合
		if (running) {
			//$DD 各tx.rx_statusへMTU変更中のためにフラグを削除する
			//$DD rx_statusの排他設定	spin_lock_irqsave
			spin_lock_irqsave(&priv->xsdevice.rx_status_lock, flags);
			for (epidx = 0; epidx < priv->xsdevice.epnum_max; epidx++) {
				if (priv->xsdevice.epid_self != epidx) {
					//$DD フラグ削除
					priv->xsdevice.ep_shm_info[epidx].tx.info->v1i.rx_status &= ~ES_RX_MTU_CHANGING_DONE;
				}
			}
			//$DD rx_statusの排他解除	spin_unlock_irqrestore
			spin_unlock_irqrestore(&priv->xsdevice.rx_status_lock, flags);
			
			//$DD netif_tx_stop_all_queuesと netif_carrier_off()で送信停止
			netif_tx_stop_all_queues(dev);
			netif_carrier_off(dev);

			//$DD cancel_work_syncによる tx_stall停止待ち(フラグにて再登録なしで終了する構造)
			cancel_work_sync(&priv->tx_stall_work);
			
			//$DD NAPI停止
			napi_disable(&priv->napi);
			
			//$DD 送信および対向からの受信の状態の完了のため、1secのwaitを行う
			msleep(1000);

			//$DD netif_tx_stop_all_queuesで送信停止(再度)
			netif_tx_stop_all_queues(dev);
		}

		
		//$DD MTU変更を反映
		dev->mtu = newMtu;
		PDEBUG_DEV(PD_MTU, "CHANGED");

		//$DD インタフェースがrunningの場合
		if (running) {
			//$DD 各txバッファの初期化によってMTU情報を反映し、rx_statusのMTU変更完了のフラグを設定する
			for (epidx = 0; epidx < priv->xsdevice.epnum_max; epidx++) {
				if (priv->xsdevice.epid_self != epidx) {

					//$DD rx_statusの排他設定	spin_lock_irqsave
					spin_lock_irqsave(&priv->xsdevice.rx_status_lock, flags);
					//$DD 送信バッファの初期化
					esmem_epbuf_setup(&(priv->xsdevice.ep_shm_info[epidx].tx),
							  dev->dev_addr,
							  dev->mtu);
					
					//$DD フラグ設定(epbuf_setupで設定済みだが、念のため)
					priv->xsdevice.ep_shm_info[epidx].tx.info->v1i.rx_status |= ES_RX_MTU_CHANGING_DONE;
					//$DD rx_statusの排他解除	spin_unlock_irqrestore
					spin_unlock_irqrestore(&priv->xsdevice.rx_status_lock, flags);
				}
			}
			
			//$DD netif_tx_wake_all_queuesとnetif_carrier_on()で送信再開
			netif_tx_wake_all_queues(dev);
			netif_carrier_on(dev);
			
			//$DD NAPI開始
			napi_enable(&priv->napi);
			napi_schedule(&priv->napi);
			
			//$DD ドライバ固有統計(napi_scheduled:全体統計のみ)更新
			priv->xsdevice.drv_stats.napi_scheduled++;
		}

		//$DD ドライバ固有統計(mtu_changed)更新
		priv->xsdevice.drv_stats.mtu_changed++;

	}

	//$DD 設定可能な値が見つかったら正常値、なかったら
	//$DD 引数が範囲外のエラーで終了(EINVAL)
	return ret;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．１５ 統計情報要求(32bit)

	欠番
	
	現状では欠番として詳細には記載すること
	
  net_device.netdev_ops.ndo_get_statsに本ハンドラ関数のポインタを登録する
  デバイスの統計情報を返す

  @param[in]     dev		net_device構造体
  
  @return  統計のポインタを返す
**/
#if 0				/* 現状不要になった */
FUNC_STATIC struct net_device_stats *esdrv_get_stats(struct net_device *dev)
{
	/*
	   net_device_statsのメンバーは unsigned long
	   rtnl_link_stats64のメンバーは __u64
	   64bit環境では unsigned longと__u64は同一
	   32bit環境では、64bitの統計値を更新する際にatomicに扱う必要有り(現状は未対応)

	   以下のようなコードで対応できると思われるが、真に32bit環境対策ができないのでコメントのみ記録
	   #if BITS_PER_LONG==64
	   struct esdrv_priv *priv = netdev_priv(dev);
	   return (struct net_device_stats*)&priv->stats64;
	   #endif
	 */

	//$DD 統計情報の保管領域を取得
	struct esdrv_priv *priv = netdev_priv(dev);

	PDEBUG_DEV(PD_ENT, "ENTER");

	//$DD 保管領域から受け渡し先へポインタ転送
	return (&priv->stats32);
}
#endif				/* 0 */

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．１６ 統計情報要求(64bit)

  net_device.netdev_ops.ndo_get_stats64に本ハンドラ関数のポインタを登録する
  デバイスの統計情報を返す

  @param[in]     dev		net_device構造体
  @param[out]    storage	統計情報の受け渡し先

  @return  storageを返す
**/
FUNC_STATIC struct rtnl_link_stats64 *esdrv_get_stats64(struct net_device *dev,
							struct rtnl_link_stats64
							*storage)
{
	//$DD 統計情報の保管領域を取得
	struct esdrv_priv *priv = netdev_priv(dev);

	PDEBUG_DEV(PD_ENT, "ENTER");

	//$DD 保管領域から受け渡し先へコピー
	memcpy(storage, &priv->stats64, sizeof(struct rtnl_link_stats64));
	return storage;
}

#ifndef ESDRV_VLAN_OFF
//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．１７ VLAN ID登録

  net_device.netdev_ops. ndo_vlan_rx_add_vidに本ハンドラ関数のポインタを登録する
  VLAN ID登録時に呼ばれる

  @param[in,out] dev	net_device構造体
  @param[in]     proto  プロトコル(3.3から)
  @param[in]     vid	VLAN ID

  Kernel 3.3から戻り値がvoidではなくintになる
  @return  エラーコード
  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
//  Kernel 3.3から戻り値がvoidではなくintになる
FUNC_STATIC int esdrv_vlan_rx_add_vid(struct net_device *dev, __be16 proto,
				      unsigned short vid)
{
	struct esdrv_priv *priv = netdev_priv(dev);
	bool ret = true;	// すでに登録済みだった場合、更新されないので
	epno_t epid;

	PDEBUG(PD_ENT, "ENTER");

	PDEBUG(PD_VLAN, "add vid:%d", vid);

	// @todo esif層に処理を委譲すべきか?
	//$DD esmem_set_vidに自分以外の各送信バッファを渡して、VIDをセット
	for (epid = 0; epid < priv->xsdevice.epnum_max; epid++) {
		//$DD 自分のEP IDは処理しない
		if (epid == priv->xsdevice.epid_self) {
			continue;
		}
		//$DD vidを登録していなければ、登録処理  esmem_has_vid()
		if (!esmem_has_vid(&(priv->xsdevice.ep_shm_info[epid].tx), vid)) {
			PDEBUG_DEV(PD_VLAN, "set VID:%d", vid);
			//$DD 登録    esmem_set_vid()
			ret =
			    esmem_set_vid(&
					  (priv->xsdevice.ep_shm_info[epid].tx),
					  vid);
		}
	}

	ES_DRV_LOGGING("%s vid=%d,ret=%d",
		__func__,vid,ret);
	//$DD ドライバ固有統計(vlan_add)更新
	if (ret) {
		priv->xsdevice.drv_stats.vlan_added++;
	}
	//$DD 結果によって0もしくはENOSPCを返す
//  Kernel 3.3から戻り値がvoidではなくintになる
	return ret ? 0 : -ENOSPC;
}
#endif				/* !ESDRV_VLAN_OFF */

#ifndef ESDRV_VLAN_OFF
//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．１８ VLAN ID削除

  net_device.netdev_ops. ndo_vlan_rx_kill_vidに本ハンドラ関数のポインタを登録する
  VLAN ID削除時に呼ばれる

  @param[in,out] dev	net_device構造体
  @param[in]     proto  プロトコル(3.3から)
  @param[in]     vid	VLAN ID

  Kernel 3.3から戻り値がvoidではなくintになる
  @return  エラーコード
  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esdrv_vlan_rx_kill_vid(struct net_device *dev, __be16 proto,
				       unsigned short vid)
{
	struct esdrv_priv *priv = netdev_priv(dev);
	epno_t epid;

	PDEBUG(PD_ENT, "ENTER");

	PDEBUG(PD_VLAN, "del vid:%d", vid);

	// @todo esif層に処理を委譲すべきか?
	//$DD esmem_del_vidに送信バッファを渡して、VIDを削除
	for (epid = 0; epid < priv->xsdevice.epnum_max; epid++) {
		//$DD 自分のEP IDは処理しない
		if (epid == priv->xsdevice.epid_self) {
			continue;
		}
		//$DD 削除処理      esmem_del_vid()
		esmem_del_vid(&(priv->xsdevice.ep_shm_info[epid].tx), vid);
	}

	ES_DRV_LOGGING("%s vid=%d",
		__func__,vid);
	//$DD ドライバ固有統計(vlan_killed)更新
	priv->xsdevice.drv_stats.vlan_killed++;

	//$DD 正常終了 0固定
//  Kernel 3.3から戻り値がvoidではなくintになる
	return 0;
}
#endif				/* !ESDRV_VLAN_OFF */

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．１９ IOCTL
	
  現在トレース機能、レジスタアクセス機能が存在
	
  @param[in,out] dev	net_device構造体
  @param[in,out] rq	ifreq 構造体
  @param[in]     cmd	コマンド番号

  @return  エラーコード
  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esdrv_do_ioctl(struct net_device *dev, struct ifreq *rq,
			       int cmd)
{
	int ret = 0;

	PDEBUG_DEV(PD_ENT, "ENTER");

	//$DD 各コマンドごとの実行
	switch (cmd) {
		//$DD サポートしていないコマンド
	default:
		//$DD サポート外コマンドならEOPNOTSUPP
		ret = -EOPNOTSUPP;
		break;
		//$DD トレース開始コマンド    esdrv_ioctl_trace_start()
	case ES_IOCTL_TRACE_START:
		ret = esdrv_ioctl_trace_start(dev, rq);
		break;
		//$DD トレース終了コマンド    esdrv_ioctl_trace_stop()
	case ES_IOCTL_TRACE_STOP:
		ret = esdrv_ioctl_trace_stop(dev, rq);
		break;
		//$DD トレース情報取得コマンド      esdrv_ioctl_trace_getcfg()
	case ES_IOCTL_TRACE_GETCFG:
		ret = esdrv_ioctl_trace_getcfg(dev, rq);
		break;
		//$DD レジスタ取得コマンド    esdrv_ioctl_reg_read()
	case ES_IOCTL_ESDEV_GETREG:
		ret = esdrv_ioctl_reg_read(dev, rq);
		break;
		//$DD レジスタ設定コマンド    esdrv_ioctl_reg_write()
	case ES_IOCTL_ESDEV_SETREG:
		ret = esdrv_ioctl_reg_write(dev, rq);
		break;
	}

	//$DD 各処理の戻り値を返す
	return ret;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．２０ proc情報表示(info)

  ドライバ情報を表示するprocファイルエントリ

  @param	m		seq_fileオブジェクト
  @param	v		付加情報
	
  @retval	0		正常
  @retval	0以外	異常
**/
FUNC_STATIC int esdrv_proc_info_show(struct seq_file *m, void *v)
{
	struct esdrv_priv *priv;

	PDEBUG(PD_ENT, "ENTER");

	priv = netdev_priv(esdrv_device);

	//$DD 固定文字列を表示、その際にMajor/Minor/Revisionとコンパイル日時を表示する
	seq_printf(m, "FUJITSU Extended Socket\n");
	seq_printf(m, "Version: " ES_DRV_VERSION_STRING " (%s)\n", __DATE__);

	return 0;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．２１ proc情報表示(status)

  デバイスとドライバの情報を表示するprocファイルエントリ

  @param	m		seq_fileオブジェクト
  @param	v		付加情報
	
  @retval	0		正常
  @retval	0以外	異常
**/
FUNC_STATIC int esdrv_proc_status_show(struct seq_file *m, void *v)
{
	struct esdrv_priv *priv;
	epno_t epidx;
	epno_t maxepid, selfepid;
	bool running;

	PDEBUG(PD_ENT, "ENTER");

	priv = netdev_priv(esdrv_device);

	selfepid = priv->xsdevice.epid_self;
	maxepid = priv->xsdevice.epnum_max;

	//$DD ヘッダ文字列を表示
#if PD_STATUS != 0
	seq_printf(m,
		   "status debug EP ID(n) n is status: EP_PARTNER_UNSHARE(%d),EP_PARTNER_SHARED(%d),EP_PARTNER_WAITING(%d),EP_PARTNER_COMPLETE(%d)\n",
		   EP_PARTNER_UNSHARE, EP_PARTNER_SHARED, EP_PARTNER_WAITING,
		   EP_PARTNER_COMPLETE);
#endif				/* PD_STATUS != 0 */
	seq_printf(m, "EP ID          SAME ZONE        CONNECTED\n");

	//$DD 各EPのZoneと接続状態を表示
	for (epidx = 0; epidx < maxepid; epidx++) {
		seq_printf(m, "%d", epidx);	// 2ケタになったら%-2dなどとする
		if (selfepid == epidx) {
			//$DD 自EPID表示は個別に実施
			seq_printf(m, "(self)        %c                %c\n",
				   '-', '-');
		} else {
			//$DD 各EPについて、Zoneが同じか、接続しているかをY/Nで表示
#if PD_STATUS != 0
			seq_printf(m, "(%d)           %c                %c\n", EP_PARTNER_STATUS(priv, epidx), esmem_same_zone(&(priv->xsdevice), epidx) ? 'Y' : 'N',	/* Zone */
				   esmem_is_shared(priv->xsdevice.share_ptrs.share, epidx) ? 'Y' : 'N'	/* Connect */
			    );
#else				/* PD_STATUS != 0 */
			seq_printf(m, "              %c                %c\n", esmem_same_zone(&(priv->xsdevice), epidx) ? 'Y' : 'N',	/* Zone */
				   esmem_is_shared(priv->xsdevice.share_ptrs.share, epidx) ? 'Y' : 'N'	/* Connect */
			    );
#endif				/* PD_STATUS != 0 */
		}
	}

	return 0;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．２２ proc情報表示(stats)

  ドライバの各EPの統計情報を表示するprocファイルエントリ

  @param	m		seq_fileオブジェクト
  @param	v		付加情報
	
  @retval	0		正常
  @retval	0以外	異常
**/
FUNC_STATIC int esdrv_proc_stats_show(struct seq_file *m, void *v)
{
	struct esdrv_priv *priv;
	epno_t epidx;
	epno_t maxepid, selfepid;

	PDEBUG(PD_ENT, "ENTER");

	priv = netdev_priv(esdrv_device);

	selfepid = priv->xsdevice.epid_self;
	maxepid = priv->xsdevice.epnum_max;

	// 表示フォーマット
	// |-- space 41 -- ||- EP 11 -||- EP 11 -|...
	//                         EP#0 (self)EP#1
	// |-- name  41 -- ||- num11 -||- num11 -|...
	// tx_xx            12345678901          -

	//$DD ヘッダ文字列を表示
	seq_printf(m, "%41s", " ");	// 41
	for (epidx = 0; epidx < maxepid; epidx++) {
		seq_printf(m, "%10s%d", (selfepid == epidx) ? "(self)EP#" : "EP#", epidx);	// 11
	}
	seq_printf(m, "\n");

	//$DD 各EPの統計情報を表示(net_stats)

#define ES_DRV_PROC_STATS_ENTRY(_name) do{               \
	seq_printf(m,"%-41s", ES_DRV_TO_STRING(_name) ); \
	for ( epidx = 0; epidx < maxepid; epidx++ ) {    \
		if(selfepid == epidx) {                  \
			seq_printf(m, "          -");    \
		}else{                                   \
			seq_printf(m, " %10llu",         \
				priv->xsdevice.ep_shm_info[epidx].net_stats._name ); \
		}                                        \
	}                                                \
	seq_printf(m,"\n");                              \
}while(0)

	ES_DRV_PROC_STATS_ENTRY(rx_packets);
	ES_DRV_PROC_STATS_ENTRY(tx_packets);
	ES_DRV_PROC_STATS_ENTRY(rx_bytes);
	ES_DRV_PROC_STATS_ENTRY(tx_bytes);
	ES_DRV_PROC_STATS_ENTRY(rx_errors);
	ES_DRV_PROC_STATS_ENTRY(tx_errors);
	ES_DRV_PROC_STATS_ENTRY(rx_dropped);
	ES_DRV_PROC_STATS_ENTRY(tx_dropped);
	ES_DRV_PROC_STATS_ENTRY(multicast);
	ES_DRV_PROC_STATS_ENTRY(collisions);
	ES_DRV_PROC_STATS_ENTRY(rx_length_errors);
	ES_DRV_PROC_STATS_ENTRY(rx_over_errors);
	ES_DRV_PROC_STATS_ENTRY(rx_crc_errors);
	ES_DRV_PROC_STATS_ENTRY(rx_frame_errors);
	ES_DRV_PROC_STATS_ENTRY(rx_fifo_errors);
	ES_DRV_PROC_STATS_ENTRY(rx_missed_errors);
	ES_DRV_PROC_STATS_ENTRY(tx_aborted_errors);
	ES_DRV_PROC_STATS_ENTRY(tx_carrier_errors);
	ES_DRV_PROC_STATS_ENTRY(tx_fifo_errors);
	ES_DRV_PROC_STATS_ENTRY(tx_heartbeat_errors);
	ES_DRV_PROC_STATS_ENTRY(tx_window_errors);
	ES_DRV_PROC_STATS_ENTRY(rx_compressed);
	ES_DRV_PROC_STATS_ENTRY(tx_compressed);

#undef ES_DRV_PROC_STATS_ENTRY

	//$DD 各EPの統計情報を表示(独自)

#define ES_DRV_PROC_EP_STATS_ENTRY(_name) do{            \
	seq_printf(m,"%-41s", ES_DRV_TO_STRING(_name) ); \
	for ( epidx = 0; epidx < maxepid; epidx++ ) {    \
		if(selfepid == epidx) {                  \
			seq_printf(m, "          -");    \
		}else{                                   \
			seq_printf(m, " %10llu",         \
				priv->xsdevice.ep_shm_info[epidx].ep_stats._name ); \
		}                                        \
	}                                                \
	seq_printf(m,"\n");                              \
}while(0)

	ES_DRV_PROC_EP_STATS_ENTRY(command_register_buffer_executed);
	ES_DRV_PROC_EP_STATS_ENTRY(command_unregister_buffer_executed);
	ES_DRV_PROC_EP_STATS_ENTRY(send_interrupts_rx);
	ES_DRV_PROC_EP_STATS_ENTRY(send_interrupts_unshare);
	ES_DRV_PROC_EP_STATS_ENTRY(send_interrupts_stop);
	ES_DRV_PROC_EP_STATS_ENTRY(send_interrupts_zoneupdate);
	ES_DRV_PROC_EP_STATS_ENTRY(receive_interrupts_rx);
	ES_DRV_PROC_EP_STATS_ENTRY(receive_interrupts_unshare);
	ES_DRV_PROC_EP_STATS_ENTRY(receive_interrupts_stop);
	ES_DRV_PROC_EP_STATS_ENTRY(receive_interrupts_zoneupdate);
	ES_DRV_PROC_EP_STATS_ENTRY(tx_buffer_full);
	ES_DRV_PROC_EP_STATS_ENTRY(tx_dropped_not_shared);
	ES_DRV_PROC_EP_STATS_ENTRY(tx_dropped_version_mismatch);
	ES_DRV_PROC_EP_STATS_ENTRY(tx_dropped_buffer_size_discrepancy);
	ES_DRV_PROC_EP_STATS_ENTRY(tx_dropped_vlan_id_mismatch);

#undef ES_DRV_PROC_EP_STATS_ENTRY

	//$DD 自EPの統計情報を表示(ドライバ単体)

#define ES_DRV_PROC_EPDRV_STATS_ENTRY(_name,_pre) do{         \
	seq_printf(m,"%-41s", _pre ES_DRV_TO_STRING(_name) ); \
	for ( epidx = 0; epidx < maxepid; epidx++ ) {         \
		if(selfepid == epidx) {                       \
			seq_printf(m, " %10llu",              \
				priv->xsdevice.drv_stats._name ); \
		}else{                                        \
			seq_printf(m, "          -");         \
		}                                             \
	}                                                     \
	seq_printf(m,"\n");                                   \
}while(0)

	ES_DRV_PROC_EPDRV_STATS_ENTRY(open, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(close, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(mtu_changed, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(vlan_added, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(vlan_killed, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(napi_scheduled, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(napi_polled, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(napi_polled_empty, "");
#if !defined(ESDRV_ETHERTOOL_OFF)
	ES_DRV_PROC_EPDRV_STATS_ENTRY(napi_poll_maxpackets, "");
#endif				/* !defined(ESDRV_ETHERTOOL_OFF) */
	ES_DRV_PROC_EPDRV_STATS_ENTRY(device_reset_executed, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(device_force_reset_scheduled, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(command_register_buffer_executed,
				      "total_");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(command_unregister_buffer_executed,
				      "total_");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(command_request_info_executed, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(command_start_trace_executed, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(command_stop_trace_executed, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(interrupts_workqueue_scheduled, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(tx_stall_workqueue_scheduled, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(unshare_workqueue_scheduled, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(zoneupdate_workqueue_scheduled, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(unshare_watch_workqueue_scheduled, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(tx_dropped_unknown_destination, "");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(send_interrupts_rx, "total_");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(send_interrupts_unshare, "total_");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(send_interrupts_stop, "total_");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(send_interrupts_zoneupdate, "total_");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(receive_interrupts_rx, "total_");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(receive_interrupts_unshare, "total_");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(receive_interrupts_stop, "total_");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(receive_interrupts_zoneupdate, "total_");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(tx_buffer_full, "total_");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(tx_dropped_not_shared, "total_");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(tx_dropped_version_mismatch, "total_");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(tx_dropped_buffer_size_discrepancy,
				      "total_");
	ES_DRV_PROC_EPDRV_STATS_ENTRY(tx_dropped_vlan_id_mismatch, "total_");

#undef ES_DRV_PROC_EPDRV_STATS_ENTRY

	return 0;
}

// シンボル定義都合などから、関数記述位置が違うことに注意
//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．４０ proc情報表示(log)

  ログを表示するprocファイルエントリ

  @param	m		seq_fileオブジェクト
  @param	v		付加情報
	
  @retval	0		正常
  @retval	0以外	異常
**/
FUNC_STATIC int esdrv_proc_log_show(struct seq_file *m, void *v)
{
	struct esdrv_priv *priv;
	u64 start;
	int count;
	int i,vid;
	int index;
	int start_index;
	epno_t epid;
	bool escapse;
	char *type;

	PDEBUG(PD_ENT, "ENTER");

	priv = netdev_priv(esdrv_device);

	//$DD 送受信バッファInfoの表示
	seq_printf(m, "TxRx: EP count=%d\n",priv->xsdevice.epnum_max);
	for (epid = 0; epid < priv->xsdevice.epnum_max; epid++) {
		union ep_buffer_info *info;
		
		if (priv->xsdevice.epid_self == epid) {
			//$DD 自EPIDはスキップ
			seq_printf(m, "txrx buffer epid=%d skip as self\n",epid);
			continue;
		}
		
		escapse = false;
		seq_printf(m, "txrx buffer epid=%d\n",epid);
		for(i = 0;i < 2;i++){
			switch(i){
			case 0:
				info = priv->xsdevice.ep_shm_info[epid].tx.info;
				type = "tx";
				break;
			case 1:
				info = priv->xsdevice.ep_shm_info[epid].rx.info;
				type = "rx";
				//$DD Rxは共有状態時のみ
				escapse = !esmem_is_shared(priv->xsdevice.share_ptrs.share, epid);
				break;
			default:
				escapse = true;
				break;
			}
			if(escapse){
				break;
			}
			
			//$DD ログ出力
			seq_printf(m, "  %s addr=%p,version=%d,info_size=%d,buffer_size=%d,"
				"count_max=%d,frame_max=%d,MAC=%pm,tx_status=%04x,rx_status=%04x,head=%d,tail=%d",
				type,
				info,
				info->v1i.version,
				info->v1i.info_size,
				info->v1i.buffer_size,
				info->v1i.count_max,
				info->v1i.frame_max,
				info->v1i.mac_addr,
				info->v1i.tx_status,
				info->v1i.rx_status,
				info->v1i.head,
				info->v1i.tail);
			
			seq_printf(m, ",vid=");
			for(vid = 0;vid < EP_BUFFER_SUPPORT_VLAN_MAX;vid++){
				seq_printf(m, "(%d=%d)",
					vid,info->v1i.vlan_id[vid]);
				if(vid < (EP_BUFFER_SUPPORT_VLAN_MAX - 1)){
					seq_printf(m, ",");
				}
			}
			seq_printf(m, "\n");
		}
	}
	seq_printf(m, "----\n");
	
	//$DD ログバッファが確保されていたら表示処理
	if(priv->log.buffer){
		//$DD ログを表示するための、表示数、開始位置計算
		count = priv->log.count < priv->log.max ? priv->log.count : priv->log.max;
		start = priv->log.count < priv->log.max ? 0 : priv->log.count - count;
		start_index = (atomic_read(&(priv->log.index)) - count) % priv->log.max;
		
		// indexはログ出力時に上書きされることに注意(ログ出力に流用)
		//index = atomic_read(&(priv->log.index));
		//ES_DRV_LOGGING("%s log output param MSG_MAX=%d,LINESIZE=%d,count=%ld,max=%d,index counter=%d,buffer=%p,output count=%d,start=%ld,start_index=%d",
		//	__func__,ES_DRV_LOG_MSG_MAX,ES_DRV_LOG_LINESIZE,
		//	priv->log.count,priv->log.max,index,priv->log.buffer,
		//	count,start,start_index);

		//$DD 累計ログ数、現在のログ位置、ログ出力範囲などをヘッダとして表示
		seq_printf(m, "Log: count max=%lld",priv->log.count);
		seq_printf(m, ",show range=%lld-%lld(index=%d count=%d)",start,priv->log.count - 1,start_index,count);
		seq_printf(m, ",start timespec=%d(sec),%lld(ns),%lld(jiffies)",(s32)(priv->log.start_time.tv_sec),(s64)(priv->log.start_time.tv_nsec),priv->log.start_jiffiy);
		seq_printf(m, ",(%ld/%d/%d %d:%d:%d)",priv->log.start_tm.tm_year+1900,priv->log.start_tm.tm_mon+1,priv->log.start_tm.tm_mday,priv->log.start_tm.tm_hour,priv->log.start_tm.tm_min,priv->log.start_tm.tm_sec);
		seq_printf(m, "\n");

		{
			phys_addr_t buff_page_addr = virt_to_phys(priv->log.buffer_page_address);
			seq_printf(m, "alloc param : addr:%p(phys:%pa,[0]:%pa),num:%ld,buffer:%p\n",
				priv->log.buffer_page_address,
				&(buff_page_addr),
				&(priv->log.buffer_page_address[0]),
				ES_DRV_LOG_MSG_PAGE_NUM,
				priv->log.buffer);
		}

		seq_printf(m, "jiffies:index:log\n");
		seq_printf(m, "----\n");
		
		//$DD ログ出力
		for(i = 0;i < count;i++){
			index = (start_index + i) % priv->log.max;
			
			seq_printf(m, "%lld:%lld:%s\n",priv->log.log_jiffies[index],start + i,
				(char *)&(priv->log.buffer[index * ES_DRV_LOG_LINESIZE]));
			
		}
	}else{
		//$DD ログのバッファが取得できていない場合の表示
		seq_printf(m, "Log: no buffer alloc\n");
	}
	return 0;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．２３ procオープンハンドラ(info)

  single_openによる処理制御
  実態はesdrv_proc_info_show

  @param	dev		net_deviceオブジェクト
	
  @return	single_openの戻り値
**/
FUNC_STATIC int esdrv_proc_info_open(struct inode *inode, struct file *file)
{
	//$DD single_openでesdrv_proc_info_showの呼び出し
	return single_open(file, esdrv_proc_info_show, inode);
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．２４ procオープンハンドラ(status)

  single_openによる処理制御
  実態はesdrv_proc_status_show

  @param	dev		net_deviceオブジェクト
	
  @return	single_openの戻り値
**/
FUNC_STATIC int esdrv_proc_status_open(struct inode *inode, struct file *file)
{
	//$DD single_openでesdrv_proc_status_showの呼び出し
	return single_open(file, esdrv_proc_status_show, inode);
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．２５ procオープンハンドラ(stats)

  single_openによる処理制御
  実態はesdrv_proc_stats_show

  @param	dev		net_deviceオブジェクト
	
  @return	single_openの戻り値
**/
FUNC_STATIC int esdrv_proc_stats_open(struct inode *inode, struct file *file)
{
	//$DD single_openでesdrv_proc_stats_showの呼び出し
	return single_open(file, esdrv_proc_stats_show, inode);
}

// シンボル定義都合などから、関数記述位置が違うことに注意
//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．４１ procオープンハンドラ(log)

  single_openによる処理制御
  実態はesdrv_proc_log_show

  @param	dev		net_deviceオブジェクト
	
  @return	single_openの戻り値
**/
FUNC_STATIC int esdrv_proc_log_open(struct inode *inode, struct file *file)
{
	//$DD single_openでesdrv_proc_log_showの呼び出し
	return single_open(file, esdrv_proc_log_show, inode);
}


//$SD 各proc fs用ops 定義
//$SD /info用
static const struct file_operations esdrv_proc_fops_info = {
	.owner = THIS_MODULE,
	.open = esdrv_proc_info_open,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release,
};

//$SD /status用
static const struct file_operations esdrv_proc_fops_status = {
	.owner = THIS_MODULE,
	.open = esdrv_proc_status_open,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release,
};

//$SD /statistics用
static const struct file_operations esdrv_proc_fops_stats = {
	.owner = THIS_MODULE,
	.open = esdrv_proc_stats_open,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release,
};

//$SD /log用
static const struct file_operations esdrv_proc_fops_log = {
	.owner = THIS_MODULE,
	.open = esdrv_proc_log_open,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = single_release,
};


//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．２６ proc登録

  info,status,statisticsの各エントリを生成
  

  @param	dev		net_deviceオブジェクト
**/
FUNC_STATIC void esdrv_proc_create(struct net_device *dev)
{
	struct esdrv_priv *priv;

	PDEBUG(PD_ENT, "ENTER");

	priv = netdev_priv(dev);

	//$DD root登録        proc_mkdir()
	if (!proc_mkdir(ES_DRV_PROC_ROOT, NULL)) {
		PDEBUG(PD_COMMON
		       || PD_INF, "unable to create /proc/%s entry.",
		       ES_DRV_PROC_ROOT);
		return;
	} else {
		esdrv_proc_table.root = true;
	}

	//$DD /info登録       proc_create_data()
	if (!proc_create_data
	    (ES_DRV_PROC_INFO, 0, NULL, &esdrv_proc_fops_info, priv)) {
		PDEBUG(PD_COMMON
		       || PD_INF, "unable to create /proc/%s entry.",
		       ES_DRV_PROC_INFO);
	} else {
		esdrv_proc_table.info = true;
	}

	//$DD /status登録     proc_create_data()
	if (!proc_create_data
	    (ES_DRV_PROC_STATUS, 0, NULL, &esdrv_proc_fops_status, priv)) {
		PDEBUG(PD_COMMON
		       || PD_INF, "unable to create /proc/%s entry.",
		       ES_DRV_PROC_STATUS);
	} else {
		esdrv_proc_table.status = true;
	}

	//$DD /statistics登録 proc_create_data()
	if (!proc_create_data
	    (ES_DRV_PROC_STATS, 0, NULL, &esdrv_proc_fops_stats, priv)) {
		PDEBUG(PD_COMMON
		       || PD_INF, "unable to create /proc/%s entry.",
		       ES_DRV_PROC_STATS);
	} else {
		esdrv_proc_table.stats = true;
	}

	//$DD /log登録 proc_create_data()
	if (!proc_create_data
	    (ES_DRV_PROC_LOG, 0, NULL, &esdrv_proc_fops_log, priv)) {
		PDEBUG(PD_COMMON
		       || PD_INF, "unable to create /proc/%s entry.",
		       ES_DRV_PROC_LOG);
	} else {
		esdrv_proc_table.log = true;
	}

}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．２７ proc削除

  info,status,statisticsの各エントリを削除
  

**/
FUNC_STATIC void esdrv_proc_remove(void)
{

	PDEBUG(PD_ENT, "ENTER");

	//$DD /log削除 remove_proc_entry()
	if (esdrv_proc_table.log)
		remove_proc_entry(ES_DRV_PROC_LOG, NULL);
	esdrv_proc_table.log = false;

	//$DD /statistics削除 remove_proc_entry()
	if (esdrv_proc_table.stats)
		remove_proc_entry(ES_DRV_PROC_STATS, NULL);
	esdrv_proc_table.stats = false;

	//$DD /status削除     remove_proc_entry()
	if (esdrv_proc_table.status)
		remove_proc_entry(ES_DRV_PROC_STATUS, NULL);
	esdrv_proc_table.status = false;

	//$DD /info削除       remove_proc_entry()
	if (esdrv_proc_table.info)
		remove_proc_entry(ES_DRV_PROC_INFO, NULL);
	esdrv_proc_table.info = false;

	//$DD root削除        remove_proc_entry()
	if (esdrv_proc_table.root)
		remove_proc_entry(ES_DRV_PROC_ROOT, NULL);
	esdrv_proc_table.root = false;

}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．２８ IOCTL：トレース開始
	
  トレースの要求/応答はrq->ifr_dataポインタに構造体es_ioctl_trace_paramとして渡される
  copy_from_user/copy_to_userでのデータ受け渡しが必要
	
  @param[in,out] dev	net_device構造体
  @param[in,out] rq	ifreq 構造体

  @return  エラーコード
  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esdrv_ioctl_trace_start(struct net_device *dev,
					struct ifreq *rq)
{
	struct esdrv_priv *priv = netdev_priv(dev);
	struct es_ioctl_trace_param param;
	int ret = 0;
	struct es_device_trace *trace;

	PDEBUG_DEV(PD_ENT, "ENTER");

	//$DD ユーザ空間からパラメタをコピー     copy_from_user()、失敗はエラー終了(EFAULT)
	if (copy_from_user(&param, rq->ifr_data, sizeof(param)))
		return -EFAULT;

	//$DD 実行中かどうかチェック
	if (priv->xsdevice.trace_start) {
		//$DD すでに実施中なので、エラーコード領域に「0x0200 : トレース開始済み」を設定
		param.res.start.err_code =
		    ES_IOCTL_TRACE_START_ERR_ALREADY_STARTED;
		//$DD 終了処理へ
		goto out;
	}
	//$DD パラメタのチェック
	if (!((ES_DEV_COMMAND_START_TRACE_REQ_LEN(param.req.start.trace_size) <= priv->xsdevice.share_ptrs.req_buf_size)	// 要求バッファ最大以下
	      && (0 == (param.req.start.trace_size % MINIMUM_PAGE_SIZE))	/* 4096の倍数 */
	    )) {
		//$DD 4G以下、かつ4Kの倍数を満たさないので、エラーコード領域に「0x0100 : パラメータ異常」を設定
		param.res.start.err_code = ES_IOCTL_TRACE_START_ERR_PARAM;
		//$DD 終了処理へ
		goto out;
	}
	//$DD トレースのためのメモリ確保   vmalloc()
	priv->xsdevice.share_ptrs.trace = vmalloc(param.req.start.trace_size);
	if (NULL == priv->xsdevice.share_ptrs.trace) {
		//$DD メモリ確保できなかったので、エラーコード領域に「0x0400 : メモリ不足」を設定
		param.res.start.err_code = ES_IOCTL_TRACE_START_ERR_MEMORY;
		//$DD 終了処理へ
		goto out;
	}
	//$DD パラメタを保持するため、保存
	priv->xsdevice.share_ptrs.trace_size = param.req.start.trace_size;
	priv->xsdevice.trace_mode = param.req.start.mode;

	//$DD トレース開始        esvmm_start_trace()
	mutex_lock(&priv->xsdevice.share_ptrs.lock);
	ret =
	    esvmm_start_trace(priv->xsdevice.base, param.req.start.mode,
			      &(priv->xsdevice.share_ptrs));
	mutex_unlock(&priv->xsdevice.share_ptrs.lock);
	// ioctl時なので、リカバリなし

	//$DD ドライバ固有統計(command_start_trace_executed)更新
	priv->xsdevice.drv_stats.command_start_trace_executed++;

	//$DD 実行結果を確認
	switch (ret) {
	case 0:
		//$DD 0は正常、トレースの実行状態をtrueへ
		param.res.start.err_code = ES_IOCTL_TRACE_START_ERR_NORMAL;
		priv->xsdevice.trace_start = true;
		break;
	case -ENOMSG:
	case -EBUSY:
		//$DD EBUSYの場合ビジーで実行できなかったので、エラーコード領域に「0x0001 : 処理中（BUSY）」を設定、ENOMSGもこのエラーとする
		param.res.start.err_code = ES_IOCTL_TRACE_START_ERR_BUSY;
		break;
	default:
		//$DD それ以外の場合で実行できなかった、エラーコード領域にエラーコード格納
		param.res.start.err_code =
		    priv->xsdevice.share_ptrs.res_buf->start_trace.code;
		break;
	}

	//$DD 正常に開始できなかった場合、バッファを解放し、各変数をクリア
	if (!(priv->xsdevice.trace_start)) {
		if (NULL != priv->xsdevice.share_ptrs.trace) {
			vfree(priv->xsdevice.share_ptrs.trace);
			priv->xsdevice.share_ptrs.trace = NULL;
		}
		priv->xsdevice.share_ptrs.trace_size = 0;
		priv->xsdevice.trace_mode = 0;
	}
	//$DD 終了処理
 out:
	//$DD ユーザ空間へパラメタをコピー        copy_to_user()
	//$DD copy_to_userがエラーだった場合、戻り値EFAULTを返す
	if (copy_to_user(rq->ifr_data, &param, sizeof(param)))
		return -EFAULT;

	return 0;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．２９ IOCTL：トレース終了
	
  トレースの要求/応答はrq->ifr_dataポインタに構造体es_ioctl_trace_paramとして渡される
  copy_from_user/copy_to_userでのデータ受け渡しが必要
	
  @param[in,out] dev	net_device構造体
  @param[in,out] rq	ifreq 構造体

  @return  エラーコード
  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esdrv_ioctl_trace_stop(struct net_device *dev, struct ifreq *rq)
{
	struct esdrv_priv *priv = netdev_priv(dev);
	struct es_ioctl_trace_param param;
	int ret = 0;
	bool trace_done = false;
	u64 min_buffer_size;
	struct es_device_trace *trace;

	PDEBUG_DEV(PD_ENT, "ENTER");

	//$DD ユーザ空間からパラメタをコピー     copy_from_user()、失敗はエラー終了(EFAULT)
	if (copy_from_user(&param, rq->ifr_data, sizeof(param)))
		return -EFAULT;

	if (!(priv->xsdevice.trace_start)) {
		//$DD トレース開始していないなら、エラーコード領域に「0x0300 : トレース開始前」を設定
		param.res.stop.err_code = ES_IOCTL_TRACE_STOP_ERR_BEFORE_START;
		//$DD 終了処理へ
		goto out;
	}
	//$DD トレース停止        esvmm_stop_trace()
	mutex_lock(&priv->xsdevice.share_ptrs.lock);
	ret =
	    esvmm_stop_trace(priv->xsdevice.base, &(priv->xsdevice.share_ptrs));
	mutex_unlock(&priv->xsdevice.share_ptrs.lock);
	// ioctl時なので、リカバリなし

	//$DD ドライバ固有統計(command_stop_trace_executed)更新
	priv->xsdevice.drv_stats.command_stop_trace_executed++;

	//$DD 実行結果を確認
	switch (ret) {
	case 0:
		//$DD 0は正常、トレースの実行状態をfalseへ
		param.res.stop.err_code = ES_IOCTL_TRACE_STOP_ERR_NORMAL;
		priv->xsdevice.trace_start = false;
		trace_done = true;
		break;
	case -ENOMSG:
	case -EBUSY:
		//$DD EBUSYの場合ビジーで実行できなかったので、エラーコード領域に「0x0001 : 処理中（BUSY）」を設定、ENOMSGもこのエラーとする
		param.res.stop.err_code = ES_IOCTL_TRACE_STOP_ERR_BUSY;
		break;
	default:
		//$DD それ以外の場合で実行できなかった、エラーコード領域にエラーコード格納
		param.res.stop.err_code =
		    priv->xsdevice.share_ptrs.res_buf->stop_trace.code;
		break;
	}

	//$DD 終了処理
 out:
	//$DD ユーザ空間へパラメタをコピー        copy_to_user()
	//$DD copy_to_userがエラーだった場合、戻り値EFAULTを返す
	if (copy_to_user(rq->ifr_data, &param, sizeof(param)))
		return -EFAULT;

	//$DD ただし、トレース終了かつ正常においては、トレースデータも格納する

	//$DD 先頭から格納できるだけ格納する
	if (trace_done) {
		u32 stop_trace_buff_size = param.req.stop.trace_buff_size;
		u32 start_trace_buff_size =
		    priv->xsdevice.share_ptrs.trace_size;

		//$DD トレース結果の格納サイズはioctlのtrace_buff_sizeか
		//$DD 現在のトレースバッファのサイズの小さい方
		trace = priv->xsdevice.share_ptrs.trace;
		min_buffer_size =
		    min(stop_trace_buff_size, start_trace_buff_size);

		//$DD トレース結果分のサイズを格納して、再度ユーザ空間へパラメタをコピー       copy_to_user()
		param.res.stop.written_size = min_buffer_size;
		ret = copy_to_user(rq->ifr_data,
				   &param,
				   sizeof(union es_ioctl_trace_req_val) +
				   sizeof(struct es_ioctl_trace_stop_res_val));

		//$DD その後ろに格納できるだけ保存        copy_to_user()
		ret = copy_to_user((rq->ifr_data
				    + sizeof(union es_ioctl_trace_req_val)
				    +
				    sizeof(struct es_ioctl_trace_stop_res_val)),
				   trace, min_buffer_size);

		//$DD トレースコピー完了後、メモリ解放と値の初期化を実施
		if (NULL != priv->xsdevice.share_ptrs.trace) {
			vfree(priv->xsdevice.share_ptrs.trace);
			priv->xsdevice.share_ptrs.trace = NULL;
		}
		priv->xsdevice.share_ptrs.trace_size = 0;
		priv->xsdevice.trace_mode = 0;
	}

	return 0;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．３０ IOCTL：トレース情報取得
	
  トレースの要求/応答はrq->ifr_dataポインタに構造体es_ioctl_trace_paramとして渡される
  copy_from_user/copy_to_userでのデータ受け渡しが必要
	
  @param[in,out] dev	net_device構造体
  @param[in,out] rq	ifreq 構造体

  @return  エラーコード
  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esdrv_ioctl_trace_getcfg(struct net_device *dev,
					 struct ifreq *rq)
{
	struct esdrv_priv *priv = netdev_priv(dev);
	struct es_ioctl_trace_param param;
	int ret = 0;

	PDEBUG_DEV(PD_ENT, "ENTER");

	//$DD ユーザ空間からパラメタをコピー     copy_from_user()、失敗はエラー終了(EFAULT)
	if (copy_from_user(&param, rq->ifr_data, sizeof(param)))
		return -EFAULT;

	if (priv->xsdevice.trace_start) {
		//$DD トレース開始していたら保持しているパラメタを戻す
		param.res.getcfg.err_code = ES_IOCTL_TRACE_GETCFG_ERR_NORMAL;
		param.res.getcfg.mode = priv->xsdevice.trace_mode;
		param.res.getcfg.trace_size =
		    priv->xsdevice.share_ptrs.trace_size;
	} else {
		//$DD トレース開始していないなら、エラーコード領域に「0x0300 : トレース開始前」を設定
		param.res.getcfg.err_code =
		    ES_IOCTL_TRACE_GETCFG_ERR_BEFORE_START;
	}

	//$DD ユーザ空間へパラメタをコピー        copy_to_user()
	//$DD copy_to_userがエラーだった場合、戻り値EFAULTを返す
	if (copy_to_user(rq->ifr_data, &param, sizeof(param)))
		return -EFAULT;

	return 0;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．３１ IOCTL：レジスタ読み出し
	
  レジスタIOの要求/応答はrq->ifr_dataポインタに構造体es_ioctl_esdev_reg_paramとして渡される
  copy_from_user/copy_to_userでのデータ受け渡しが必要
	
  @param[in,out] dev	net_device構造体
  @param[in,out] rq	ifreq 構造体

  @return  エラーコード
  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esdrv_ioctl_reg_read(struct net_device *dev, struct ifreq *rq)
{
	struct esdrv_priv *priv = netdev_priv(dev);
	struct es_ioctl_esdev_reg_param reg;

	PDEBUG_DEV(PD_ENT, "ENTER");

	//$DD ユーザ空間からパラメタをコピー     copy_from_user()、失敗はエラー終了(EFAULT)
	if (copy_from_user(&reg, rq->ifr_data, sizeof(reg)))
		return -EFAULT;

	//$DD レジスタが獲得できていれば処理を行う
	if (priv->xsdevice.base) {
		//$DD 値の読み出し
		reg.val =
		    esvmm_readl((void *)(priv->xsdevice.base +
					 reg.offset_addr));
	} else {
		//$DD レジスタ取得できていないなら戻り値EFAULTで終了
		return -EFAULT;
	}

	//$DD ユーザ空間へパラメタをコピー        copy_to_user()
	//$DD copy_to_userがエラーだった場合、戻り値EFAULT
	if (copy_to_user(rq->ifr_data, &reg, sizeof(reg)))
		return -EFAULT;

	return 0;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．３２ IOCTL：レジスタ書き込み
	
  レジスタIOの要求/応答はrq->ifr_dataポインタに構造体es_ioctl_esdev_reg_paramとして渡される
  copy_from_user/copy_to_userでのデータ受け渡しが必要
	
  @param[in,out] dev	net_device構造体
  @param[in,out] rq	ifreq 構造体

  @return  エラーコード
  @retval  0		正常終了
  @retval  負の値	<linuix/errno.h>で定義されるエラーコード
**/
FUNC_STATIC int esdrv_ioctl_reg_write(struct net_device *dev, struct ifreq *rq)
{
	struct esdrv_priv *priv = netdev_priv(dev);
	struct es_ioctl_esdev_reg_param reg;

	PDEBUG_DEV(PD_ENT, "ENTER");

	//$DD ユーザ空間からパラメタをコピー     copy_from_user()、失敗はエラー終了(EFAULT)
	if (copy_from_user(&reg, rq->ifr_data, sizeof(reg)))
		return -EFAULT;

	//$DD レジスタが獲得できていれば処理を行う
	if (priv->xsdevice.base) {
		//$DD 値の書き込み
		esvmm_writel(reg.val,
			     ((void *)(priv->xsdevice.base + reg.offset_addr)));
	} else {
		//$DD レジスタ取得できていないなら戻り値EFAULTで終了
		return -EFAULT;
	}

	//$DD ユーザ空間へパラメタをコピー        copy_to_user()
	//$DD copy_to_userがエラーだった場合、戻り値EFAULT
	if (copy_to_user(rq->ifr_data, &reg, sizeof(reg)))
		return -EFAULT;

	return 0;
}

#if !defined(ESDRV_ETHERTOOL_OFF)
//$DD-----------------------------------------------------------------------------
//$DD ethtoolサポート

/* 参考 e1000e\ethtool.c xen-netfront.c */

//$SD Ethertoolの統計情報構造体
struct esdrv_stats {
	char stat_string[ETH_GSTRING_LEN];
	int sizeof_stat;	//将来64bit以外のサイズが混在するようになったときの為に残しておく
	int stat_offset;
};

#define ESDRV_STAT(m) sizeof(((struct esdrv_priv *)0)->m), offsetof(struct esdrv_priv, m)

//$SD Ethertool用の統計情報参照配列
static const struct esdrv_stats esdrv_gstrings_stats[] = {
	//基本統計情報(priv->stats64)より使用している変数のみ
	{"rx_packets", ESDRV_STAT(stats64.rx_packets)},
	{"tx_packets", ESDRV_STAT(stats64.tx_packets)},
	{"rx_bytes", ESDRV_STAT(stats64.rx_bytes)},
	{"tx_bytes", ESDRV_STAT(stats64.tx_bytes)},
	{"rx_dropped", ESDRV_STAT(stats64.rx_dropped)},
	{"tx_dropped", ESDRV_STAT(stats64.tx_dropped)},
	//デバイス独自の統計情報よりNAPI関連
	{"poll_schedule", ESDRV_STAT(xsdevice.drv_stats.napi_scheduled)},
	{"poll_call", ESDRV_STAT(xsdevice.drv_stats.napi_polled)},
	{"poll_maxpackets",
	 ESDRV_STAT(xsdevice.drv_stats.napi_poll_maxpackets)},
};

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．３３ ethertool：統計取得
	
  net_device.ethtool_ops.get_ethtool_statsに本ハンドラ関数のポインタを登録する
	
  @param[in,out] dev	net_device構造体
  @param[in,out] stats	ethtool_stats 構造体(未使用)
  @param[in,out] data	補助データ

**/
FUNC_STATIC void ethtool_get_ethtool_stats(struct net_device *netdev,
					   struct ethtool_stats __always_unused
					   * stats, u64 * data)
{
	struct esdrv_priv *priv = netdev_priv(netdev);
	int i;
	char *p = NULL;

	for (i = 0; i < ARRAY_SIZE(esdrv_gstrings_stats); i++) {
		p = (char *)priv + esdrv_gstrings_stats[i].stat_offset;
		data[i] =
		    (esdrv_gstrings_stats[i].sizeof_stat ==
		     sizeof(u64)) ? *(u64 *) p : *(u32 *) p;
	}
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．３４ ethertool：文字列取得
	
  net_device.ethtool_ops.get_stringsに本ハンドラ関数のポインタを登録する
	
  @param[in,out] dev	net_device構造体(未使用)
  @param[in,out] stringset	文字列種別
  @param[in,out] data	補助データ

**/
FUNC_STATIC void ethtool_get_strings(struct net_device __always_unused * netdev,
				     u32 stringset, u8 * data)
{
	u8 *p = data;
	int i;

	switch (stringset) {
	case ETH_SS_STATS:
		for (i = 0; i < ARRAY_SIZE(esdrv_gstrings_stats); i++) {
			memcpy(p, esdrv_gstrings_stats[i].stat_string,
			       ETH_GSTRING_LEN);
			p += ETH_GSTRING_LEN;
		}
		break;
	}
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．３５ ethertool：種別数
	
  net_device.ethtool_ops.get_sset_countに本ハンドラ関数のポインタを登録する
	
  @param[in,out] dev	net_device構造体(未使用)
  @param[in,out] sset	種別タイプ

**/
FUNC_STATIC int ethtool_get_sset_count(struct net_device __always_unused *
				       netdev, int sset)
{
	switch (sset) {
	case ETH_SS_STATS:
		return ARRAY_SIZE(esdrv_gstrings_stats);
	default:
		return -EOPNOTSUPP;
	}
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．３６ ethertool：ドライバ情報
	
  net_device.ethtool_ops.get_drvinfoに本ハンドラ関数のポインタを登録する
	
  @param[in,out] dev	net_device構造体
  @param[in,out] drvinfo	ethtool_drvinfo構造体

**/
FUNC_STATIC void ethtool_get_drvinfo(struct net_device *netdev,
				     struct ethtool_drvinfo *drvinfo)
{
	strlcpy(drvinfo->driver, ES_DRV_MODULE_NAME, sizeof(drvinfo->driver));
	strlcpy(drvinfo->version, ES_DRV_VERSION_STRING,
		sizeof(drvinfo->version));

	strlcpy(drvinfo->fw_version, "none", sizeof(drvinfo->fw_version));
	strlcpy(drvinfo->bus_info, "none", sizeof(drvinfo->bus_info));
	drvinfo->regdump_len = 0;
	drvinfo->eedump_len = 0;
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．３７ ethertool：設定情報
	
  net_device.ethtool_ops.get_settingsに本ハンドラ関数のポインタを登録する
	
  @param[in,out] dev	net_device構造体
  @param[in,out] ecmd	ethtool_cmd構造体

**/
FUNC_STATIC int ethtool_get_settings(struct net_device *netdev,
				     struct ethtool_cmd *ecmd)
{
	ecmd->supported = 0;
	ecmd->advertising = 0;
	ecmd->duplex = DUPLEX_FULL;
	ecmd->autoneg = AUTONEG_DISABLE;
	ecmd->transceiver = XCVR_DUMMY1;
	ecmd->port = PORT_NONE;
	ethtool_cmd_speed_set(ecmd, 20000);	//20Gb/s
	return 0;
}

//$SD Ethertoolハンドラ関数テーブル定数
// esdrv_netdev_setup()で参照される定数 ethtoolの設定 []はethtoolのオプションパラメータ
const struct ethtool_ops esdrv_ethtool_ops = {
	.get_strings = &ethtool_get_strings,
	.get_sset_count = &ethtool_get_sset_count,
	.get_ethtool_stats = &ethtool_get_ethtool_stats,	//[-S|--statistics]     統計情報
	.get_drvinfo = &ethtool_get_drvinfo,	//[-i|--driver]         ドライバ情報
	.get_settings = &ethtool_get_settings,	//[オプション無し]

	//.set_settings         //[-s|--change]
	//.get_regs_len         //[-d|--register-dump]
	//.get_regs             //[-d|--register-dump]  レジスタ表示
	//.get_wol              //[オプション無し]               WOL未対応
	//.set_wol              //[-s|--change wol]     WOL未対応
	//.get_msglevel         //[オプション無し]
	//.set_msglevel         //[-s|--change msglvl]
	//.nway_reset           //[-r|--negotiate]
	//.get_link             //[オプション無し]
	//.get_eeprom_len       //[-e|--eeprom-dump/-E|--change-eeprom] EEPROM無し
	//.get_eeprom           //[-e|--eeprom-dump]    EEPROM無し
	//.set_eeprom           //[-E|--change-eeprom]  EEPROM無し
	//.get_coalesce         //[-c|--show-coalesce]  Interrupt Coalescing(割り込み間引き) パラメータ表示
	//.set_coalesce         //[-C|--coalesce]       Interrupt Coalescing(割り込み間引き) パラメータ設定
	//.get_ringparam        //[-g|--show-ring]
	//.set_ringparam        //[-G|--set-ring]
	//.get_pauseparam       //[-a|--show-pause]
	//.set_pauseparam       //[-A|--pause]
	//.self_test            //[-t|--test]
	//.set_phys_id          //[-p|--identify]       ID(対応デバイスのLED表示等)
	//.begin
	//.complete
	//.get_priv_flags       //[--show-priv-flags]
	//.set_priv_flags       //[--set-priv-flags]
	//.get_rxnfc            //[-n|-u|--show-nfc|--show-ntuple]
	//.set_rxnfc            //[-N|-U|--config-nfc|--config-ntuple]
	//.flash_device         //[-f|--flash]  flash-memory無し
	//.reset                //ethtoolでは未対応？
	//.get_rxfh_indir_size  //[-x|--show-rxfh-indir/-X|--set-rxfh-indir]
	//.get_rxfh_indir       //[-x|--show-rxfh-indir]        ★linux-3.16では変更有り
	//.set_rxfh_indir       //[-X|--set-rxfh-indir]         ★linux-3.16では変更有り
	//.get_channels         //[-l|--show-channels]
	//.set_channels         //[-L|--set-channels]
	//.get_dump_flag        //[-w|--get-dump] ファームウェアダンプ
	//.get_dump_data        //[-w|--get-dump] ファームウェアダンプ
	//.set_dump             //[-W|--set-dump] ファームウェアダンプ
	//.get_ts_info          //[-T|--show-time-stamping]     PTP(Precision Time Protocol)サポート　デフォルトのまま
	//.get_module_info      //[-m|--dump-module-eeprom]     EEPROM無し
	//.get_module_eeprom    //[-m|--dump-module-eeprom]     EEPROM無し
	//.get_eee              //[--show-eee]  EEE(Energy Efficient Ethernet)未対応
	//.set_eee              //[--set-eee]   EEE(Energy Efficient Ethernet)未対応
};
#endif				/* !defined(ESDRV_ETHERTOOL_OFF) */

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．３８ 割り込みの遅延監視ワークキュー
	
  @param[int]     work	ワークキューエントリの構造体
**/
FUNC_STATIC void esdrv_irq_status_work(struct work_struct *work)
{
	struct esdrv_priv *priv =
	    container_of(to_delayed_work(work), struct esdrv_priv,
			 interrupt_watch_work);
	intreg_t intr_cause;	//割り込み原因

	PDEBUG_DEV(PD_ENT, "ENTER");

	//$DD 割り込みを抑制してから割り込みハンドラを呼出す
	local_irq_disable();
	esif_vmm_irq(priv->xsdevice.config->irq,
		     (memptr_t) (priv->xsdevice.container_dev));
	local_irq_enable();

	//$DD NAPI監視のための補助処理(受信フレームの有無チェックによる起動)
	//$DD 残りフレーム数のチェック      esif_rxframe_search_exist()
	if (0 <= esif_rxframe_search_exist(priv, 0)) {
		//$DD 受信ポーリング esdrv_rx_napi_poll() をスケジュールする    napi_schedule()
		napi_schedule(&priv->napi);

		//$DD ドライバ固有統計(napi_scheduled:全体統計のみ)更新
		priv->xsdevice.drv_stats.napi_scheduled++;
	}

	//$DD 停止になっていなければ、割り込みの遅延監視ワークキューを再登録
	if (priv->interrupt_watch_enable) {
		if (!delayed_work_pending(&priv->interrupt_watch_work)) {
			queue_delayed_work(priv->control_wq,
					   &priv->interrupt_watch_work,
					   ES_DRV_IRQ_WATCH_DELAY);
		}
	}
}

//$DD-----------------------------------------------------------------------------
/**$DD
  ２．２．３９ 強制クローズ処理

  ワークキューに登録する、強制クローズ処理
  システムのワークキューに登録すること(デバイスのキューが使えないため)
	
  @param[in,out] work	ワークキューエントリの構造体
**/
FUNC_STATIC void esdrv_force_close_work(struct work_struct *work)
{
	struct esdrv_priv *priv =
	    container_of(work, struct esdrv_priv, force_close);

	PDEBUG(PD_COMMON, "ENTER");

	//$DD 強制クローズ処理(ES_DRV_INTERNAL_ERR_FORCE_CLOSE = 21)
	//pr_err(ES_DRV_MSG_INTERNAL_ERR1, ES_DRV_MSG_TIME, ES_DRV_INTERNAL_ERR_FORCE_CLOSE);
	ES_DRV_LOGGING("%s Intarnal=%d",
		__func__, ES_DRV_INTERNAL_ERR_FORCE_CLOSE);

	rtnl_lock();
	dev_close(priv->xsdevice.container_dev);
	rtnl_unlock();

	PDEBUG(PD_COMMON, "DONE");
}


//↑関数
//-----------------------------------------------------------------------------
// モジュール情報

//モジュールエントリ関数の割り付け
module_init(esdrv_module_init);
module_exit(esdrv_module_exit);

MODULE_AUTHOR(ES_DRV_MODULE_AUTHOR);
MODULE_DESCRIPTION(ES_DRV_MODULE_DESC);
MODULE_LICENSE("GPL");
MODULE_VERSION(ES_DRV_VERSION_STRING);
